@@ -1389,7 +1389,7 @@ gl_locale_name_default (void)
 	switch (sub)
 	  {
 	  case SUBLANG_SINDHI_PAKISTAN: return "sd_PK";
-	  case SUBLANG_SINDHI_AFGHANISTAN: return "sd_AF";
+	  // case SUBLANG_SINDHI_AFGHANISTAN: return "sd_AF";
 	  }
 	return "sd";
       case LANG_SINHALESE: return "si_LK";
