@@ -3,7 +3,7 @@
  *  file extra.c
  *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *  Copyright (C) 2004	      The R Foundation
- *  Copyright (C) 2005--2019  The R Core Team
+ *  Copyright (C) 2005--2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,22 +29,20 @@
 
 #include "win-nls.h"
 
-
+#include <float.h>
 #include <stdio.h>
+#include <string.h>
 #include <time.h>
 #include "Defn.h"
 #include <Internal.h>
 #include "Fileio.h"
 #include <direct.h>
 #include "graphapp/ga.h"
-/* Mingw-w64 defines this to be 0x0502 */
-#ifndef _WIN32_WINNT
-# define _WIN32_WINNT 0x0502 /* for GetLongPathName, KEY_WOW64_64KEY */
-#endif
+#include "rlocale.h"
 #include <windows.h>
 #include "rui.h"
 #undef ERROR
-#include <R_ext/RS.h> /* for Calloc */
+#include <R_ext/RS.h> /* formerly for Calloc */
 
 #include <winbase.h>
 
@@ -59,7 +57,8 @@ void internal_shellexec(const char * file)
     home = getenv("R_HOME");
     if (home == NULL)
 	error(_("R_HOME not set"));
-    strncpy(home2, home, 10000);
+    strncpy(home2, home, 10000 - 1);
+    home2[10000 - 1] = '\0';
     for(p = home2; *p; p++) if(*p == '/') *p = '\\';
     ret = (uintptr_t) ShellExecute(NULL, "open", file, NULL, home2, SW_SHOW);
     if(ret <= 32) { /* an error condition */
@@ -118,19 +117,23 @@ SEXP do_shellexec(SEXP call, SEXP op, SEXP args, SEXP env)
     return R_NilValue;
 }
 
-int check_doc_file(const char * file)
+int check_doc_file(const char *file)
 {
     const char *home;
-    char path[MAX_PATH];
+    char *path;
 
     home = getenv("R_HOME");
     if (home == NULL)
 	error(_("R_HOME not set"));
-    if(strlen(home) + strlen(file) + 1 >= MAX_PATH) return(1); /* cannot exist */
+    path = (char *) malloc(strlen(home) + 1 + strlen(file) + 1);
+    if (!path)
+	return 0; /* treat error as no access, used in GUI */
     strcpy(path, home);
     strcat(path, "/");
     strcat(path, file);
-    return access(path, 4) == 0; /* read access */
+    int res = (access(path, 4) == 0); /* read access is granted */
+    free(path);
+    return res;
 }
 
 #include "Startup.h"
@@ -140,9 +143,11 @@ void Rwin_fpset(void)
     /* Under recent MinGW this is what fpreset does.  It sets the
        control word to 0x37f which corresponds to 0x8001F as used by
        _controlfp.  That is all errors are masked, 64-bit mantissa and
-       rounding are selected. */
+       rounding are selected:
 
-    __asm__ ( "fninit" ) ;
+       __asm__ ( "fninit" ) ;
+    */
+    _fpreset();
 }
 
 
@@ -158,7 +163,7 @@ SEXP in_loadRconsole(SEXP sfile)
 	error(_("invalid '%s' argument"), "file");
     getActive(&gui);  /* Will get defaults if there's no active console */
     if (loadRconsole(&gui, translateChar(STRING_ELT(sfile, 0)))) applyGUI(&gui);
-    if (strlen(gui.warning)) warning(gui.warning);
+    if (strlen(gui.warning)) warning("%s", gui.warning);
     vmaxset(vmax);
     return R_NilValue;
 }
@@ -228,6 +233,8 @@ SEXP do_sysinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(NULL != pGNSI) pGNSI(&si); else GetSystemInfo(&si);
 	if(si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64)
 	    strcat(ver, " x64");
+	else if(si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_ARM64)
+	    strcat(ver, " arm64");
     }
     SET_STRING_ELT(ans, 1, mkChar(ver));
 
@@ -244,7 +251,9 @@ SEXP do_sysinfo(SEXP call, SEXP op, SEXP args, SEXP rho)
     GetComputerNameW(name, &namelen);
     wcstoutf8(buf, name, sizeof(buf));
     SET_STRING_ELT(ans, 3, mkCharCE(buf, CE_UTF8));
-#ifdef _WIN64
+#ifdef __aarch64__
+    SET_STRING_ELT(ans, 4, mkChar("aarch64"));
+#elif defined(_WIN64)
     SET_STRING_ELT(ans, 4, mkChar("x86-64"));
 #else
     SET_STRING_ELT(ans, 4, mkChar("x86"));
@@ -281,60 +290,6 @@ void Rsleep(double timeint)
 
 }
 
-
-#define MALLINFO_FIELD_TYPE size_t
-struct mallinfo {
-    MALLINFO_FIELD_TYPE arena;    /* non-mmapped space allocated from system */
-    MALLINFO_FIELD_TYPE ordblks;  /* number of free chunks */
-    MALLINFO_FIELD_TYPE smblks;   /* number of fastbin blocks */
-    MALLINFO_FIELD_TYPE hblks;    /* number of mmapped regions */
-    MALLINFO_FIELD_TYPE hblkhd;   /* space in mmapped regions */
-    MALLINFO_FIELD_TYPE usmblks;  /* maximum total allocated space */
-    MALLINFO_FIELD_TYPE fsmblks;  /* space available in freed fastbin blocks */
-    MALLINFO_FIELD_TYPE uordblks; /* total allocated space */
-    MALLINFO_FIELD_TYPE fordblks; /* total free space */
-    MALLINFO_FIELD_TYPE keepcost; /* top-most, releasable (via malloc_trim) space */
-};
-extern R_size_t R_max_memory;
-
-struct mallinfo mallinfo(void);
-
-SEXP in_memsize(SEXP ssize)
-{
-    SEXP ans;
-    int maxmem = NA_LOGICAL;
-
-    if(isLogical(ssize)) 
-	maxmem = asLogical(ssize);
-    else if(isReal(ssize)) {
-	R_size_t newmax;
-	double mem = asReal(ssize);
-	if (!R_FINITE(mem))
-	    error(_("incorrect argument"));
-#ifndef _WIN64
-	if(mem >= 4096)
-	    error(_("don't be silly!: your machine has a 4Gb address limit"));
-#endif
-	newmax = mem * 1048576.0;
-	if (newmax < R_max_memory)
-	    warning(_("cannot decrease memory limit: ignored"));
-	else
-	    R_max_memory = newmax;
-    } else
-	error(_("incorrect argument"));
-	
-    PROTECT(ans = allocVector(REALSXP, 1));
-    if(maxmem == NA_LOGICAL)
-	REAL(ans)[0] = R_max_memory;
-    else if(maxmem)
-	REAL(ans)[0] = mallinfo().usmblks;
-    else
-	REAL(ans)[0] = mallinfo().uordblks;
-    REAL(ans)[0] /= 1048576.0;
-    UNPROTECT(1);
-    return ans;
-}
-
 SEXP do_dllversion(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP path = R_NilValue, ans;
@@ -434,15 +389,290 @@ const char *formatError(DWORD res)
     return buf;
 }
 
+#if _WIN32_WINNT < 0x0602
+/* These constants were added to FILE_INFO_BY_HANDLE_CLASS in Windows 8 */
+enum {
+  FileStorageInfo = FileFullDirectoryRestartInfo + 1,
+  FileAlignmentInfo,
+  FileIdInfo,
+  FileIdExtdDirectoryInfo,
+  FileIdExtdDirectoryRestartInfo
+};
+#endif
+
+#if _WIN32_WINNT < 0x602 || !defined(__MINGW32__)
+/* Available in Windows Server 2012, but also in MinGW from Windows 8.  */
+typedef struct _FILE_ID_INFO {
+  ULONGLONG   VolumeSerialNumber;
+  FILE_ID_128 FileId;
+} FILE_ID_INFO, *PFILE_ID_INFO;
+#endif
+
+typedef BOOL (WINAPI *LPFN_GFIBH_EX) (HANDLE, FILE_INFO_BY_HANDLE_CLASS,
+                                      LPVOID, DWORD);
+
+static int isSameFile(HANDLE a, HANDLE b)
+{
+    FILE_ID_INFO aid, bid;
+
+    memset(&aid, 0, sizeof(FILE_ID_INFO));
+    memset(&bid, 0, sizeof(FILE_ID_INFO));
+    if (!GetFileInformationByHandleEx(a, FileIdInfo, &aid, sizeof(FILE_ID_INFO)) ||
+        !GetFileInformationByHandleEx(b, FileIdInfo, &bid, sizeof(FILE_ID_INFO)))
+	/* on Vista and Win7 it is expected to fail because FileIdInfo
+	   is not supported */
+	return -1;
+
+    if (aid.VolumeSerialNumber == bid.VolumeSerialNumber &&
+	!memcmp(&aid.FileId, &bid.FileId, sizeof(FILE_ID_128)))
+
+	return 1;
+    else
+	return 0;
+}
+
+/* returns R_alloc'd result */
+static char *getFinalPathName(const char *orig)
+{
+    HANDLE horig, hres;
+    int ret, ret1;
+
+    /* FILE_FLAG_BACKUP_SEMANTICS needed to open a directory */
+    horig = CreateFile(orig, 0,
+                       FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                       NULL, OPEN_EXISTING,
+	               FILE_ATTRIBUTE_HIDDEN | FILE_FLAG_BACKUP_SEMANTICS,
+                       NULL);
+    if (horig == INVALID_HANDLE_VALUE) 
+	return NULL;
+
+    ret = GetFinalPathNameByHandle(horig, NULL, 0, VOLUME_NAME_DOS);
+    if (ret <= 0) {
+	CloseHandle(horig);
+	return NULL;
+    }
+    /* while the documentation says that "ret" shall include the size needed
+       including the terminator, apparently it does not include the terminator.
+       (seen on Windows 10, build 19045), so increase the size. */
+    ret++;
+
+    char *res = R_alloc(ret, 1);
+    ret1 = GetFinalPathNameByHandle(horig, res, ret, VOLUME_NAME_DOS);
+    if (ret1 <= 0 || ret1 >= ret) {
+	CloseHandle(horig);
+	return NULL;
+    }
+    
+    /* get rid of the \\?\ prefix */
+    int len = ret;
+    int strip = 0;
+    if (len < 4 || strncmp("\\\\?\\", res, 4)) {
+	/* res should start with \\?\ */
+	CloseHandle(horig);
+	return NULL;
+    }
+    
+    if (len > 8 && !strncmp("UNC\\", res+4, 4)) {
+	/* UNC path \\?\UNC */
+	res[6] = '\\'; /* replace the "C" in "UNC" to get "\\" prefix */
+	strip = 6;
+    } else if (len >= 6 && isalpha(res[4]) && res[5] == ':' && res[6] == '\\')
+	/* \\?\D: */
+	strip = 4;
+    else {
+	CloseHandle(horig);
+	return NULL;
+    }
+    memmove(res, res+strip, len-strip+1);
+
+    /* sanity check if the file exists using the normalized path, a normalized
+       path to an existing file should still be working */
+    /* FILE_FLAG_BACKUP_SEMANTICS needed to open a directory */
+    hres = CreateFile(res, 0,
+                      FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                      NULL, OPEN_EXISTING,
+                      FILE_ATTRIBUTE_HIDDEN | FILE_FLAG_BACKUP_SEMANTICS,
+                      NULL);
+    if (hres == INVALID_HANDLE_VALUE) {
+	CloseHandle(horig);
+	return NULL;
+    }
+
+    /* check that the handles point to the same file, which may not be
+       always the case because of silent best-fit encoding conversion
+       done by Windows */
+    ret = isSameFile(horig, hres);
+    CloseHandle(horig);
+    CloseHandle(hres);
+
+    return (ret == 1) ? res : NULL;
+}
+
+/* returns R_alloc'd result */
+static wchar_t *getFinalPathNameW(const wchar_t *orig)
+{
+    HANDLE horig, hres;
+    int ret, ret1;
+
+    /* FILE_FLAG_BACKUP_SEMANTICS needed to open a directory */
+    horig = CreateFileW(orig, 0,
+                        FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                        NULL, OPEN_EXISTING,
+                        FILE_ATTRIBUTE_HIDDEN | FILE_FLAG_BACKUP_SEMANTICS,
+                        NULL);
+    if (horig == INVALID_HANDLE_VALUE) 
+	return NULL;
+
+    ret = GetFinalPathNameByHandleW(horig, NULL, 0, VOLUME_NAME_DOS);
+    if (ret <= 0) {
+	CloseHandle(horig);
+	return NULL;
+    }
+
+    wchar_t *wres = (wchar_t *)R_alloc(ret, sizeof(wchar_t));
+    ret1 = GetFinalPathNameByHandleW(horig, wres, ret, VOLUME_NAME_DOS);
+    if (ret1 <= 0 || ret1 >= ret) {
+	CloseHandle(horig);
+	return NULL;
+    }
+    
+    /* get rid of the \\?\ prefix */
+    size_t len = ret;
+    int strip = 0;
+    if (len < 4 || wcsncmp(L"\\\\?\\", wres, 4)) {
+	/* res should start with \\?\ */
+	CloseHandle(horig);
+	return NULL;
+    }
+    
+    if (len > 8 && !wcsncmp(L"UNC\\", wres+4, 4)) {
+	/* UNC path \\?\UNC */
+	wres[6] = L'\\';
+	strip = 6;
+    } else if (len >= 6 && Ri18n_iswctype(wres[4], Ri18n_wctype("alpha"))
+	     && wres[5] == L':' && wres[6] == L'\\')
+	/* \\?\D: */
+	strip = 4;
+    else {
+	CloseHandle(horig);
+	return NULL;
+    }
+    wmemmove(wres, wres+strip, len-strip+1);
+
+    /* sanity check if the file exists using the normalized path, a normalized
+       path to an existing file should still be working */
+    /* FILE_FLAG_BACKUP_SEMANTICS needed to open a directory */
+    hres = CreateFileW(wres, 0,
+                       FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                       NULL, OPEN_EXISTING,
+                       FILE_ATTRIBUTE_HIDDEN | FILE_FLAG_BACKUP_SEMANTICS,
+                       NULL);
+    if (hres == INVALID_HANDLE_VALUE) {
+	CloseHandle(horig);
+	return NULL;
+    }
+
+    /* sanity check that the handles point to the same file; they should, but
+       better be safe wrt to undocumented features/changes of gfpnbhw */
+    ret = isSameFile(horig, hres);
+    CloseHandle(horig);
+    CloseHandle(hres);
+
+    /* return wres elso when isSameFile fails with -1 */
+    return ret ? wres : NULL;
+}
+
+/* returns R_alloc'd result */
+attribute_hidden wchar_t *R_getFullPathNameW(const wchar_t *orig)
+{
+    DWORD ret, ret1;
+
+    ret = GetFullPathNameW(orig, 0, NULL, NULL);
+    if (ret <= 0)
+	return NULL;
+    wchar_t *wres = (wchar_t*)R_alloc(ret, sizeof(wchar_t));
+    ret1 = GetFullPathNameW(orig, ret, wres, NULL);
+    if (ret1 <= 0 || ret1 >= ret) 
+	return NULL;
+    else
+	return wres;
+}
+
+/* returns R_alloc'd result */
+attribute_hidden char *R_getFullPathName(const char *orig)
+{
+    DWORD ret, ret1;
+
+    ret = GetFullPathName(orig, 0, NULL, NULL);
+    if (ret == 0 && GetLastError() == ERROR_FILENAME_EXCED_RANGE) {
+	/* GetFullPathNameA unfortunately does not work with long paths
+	   (tested on Windows 10 19045), it fails with
+	   ERROR_FILENAME_EXCED_RANGE even when long paths are enabled. */
+	size_t cnt = mbstowcs(NULL, orig, 0);
+	if (cnt != (size_t)-1) {
+	    cnt++;
+	    wchar_t *worig = (wchar_t*) R_alloc(cnt, sizeof(wchar_t));
+	    mbstowcs(worig, orig, cnt);
+	    wchar_t *wres = R_getFullPathNameW(worig);
+	    if (wres) {
+		cnt = wcstombs(NULL, wres, 0) + 1;
+		if (cnt != (size_t)-1) {
+		    char *res = R_alloc(cnt, 1);
+		    wcstombs(res, wres, cnt);
+		    return res;
+		}
+	    }
+	}
+    }
+    if (ret <= 0)
+	return NULL;
+    char *res = R_alloc(ret, 1);
+    ret1 = GetFullPathName(orig, ret, res, NULL);
+    if (ret1 <= 0 || ret1 >= ret) 
+	return NULL;
+    else
+	return res;
+}
+
+/* returns R_alloc'd result */
+static wchar_t *getLongPathNameW(const wchar_t *orig)
+{
+    DWORD ret, ret1;
+
+    ret = GetLongPathNameW(orig, NULL, 0);
+    if (ret <= 0)
+	return NULL;
+    wchar_t *wres = (wchar_t*)R_alloc(ret, sizeof(wchar_t));
+    ret1 = GetLongPathNameW(orig, wres, ret);
+    if (ret1 <= 0 || ret1 >= ret)
+	return NULL;
+    else
+	return wres;
+}
+
+/* returns R_alloc'd result */
+static char *getLongPathName(const char *orig)
+{
+    DWORD ret, ret1;
+
+    ret = GetLongPathName(orig, NULL, 0);
+    if (ret <= 0)
+	return NULL;
+    char *res = R_alloc(ret, 1);
+    ret1 = GetLongPathName(orig, res, ret);
+    if (ret1 <= 0 || ret1 >= ret) 
+	return NULL;
+    else 
+	return res;
+}
 
 void R_UTF8fixslash(char *s); /* from main/util.c */
 SEXP do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, paths = CAR(args), el, slash;
-    int i, n = LENGTH(paths), res;
-    char tmp[4*MAX_PATH+1], longpath[4*MAX_PATH+1], *tmp2;
-    wchar_t wtmp[32768], wlongpath[32768], *wtmp2;
+    int i, n = LENGTH(paths);
     int mustWork, fslash = 0;
+    const void *vmax = vmaxget();
 
     checkArity(op, args);
     if(!isString(paths))
@@ -451,7 +681,7 @@ SEXP do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
     slash = CADR(args);
     if(!isString(slash) || LENGTH(slash) != 1)
 	errorcall(call, "'winslash' must be a character string");
-    const char *sl = CHAR(STRING_ELT(slash, 0));
+    const char *sl = translateCharFP(STRING_ELT(slash, 0));
     if (strcmp(sl, "/") && strcmp(sl, "\\"))
 	errorcall(call, "'winslash' must be '/' or '\\\\'");
     if (strcmp(sl, "/") == 0) fslash = 1;
@@ -460,7 +690,6 @@ SEXP do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     PROTECT(ans = allocVector(STRSXP, n));
     for (i = 0; i < n; i++) {
-    	int warn = 0;
     	SEXP result;
 	el = STRING_ELT(paths, i);
 	result = el;
@@ -471,74 +700,87 @@ SEXP do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    else if(mustWork == NA_LOGICAL)
 		warningcall(call, "path[%d]=NA", i+1);
 	} else if(getCharCE(el) == CE_UTF8) {
-	    if ((res = GetFullPathNameW(filenameToWchar(el, FALSE), 32768, 
-					wtmp, &wtmp2)) && res <= 32768) {
-		if ((res = GetLongPathNameW(wtmp, wlongpath, 32768))
-		    && res <= 32768) {
-	    	    wcstoutf8(longpath, wlongpath, sizeof(longpath));
-		    if(fslash) R_UTF8fixslash(longpath);
-	    	    result = mkCharCE(longpath, CE_UTF8);
-		} else if(mustWork == 1) {
-		    errorcall(call, "path[%d]=\"%s\": %s", i+1, 
-			      translateChar(el), 
-			      formatError(GetLastError()));	
-	    	} else {
-	    	    wcstoutf8(tmp, wtmp, sizeof(tmp));
-		    if(fslash) R_UTF8fixslash(tmp);
-	    	    result = mkCharCE(tmp, CE_UTF8);
-	    	    warn = 1;
-	    	}
-	    } else if(mustWork == 1) {
-		errorcall(call, "path[%d]=\"%s\": %s", i+1, 
-			  translateChar(el), 
-			  formatError(GetLastError()));	
+	    const wchar_t *wel = filenameToWchar(el, FALSE);
+	    wchar_t *wfull = R_getFullPathNameW(wel);
+	    wchar_t *wnorm = getFinalPathNameW(wel);
+
+	    /* if normalized to UNC path but full path is D:..., fall back
+	       to GetLongPathName */
+	    if (wnorm && wnorm[0] == L'\\' && wnorm[1] == L'\\' &&
+		wfull && Ri18n_iswctype(wfull[0], Ri18n_wctype("alpha")) &&
+	        wfull[1] == L':') wnorm = NULL;
+	    if (!wnorm && wfull)
+		/* silently fall back to GetFullPathName/GetLongPathName */
+		/* getLongPathName will fail for non-existent paths */
+		wnorm = getLongPathNameW(wfull);
+	    if (wnorm) {
+		if (fslash)
+		    R_wfixslash(wnorm);
+		result = mkCharWUTF8(wnorm);
 	    } else {
-		if (fslash) {
-		    strcpy(tmp, translateCharUTF8(el));
-		    R_UTF8fixslash(tmp);
-	    	    result = mkCharCE(tmp, CE_UTF8);
+		if (mustWork == 1) {
+		    errorcall(call, "path[%d]=\"%ls\": %s", i+1, 
+			      wel, formatError(GetLastError()));
+		} else if (mustWork == NA_LOGICAL) {
+		    warningcall(call, "path[%d]=\"%ls\": %s", i+1, 
+				wel, formatError(GetLastError()));
+		}
+		if (wfull) {
+		    if (fslash)
+			R_wfixslash(wfull);
+		    result = mkCharWUTF8(wfull);
+		} else {
+		    const char *elutf8 = translateCharUTF8(el);
+		    if (fslash) {
+			char *normutf8 = R_alloc(strlen(elutf8) + 1, 1);
+			strcpy(normutf8, elutf8);
+			R_UTF8fixslash(normutf8);
+			result = mkCharCE(normutf8, CE_UTF8);
+		    } else
+			result = mkCharCE(elutf8, CE_UTF8);
 		}
-	    	warn = 1;
 	    }
-	    if (warn && (mustWork == NA_LOGICAL))
-	    	warningcall(call, "path[%d]=\"%ls\": %s", i+1, 
-			    filenameToWchar(el,FALSE), 
-			    formatError(GetLastError()));
 	} else {
-	    if ((res = GetFullPathName(translateChar(el), MAX_PATH, tmp, &tmp2)) 
-		&& res <= MAX_PATH) {
-	    	if ((res = GetLongPathName(tmp, longpath, MAX_PATH))
-		    && res <= MAX_PATH) {
-		    if(fslash) R_fixslash(longpath);
-	    	    result = mkChar(longpath);
-		} else if(mustWork == 1) {
-		    errorcall(call, "path[%d]=\"%s\": %s", i+1, 
-			      translateChar(el), 
-			      formatError(GetLastError()));	
-	    	} else {
-		    if(fslash) R_fixslash(tmp);
-	    	    result = mkChar(tmp);
-	    	    warn = 1;
-	    	}
-	    } else if(mustWork == 1) {
-		errorcall(call, "path[%d]=\"%s\": %s", i+1, 
-			  translateChar(el), 
-			  formatError(GetLastError()));	
+	    const char *tel = translateChar(el);
+	    char *full = R_getFullPathName(tel);
+	    char *norm = getFinalPathName(tel);
+
+	    /* if normalized to UNC path but full path is D:..., fall back
+	       to GetLongPathName */
+	    if (norm && norm[0] == '\\' && norm[1] == '\\' &&
+		full && isalpha(full[0]) && full[1] == ':') norm = NULL;
+	    if (!norm && full)
+		/* silently fall back to GetFullPathName/GetLongPathName */
+		/* getLongPathName will fail for non-existent paths */
+		norm = getLongPathName(full);
+	    if (norm) {
+		if (fslash)
+		    R_fixslash(norm);
+		result = mkChar(norm);
 	    } else {
-		if (fslash) {
-		    strcpy(tmp, translateChar(el));
-		    R_fixslash(tmp);
-		    result = mkChar(tmp);
+		if (mustWork == 1) {
+		    errorcall(call, "path[%d]=\"%s\": %s", i+1, 
+			      tel, formatError(GetLastError()));
+		} else if (mustWork == NA_LOGICAL) {
+		    warningcall(call, "path[%d]=\"%s\": %s", i+1, 
+				tel, formatError(GetLastError()));
 		}
-	    	warn = 1;
+		if (full) {
+		    if (fslash)
+			R_fixslash(full);
+		    result = mkChar(full);
+		} else if (fslash) {
+		    norm = R_alloc(strlen(tel) + 1, 1);
+		    strcpy(norm, tel);
+		    R_fixslash(norm);
+		    result = mkChar(norm);
+		} else
+		    result = mkChar(tel);
 	    }
-	    if (warn && (mustWork == NA_LOGICAL))
-		warningcall(call, "path[%d]=\"%s\": %s", i+1, 
-			    translateChar(el), 
-			    formatError(GetLastError()));	
 	}
 	SET_STRING_ELT(ans, i, result);
     }
+    vmaxset(vmax);
     UNPROTECT(1);
     return ans;
 }
@@ -548,8 +790,6 @@ SEXP in_shortpath(SEXP paths)
 {
     SEXP ans, el;
     int i, n = LENGTH(paths);
-    char tmp[4*MAX_PATH+1];
-    wchar_t wtmp[32768];
     DWORD res;
     const void *vmax = vmaxget();
 
@@ -559,23 +799,41 @@ SEXP in_shortpath(SEXP paths)
     for (i = 0; i < n; i++) {
 	el = STRING_ELT(paths, i);
 	if(getCharCE(el) == CE_UTF8) {
-	    res = GetShortPathNameW(filenameToWchar(el, FALSE), wtmp, 32768);
-	    if (res && res <= 32768)
-		wcstoutf8(tmp, wtmp, sizeof(tmp));
-	    else
-		strcpy(tmp, translateChar(el));
-	    /* documented to return paths using \, which the API call does
-	       not necessarily do */
-	    R_fixbackslash(tmp);
-	    SET_STRING_ELT(ans, i, mkCharCE(tmp, CE_UTF8));
+	    wchar_t *wfn = filenameToWchar(el, FALSE);
+	    res = GetShortPathNameW(wfn, NULL, 0);
+	    if (res > 0) {
+		wchar_t *wsfn = (wchar_t*)R_alloc(res, sizeof(wchar_t));
+		DWORD res1 = GetShortPathNameW(wfn, wsfn, res);
+		if (res1 > 0 && res1 < res) {
+		    /* documented to return paths using \, which the API call
+		       does not necessarily do */
+		    R_wfixbackslash(wsfn);
+		    SET_STRING_ELT(ans, i, mkCharWUTF8(wsfn));
+		    continue;
+		}
+	    }
 	} else {
-	    res = GetShortPathName(translateChar(el), tmp, MAX_PATH);
-	    if (res == 0 || res > MAX_PATH) strcpy(tmp, translateChar(el));
-	    /* documented to return paths using \, which the API call does
-	       not necessarily do */
-	    R_fixbackslash(tmp);
-	    SET_STRING_ELT(ans, i, mkChar(tmp));
+	    const char *fn = translateChar(el);
+	    res = GetShortPathName(fn, NULL, 0);
+	    if (res > 0) {
+		char *sfn = R_alloc(res, 1);
+		DWORD res1 = GetShortPathName(fn, sfn, res);
+		if (res1 > 0 && res1 < res) {
+		    /* documented to return paths using \, which the API call
+		       does not necessarily do */
+		    R_fixbackslash(sfn);
+		    SET_STRING_ELT(ans, i, mkChar(sfn));
+		    continue;
+		}
+	    }
 	}
+	/* we didn't get a short name, so return the original with
+	   backslashes as separators (as documented) */
+	const char *fn = translateChar(el);
+	char *ffn = R_alloc(strlen(fn) + 1, 1);
+	strcpy(ffn, fn);
+	R_fixbackslash(ffn);
+	SET_STRING_ELT(ans, i, mkChar(ffn));
     }
     UNPROTECT(1);
     vmaxset(vmax);
@@ -783,97 +1041,19 @@ char *getDLLVersion(void)
     return (DLLversion);
 }
 
-
-
-/* UTF-8 support ----------------------------------------------- */
-
-#ifdef SUPPORT_UTF8_WIN32
-/* This is currently unused: for faking UTF-8 locale conversions */
-
-#define FAKE_UTF8 1
-
-
-size_t Rmbrtowc(wchar_t *wc, const char *s)
-{
-#ifdef FAKE_UTF8
-    unsigned int byte;
-    wchar_t local, *w;
-    byte = *((unsigned char *)s);
-    w = wc ? wc: &local;
-
-    if (byte == 0) {
-	*w = (wchar_t) 0;
-	return 0;
-    } else if (byte < 0xC0) {
-	*w = (wchar_t) byte;
-	return 1;
-    } else if (byte < 0xE0) {
-	if(strlen(s) < 2) return -2;
-	if ((s[1] & 0xC0) == 0x80) {
-	    *w = (wchar_t) (((byte & 0x1F) << 6) | (s[1] & 0x3F));
-	    return 2;
-	} else return -1;
-    } else if (byte < 0xF0) {
-	if(strlen(s) < 3) return -2;
-	if (((s[1] & 0xC0) == 0x80) && ((s[2] & 0xC0) == 0x80)) {
-	    *w = (wchar_t) (((byte & 0x0F) << 12)
-			    | ((s[1] & 0x3F) << 6) | (s[2] & 0x3F));
-	    byte = *w;
-	    if(byte >= 0xD800 && byte <= 0xDFFF) return -1; /* surrogate */
-	    if(byte == 0xFFFE || byte == 0xFFFF) return -1;
-	    return 3;
-	} else return -1;
-    }
-    return -2;
-#else
-    return mbrtowc(wc, s, MB_CUR_MAX, NULL);
-#endif
-}
-
-size_t Rmbstowcs(wchar_t *wc, const char *s, size_t n)
-{
-#ifdef FAKE_UTF8
-    int m, res=0;
-    const char *p;
-
-    if(wc) {
-	for(p = s; ; p+=m) {
-	    m = Rmbrtowc(wc+res, p);
-	    if(m < 0) error(_("invalid input in 'Rmbstowcs'"));
-	    if(m <= 0) break;
-	    res++;
-	    if(res >= n) break;
-	}
-    } else {
-	for(p = s; ; p+=m) {
-	    m  = Rmbrtowc(NULL, p);
-	    if(m < 0) error(_("invalid input in 'Rmbstowcs'"));
-	    if(m <= 0) break;
-	    res++;
-	}
-    }
-    return res;
-#else
-    return mbstowcs(wc, s, n);
-#endif
-}
-#endif
-
 /* base::file.choose */
 SEXP attribute_hidden do_filechoose(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
     wchar_t *fn;
-    char str[4*MAX_PATH+1];
 
     checkArity(op, args);
     setuserfilterW(L"All files (*.*)\0*.*\0\0");
     fn = askfilenameW(G_("Select file"), "");
     if (!fn)
 	error(_("file choice cancelled"));
-    wcstoutf8(str, fn, sizeof(str));
     PROTECT(ans = allocVector(STRSXP, 1));
-    SET_STRING_ELT(ans, 0, mkCharCE(str, CE_UTF8));
+    SET_STRING_ELT(ans, 0, mkCharWUTF8(fn));
     UNPROTECT(1);
     return ans;
 }
