@@ -24,12 +24,26 @@ R_CallMethodDef CallMethods[] = {
   {"Rkdnearest",(DL_FUNC)&Rkdnearest,4},
   {"Rkradius",(DL_FUNC)&Rkradius,5},
   {"sXWXd",(DL_FUNC)&sXWXd,5},
+  {"CXWXd0",(DL_FUNC)&CXWXd0,14},
+  {"CXWXd1",(DL_FUNC)&CXWXd1,16},
   {"sXbd",(DL_FUNC)&sXbd,3},
+  {"CXbd",(DL_FUNC)&CXbd,13},
   {"sXyd",(DL_FUNC)&sXyd,3},
+  {"CXWyd",(DL_FUNC)&CXWyd,17},
   {"sdiagXVXt",(DL_FUNC)&sdiagXVXt,4},
+  {"CdiagXVXt",(DL_FUNC)&CdiagXVXt,14},
   {"stmm",(DL_FUNC)&stmm,1},
   {"AddBVB",(DL_FUNC)&AddBVB,3},
   {"isa1p",(DL_FUNC)&isa1p,3},
+  {"mrow_sum",(DL_FUNC)&mrow_sum,3},
+  {"ncv",(DL_FUNC)&ncv,17},
+  {"Rncv",(DL_FUNC)&Rncv,19},
+  {"ncvls",(DL_FUNC)&ncvls,18},
+  {"Rncvls",(DL_FUNC)&Rncvls,19},
+  {"nei_cov",(DL_FUNC)&nei_cov,4},
+  {"dpdev",(DL_FUNC)&dpdev,1},
+  {"spdev",(DL_FUNC)&spdev,1},
+  {"mvnll",(DL_FUNC)&mvnll,13},
   {NULL, NULL, 0}
 };
 
@@ -38,7 +52,6 @@ R_CMethodDef CEntries[] = {
     {"davies",(DL_FUNC) davies,10},
     {"tri_chol",(DL_FUNC) tri_chol,4},
     {"diagXVXt", (DL_FUNC) &diagXVXt,21},
-    {"XWXd", (DL_FUNC) &XWXd,18},
     {"XWXd0", (DL_FUNC) &XWXd0,18},
     {"XWXd1", (DL_FUNC) &XWXd1,22},
     {"XWyd", (DL_FUNC) &XWyd,21},
@@ -50,7 +63,7 @@ R_CMethodDef CEntries[] = {
     {"coxpred", (DL_FUNC) &coxpred, 14},
     {"coxpp", (DL_FUNC) &coxpp, 10},
     {"coxlpl", (DL_FUNC) &coxlpl, 17},
-    {"mvn_ll", (DL_FUNC) &mvn_ll,15},
+    // {"mvn_ll", (DL_FUNC) &mvn_ll,15},
     {"RMonoCon", (DL_FUNC) &RMonoCon, 7},
     {"RuniqueCombs", (DL_FUNC) &RuniqueCombs, 4},
     {"RPCLS", (DL_FUNC) &RPCLS, 13},
@@ -95,6 +108,9 @@ R_CMethodDef CEntries[] = {
     {"mgcv_pqr",(DL_FUNC)&mgcv_pqr,6},
     {"getRpqr",(DL_FUNC)&getRpqr,6},
     {"mgcv_pqrqy",(DL_FUNC)&mgcv_pqrqy,8},
+    {"minres",(DL_FUNC)&minres,7},
+    {"Zb",(DL_FUNC)&Zb,6},
+    {"Ztb",(DL_FUNC)&Ztb,7},
     {NULL, NULL, 0}
 };
 
@@ -102,7 +118,7 @@ void R_init_mgcv(DllInfo *dll)
 {
     R_registerRoutines(dll, CEntries, CallMethods, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
-    R_RegisterCCallable("mgcv","mgcv_pmmult2", (DL_FUNC) &mgcv_pmmult2); 
+    R_RegisterCCallable("mgcv","mgcv_pmmult2", (DL_FUNC) &mgcv_pmmult2); // allows calling from other packages
     R_RegisterCCallable("mgcv","pls_fit1", (DL_FUNC) &pls_fit1);
     R_RegisterCCallable("mgcv","gdi2", (DL_FUNC) &gdi2); 
 }
