@@ -17,7 +17,7 @@
 #  https://www.R-project.org/Licenses/
 
 Reduce <-
-function(f, x, init, right = FALSE, accumulate = FALSE)
+function(f, x, init, right = FALSE, accumulate = FALSE, simplify = TRUE)
 {
     mis <- missing(init)
     len <- length(x)
@@ -95,7 +95,7 @@ function(f, x, init, right = FALSE, accumulate = FALSE)
         ## If all results have length one, we can simplify.
         ## (Note that we do not simplify to arrays in case all results
         ## have a common length > 1.)
-	if(all(lengths(out) == 1L))
+	if(all(lengths(out) == 1L) && simplify)
             out <- unlist(out, recursive = FALSE)
         out
     }
@@ -104,6 +104,7 @@ function(f, x, init, right = FALSE, accumulate = FALSE)
 Filter <-
 function(f, x)
 {
+    f <- match.fun(f)
     ind <- as.logical(unlist(lapply(x, f)))
     x[which(ind)]
 }
@@ -126,6 +127,8 @@ function(f)
 Position <-
 function(f, x, right = FALSE, nomatch = NA_integer_)
 {
+    f <- match.fun(f)
+    
     ind <- if(right) rev(seq_along(x)) else seq_along(x)
 
     for(i in ind)
