@@ -1,7 +1,7 @@
 #  File src/library/base/R/load.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2022 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -90,7 +90,8 @@ save <- function(..., list = character(),
             }
         }
         if (is.character(file)) {
-	    if(!nzchar(file)) stop("'file' must be non-empty string")
+	    if(!nzchar(file)) 
+                stop(gettextf("'%s' must be a non-empty string", "file"), domain = NA)
 	    if(!is.character(compress)) {
 		if(!is.logical(compress))
 		    stop("'compress' must be logical or character")
@@ -128,9 +129,8 @@ save <- function(..., list = character(),
 save.image <- function (file = ".RData", version = NULL, ascii = FALSE,
                         compress = !ascii, safe = TRUE)
 {
-    if (! is.character(file) || file == "")
-        stop("'file' must be non-empty string")
-
+    if (!is.character(file) || length(file) != 1 || file == "")
+        stop(gettextf("'%s' must be a non-empty string", "file"), domain = NA)
     opts <- getOption("save.image.defaults")
     if(is.null(opts)) opts <- getOption("save.defaults")
 
@@ -171,6 +171,9 @@ sys.load.image <- function(name, quiet)
 {
     if (file.exists(name)) {
         load(name, envir = .GlobalEnv)
+        sample.kind <- .Internal(RNGkind(NULL, NULL, NULL))[3L]
+        if (sample.kind == 0L)
+            warning("non-uniform 'Rounding' sampler used", domain = NA)
         if (! quiet)
 	    message("[Previously saved workspace restored]", "\n")
     }
