@@ -5,7 +5,7 @@
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
-#  the Free Software Foundation; either version 2 of the License, or
+#  the Free Software Foundation; either version 3 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
@@ -24,17 +24,14 @@
 ##  2) We use  ':::' instead of '::' inside the code below, for efficiency only
 
 getNamespace <- function(name) {
-    ns <- .Internal(getRegisteredNamespace(name))
-    if (! is.null(ns)) ns
-    else loadNamespace(name)
+    .Internal(getRegisteredNamespace(name)) %||% loadNamespace(name)
 }
 
 .getNamespace <- function(name) .Internal(getRegisteredNamespace(name))
 
 ..getNamespace <- function(name, where) {
-    ns <- .Internal(getRegisteredNamespace(name))
-    if (!is.null(ns)) ns
-    else tryCatch(loadNamespace(name), error = function(e) {
+    .Internal(getRegisteredNamespace(name)) %||%
+	tryCatch(loadNamespace(name), error = function(e) {
              tr <- Sys.getenv("_R_NO_REPORT_MISSING_NAMESPACES_")
              if( tr == "false" || (where != "<unknown>" && !nzchar(tr)) ) {
                  warning(gettextf("namespace %s is not available and has been replaced\nby .GlobalEnv when processing object %s",
@@ -208,6 +205,7 @@ loadNamespace <- function (package, lib.loc = NULL,
                               sQuote(package), current, zop, zversion),
                      domain = NA)
         }
+        ## return
         ns
     } else {
         lev <- 0L
@@ -285,7 +283,7 @@ loadNamespace <- function (package, lib.loc = NULL,
         bindTranslations <- function(pkgname, pkgpath)
         {
             ## standard packages are treated differently
-            std <- c("compiler", "foreign", "grDevices", "graphics", "grid",
+            std <- c("compiler", "grDevices", "graphics", "grid",
                      "methods", "parallel", "splines", "stats", "stats4",
                      "tcltk", "tools", "utils")
             popath <- if (pkgname %in% std) .popath else file.path(pkgpath, "po")
@@ -414,7 +412,7 @@ loadNamespace <- function (package, lib.loc = NULL,
                      call. = FALSE, domain = NA)
             ## we need to ensure that S4 dispatch is on now if the package
             ## will require it, or the exports will be incomplete.
-            dependsMethods <- "methods" %in% names(pkgInfo$Depends)
+            dependsMethods <- "methods" %in% c(names(pkgInfo$Depends), names(vI))
             if(dependsMethods) loadNamespace("methods")
             if(!is.null(zop <- versionCheck[["op"]]) &&
                !is.null(zversion <- versionCheck[["version"]]) &&
@@ -808,7 +806,8 @@ loadNamespace <- function (package, lib.loc = NULL,
         if (length(exports)) {
             stoplist <- c(".__NAMESPACE__.", ".__S3MethodsTable__.",
                           ".packageName", ".First.lib", ".onLoad",
-                          ".onAttach", ".conflicts.OK", ".noGenerics")
+                          ".onAttach", ".conflicts.OK", ".noGenerics",
+                          ".__global__", ".__suppressForeign__")
             exports <- exports[! exports %in% stoplist]
         }
 	if(lev > 2L) message("--- processing exports for ", dQuote(package))
@@ -1532,7 +1531,7 @@ parseNamespaceFile <- function(package, package.lib, mustExist = TRUE)
          S3methods = unique(S3methods[seq_len(nS3), , drop = FALSE]) )
 } ## end{parseNamespaceFile}
 
-## Still used inside registerS3methods().
+## used inside registerS3methods(); workhorse of .S3method()
 registerS3method <- function(genname, class, method, envir = parent.frame()) {
     addNamespaceS3method <- function(ns, generic, class, method) {
 	regs <- rbind(.getNamespaceInfo(ns, "S3methods"),
@@ -1560,12 +1559,12 @@ registerS3method <- function(genname, class, method, envir = parent.frame()) {
             delayedAssign(x, get(method, envir = home), assign.env = envir)
         }
         if(!exists(method, envir = envir)) {
-            ## need to avoid conflict with message at l.1298
+            ## need to avoid conflict with any(notex) warning message
             warning(gettextf("S3 method %s was declared but not found",
                              sQuote(method)), call. = FALSE)
         } else {
 	    assignWrapped(paste(genname, class, sep = "."), method, home = envir,
-	    	    envir = table)
+			  envir = table)
         }
     }
     else if (is.function(method))
@@ -1631,15 +1630,20 @@ registerS3methods <- function(info, package, env)
     ## can remain unchanged.
     if(ncol(info) == 3L)
         info <- cbind(info, NA_character_)
-    Info <- cbind(info[, 1L : 3L, drop = FALSE], methname, info[, 4L])
+    Info <- cbind(info[, 1L:3L, drop = FALSE], methname, info[, 4L])
     loc <- names(env)
-    if(any(notex <- match(info[,3], loc, nomatch=0L) == 0L)) { # not %in%
+    if(any(notex <- match(info[,3L], loc, nomatch=0L) == 0L)) { # not %in%
+      ## Try harder, as in registerS3method(); parent since *not* in env:
+      found <- vapply(info[notex, 3L], exists, logical(1), envir = parent.env(env))
+      notex[notex] <- !found
+      if(any(notex)) {
         warning(sprintf(ngettext(sum(notex),
                                  "S3 method %s was declared in NAMESPACE but not found",
                                  "S3 methods %s were declared in NAMESPACE but not found"),
                         paste(sQuote(info[notex, 3]), collapse = ", ")),
                 call. = FALSE, domain = NA)
         Info <- Info[!notex, , drop = FALSE]
+      }
     }
     eager <- is.na(Info[, 5L])
     delayed <- Info[!eager, , drop = FALSE]
