@@ -1,7 +1,7 @@
 #  File src/library/base/R/serialize.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2018 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,8 @@ saveRDS <-
              compress = TRUE, refhook = NULL)
 {
     if(is.character(file)) {
-	if(file == "") stop("'file' must be non-empty string")
+        if(length(file) != 1 || file == "")
+            stop(gettextf("'%s' must be a non-empty string", "file"), domain = NA)
 	object <- object # do not create corrupt file if object does not exist
 	mode <- if(ascii %in% FALSE) "wb" else "w"
 	con <- if (is.logical(compress))
@@ -55,6 +56,17 @@ readRDS <- function(file, refhook = NULL)
     .Internal(unserializeFromConn(con, refhook))
 }
 
+infoRDS <- function(file)
+{
+    if(is.character(file)) {
+        con <- gzfile(file, "rb")
+        on.exit(close(con))
+    } else if (inherits(file, "connection"))
+        con <- if(inherits(file, "url")) gzcon(file) else file
+    else stop("bad 'file' argument")
+    .Internal(serializeInfoFromConn(con))
+}
+
 serialize <-
     function(object, connection, ascii = FALSE, xdr = TRUE,
              version = NULL, refhook = NULL)
