@@ -186,6 +186,12 @@ static void PicTeX_Text(double x, double y, const char *str,
 			double rot, double hadj, 
 			const pGEcontext gc,
 			pDevDesc dd);
+static SEXP     PicTeX_setPattern(SEXP pattern, pDevDesc dd);
+static void     PicTeX_releasePattern(SEXP ref, pDevDesc dd);
+static SEXP     PicTeX_setClipPath(SEXP path, SEXP ref, pDevDesc dd);
+static void     PicTeX_releaseClipPath(SEXP ref, pDevDesc dd);
+static SEXP     PicTeX_setMask(SEXP path, SEXP ref, pDevDesc dd);
+static void     PicTeX_releaseMask(SEXP ref, pDevDesc dd);
 
 	/* Support routines */
 
@@ -460,7 +466,13 @@ static double PicTeX_StrWidth(const char *str,
 	    if (status >= 0) 
 		for (i = 0; i < ucslen; i++)
 		    if(ucs[i] < 128) sum += charwidth[ptd->fontface-1][ucs[i]];
-		    else sum += (double) Ri18n_wcwidth(ucs[i]) * 0.5; /* A guess */
+		    else {
+#ifdef USE_RI18N_WIDTH
+			sum += (double) Ri18n_wcwidth(ucs[i]) * 0.5; /* A guess */
+#else
+			sum += (double) wcwidth((wchar_t)ucs[i]) * 0.5; /* A guess */
+#endif
+		    }
 	    else
 		warning(_("invalid string in '%s'"), "PicTeX_StrWidth");
 	} else
@@ -599,6 +611,25 @@ static void PicTeX_Text(double x, double y, const char *str,
     fprintf(ptd->texfp," at %.2f %.2f\n", x, y);
 }
 
+static SEXP PicTeX_setPattern(SEXP pattern, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void PicTeX_releasePattern(SEXP ref, pDevDesc dd) {} 
+
+static SEXP PicTeX_setClipPath(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void PicTeX_releaseClipPath(SEXP ref, pDevDesc dd) {}
+
+static SEXP PicTeX_setMask(SEXP path, SEXP ref, pDevDesc dd) {
+    return R_NilValue;
+}
+
+static void PicTeX_releaseMask(SEXP ref, pDevDesc dd) {}
+
+
 static
 Rboolean PicTeXDeviceDriver(pDevDesc dd, const char *filename, 
 			    const char *bg, const char *fg,
@@ -638,6 +669,12 @@ Rboolean PicTeXDeviceDriver(pDevDesc dd, const char *filename,
     dd->metricInfo = PicTeX_MetricInfo;
     dd->hasTextUTF8 = FALSE;
     dd->useRotatedTextInContour = FALSE;
+    dd->setPattern      = PicTeX_setPattern;
+    dd->releasePattern  = PicTeX_releasePattern;
+    dd->setClipPath     = PicTeX_setClipPath;
+    dd->releaseClipPath = PicTeX_releaseClipPath;
+    dd->setMask         = PicTeX_setMask;
+    dd->releaseMask     = PicTeX_releaseMask;
 
     /* Screen Dimensions in Pixels */
 
@@ -696,6 +733,8 @@ Rboolean PicTeXDeviceDriver(pDevDesc dd, const char *filename,
 
     dd->deviceSpecific = (void *) ptd;
     dd->displayListOn = FALSE;
+    dd->deviceVersion = R_GE_definitions;
+
     return TRUE;
 }
 
@@ -718,7 +757,11 @@ SEXP PicTeX(SEXP args)
 
     const void *vmax = vmaxget();
     args = CDR(args); /* skip entry point name */
-    file = translateCharFP(asChar(CAR(args))); args = CDR(args);
+
+     SEXP tmp = asChar(CAR(args));
+    if (tmp == NA_STRING)
+	error(_("invalid 'file' parameter in %s"), "pictex");
+    file = translateCharFP(tmp);  args = CDR(args);
     bg = CHAR(asChar(CAR(args)));   args = CDR(args);
     fg = CHAR(asChar(CAR(args)));   args = CDR(args);
     width = asReal(CAR(args));	     args = CDR(args);
