@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 2001-3 Paul Murrell
- *                2003-2020 The R Core Team
+ *                2003-2024 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -353,6 +353,11 @@ double evaluateGrobUnit(double value, SEXP grob,
      * function call rather than directly playing with dd->recordGraphics
      */
     Rboolean record = dd->recordGraphics;
+#ifdef R_GE_DEBUG
+    if (getenv("R_GE_DEBUG_record")) {
+        printf("evaluateGrobUnit: record = FALSE\n");
+    }
+#endif
     dd->recordGraphics = FALSE;
     /*
      * Save the current viewport transform 
@@ -575,6 +580,11 @@ double evaluateGrobUnit(double value, SEXP grob,
      * evaluation, dd->recordGraphics is set to TRUE
      * on all graphics devices (see GEonExit(); called in errors.c)
      */
+#ifdef R_GE_DEBUG
+    if (getenv("R_GE_DEBUG_record")) {
+        printf("evaluateGrobUnit: record = %d\n", record);
+    }
+#endif
     dd->recordGraphics = record;
     return result;
 }
@@ -1939,14 +1949,15 @@ SEXP summaryUnits(SEXP units, SEXP op_type) {
 	SET_STRING_ELT(cl, 0, mkChar("unit"));
 	SET_STRING_ELT(cl, 1, mkChar("unit_v2"));
 
-	int is_type[m];
+	int is_type[m > 0 ? m : 1L]; // mmight be zero which would not be legal C.
 	int all_type = 1;
 	
 	for (int i = 0; i < n; i++) {
 		int k = 0;
-		int first_type, current_type;
+		int first_type = -1;
+                int current_type;
 		SEXP unit = SET_VECTOR_ELT(out, i, allocVector(VECSXP, 3));
-		SEXP first_data;
+		SEXP first_data = R_NilValue;
 		for (int j = 0; j < m; j++) {
 			SEXP unit_temp = PROTECT(unitScalar(VECTOR_ELT(units, j), i));
 			current_type = uUnit(unit_temp);
