@@ -1,7 +1,7 @@
 #  File src/library/methods/R/Methods.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2020 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -267,31 +267,31 @@ isGeneric <-
         ## a successful search will usually end here w/o other tests
          if(!is.null(fdef))
            return(if(getName) fdef@generic else TRUE)
-     }
+    }
     if(is.null(fdef))
         fdef <- getFunction(f, where=where, mustFind = FALSE)
     if(is.null(fdef))
       return(FALSE)
     ## check primitives. These are never found as explicit generic functions.
-    if(isBaseFun(fdef)) {
-        if(is.character(f) && f %in% "as.double") f <- "as.numeric"
-        ## the definition of isGeneric() for a base function is that methods are defined
+    if(is.primitive(fdef)) {
+        ## the definition of isGeneric() for such a function is that methods are defined
         ## (other than the default primitive)
-        gen <- genericForBasic(f, mustFind = FALSE)
-        return(is.function(gen) && length(names(.getMethodsTable(gen))) > 1L)
+        gen <- genericForBasic(.primname(fdef), mustFind = FALSE)
+        return(
+            if(is.function(gen) && length(names(.getMethodsTable(gen))) > 1L)
+                if(getName) gen@generic else TRUE
+            else FALSE)
     }
     if(!is(fdef, "genericFunction"))
         return(FALSE)
     gen <- fdef@generic # the name with package attribute
     if(missing(f) || .identC(gen, f)) {
-	if(getName)
-	    gen
-	else
-	    TRUE
+	if(getName) gen else TRUE
     }
     else {
-        warning(gettextf("function %s appears to be a generic function, but with generic name %s",
-                         sQuote(f), sQuote(gen)),
+        warning(gettextf(
+	    "fdef appears to be a generic function, but with generic name %s instead of %s",
+			 sQuote(gen), sQuote(f)),
                 domain = NA)
         FALSE
     }
@@ -1330,8 +1330,8 @@ isSealedMethod <- function(f, signature, fdef = getGeneric(f, FALSE, where = whe
     else {
         sealed <- !is.na(match(signature[[1L]], .BasicClasses))
         if(sealed &&
-           (!is.na(match("Ops", c(f, getGroup(f, TRUE))))
-            || !is.na(match(f, c("%*%", "crossprod")))))
+           (!is.na(match("Ops", c(f, getGroup(f, TRUE)))) ||
+            !is.na(match(f, c("%*%", "crossprod", "tcrossprod")))))
             ## Ops methods are only sealed if both args are basic classes
             sealed <- sealed && (length(signature) > 1L) &&
                       !is.na(match(signature[[2L]], .BasicClasses))
@@ -1630,7 +1630,7 @@ findMethodSignatures <- function(..., target = TRUE, methods = findMethods(...))
         sigs <- lapply(methods, function(x)
                        if(is.primitive(x)) anySig else as.character(x@defined))
     }
-    lens <- unique(vapply(sigs, length, 1, USE.NAMES=FALSE))
+    lens <- unique(lengths(sigs, use.names=FALSE))
     if(length(lens) == 0)
         return(matrix(character(), 0, length(methods@arguments)))
     if(length(lens) > 1L) {
