@@ -36,17 +36,17 @@ getFunction <- function(name, generic = TRUE, mustFind = TRUE,
                         where = topenv(parent.frame()))
       ## find the object as a function.
 {
-    if(!nzchar(name))
-        stop(gettextf('expected a non-empty character string for argument name'), domain = NA)
+    if(length(name) != 1L || !nzchar(name))
+        stop(gettextf("expected a non-empty character string for argument 'name'"), domain = NA)
     found <- FALSE
     where <- as.environment(where)
     f <- NULL
     ## parent.env sequence of a namespace ends in the base package namespace,
     ## of a non-namespace ends in NULL (equiv. to base environment) [sigh]
-    lastEnv <- if(isNamespace(where)) function(where) isBaseNamespace(where) else
-    function(where) identical(where, baseenv())
+    lastEnv <- if(isNamespace(where)) function(where) isBaseNamespace(where)
+               else                   function(where) identical(where, baseenv())
     repeat {
-      if(!is.null(f <- get0(name, envir = where, mode = "function", inherits = FALSE)))
+        if(!is.null(f <- get0(name, envir = where, mode = "function", inherits = FALSE)))
             found <- generic || !is(f, "genericFunction")
         if(found || lastEnv(where))
             break
