@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
 
- *  Copyright (C) 1999-2015   The R Core Team
+ *  Copyright (C) 1999-2022   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-/* do this first to get the right options for math.h */
-#include <R_ext/Arith.h>
 
 #include <R.h>
 #include "ts.h"
@@ -33,7 +31,7 @@
 #endif
 
 // currently ISNAN includes NAs
-#define my_isok(x) (!ISNA(x) & !ISNAN(x))
+#define my_isok(x) (!ISNA(x) && !ISNAN(x))
 
 SEXP cfilter(SEXP sx, SEXP sfilter, SEXP ssides, SEXP scircular)
 {
@@ -95,6 +93,7 @@ SEXP rfilter(SEXP x, SEXP filter, SEXP out)
 
     for(R_xlen_t i = 0; i < nx; i++) {
 	sum = rx[i];
+	if (!my_isok(sum)) { r[nf + i] = NA_REAL; goto bad3; }
 	for (R_xlen_t j = 0; j < nf; j++) {
 	    tmp = r[nf + i - j - 1];
 	    if(my_isok(tmp)) sum += tmp * rf[j];
