@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2latex.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2022 The R Core Team
+#  Copyright (C) 1995-2024 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -72,24 +72,26 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                 writeLines(x, con, useBytes = TRUE, ...)
         } else {
             function(x, con, outputEncoding, ...) {
-                x <- iconv(x, "UTF-8", outputEncoding,  mark = FALSE)
-                if (anyNA(x)) {
-                    x <- iconv(x, "UTF-8", outputEncoding,
+                y <- iconv(x, "UTF-8", outputEncoding,  mark = FALSE)
+                if (anyNA(y)) {
+                    y <- iconv(x, "UTF-8", outputEncoding,
                                sub = "byte", mark = FALSE)
                     encode_warn <<- TRUE
                 }
-                writeLines(x, con, useBytes = TRUE, ...)
+                writeLines(y, con, useBytes = TRUE, ...)
             }
     }
 
-    if (concordance)
-    	conc <- activeConcordance()
-    else
-    	conc <- NULL
+    conc <- if(concordance) activeConcordance() # else NULL
     
     last_char <- ""
+    skipNewline <- FALSE
     of0 <- function(...) of1(paste0(...))
     of1 <- function(text) {
+        if (skipNewline) {
+            skipNewline <<- FALSE
+            if (text == "\n") return()
+        }
     	if (concordance)
     	    conc$addToConcordance(text)
         nc <- nchar(text)
@@ -123,14 +125,13 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
 
     startByte <- function(x) {
     	srcref <- attr(x, "srcref")
-    	if (is.null(srcref)) NA
+    	if (is.null(srcref)) -1L
     	else srcref[2L]
     }
 
     addParaBreaks <- function(x, tag) {
-        start <- startByte(x)
         if (isBlankLineRd(x)) "\n"
-	else if (identical(start, 1L)) psub("^\\s+", "", x)
+        else if (startByte(x) == 1L) psub("^\\s+", "", x)
         else x
     }
 
@@ -368,8 +369,10 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                TEXT = of1(addParaBreaks(texify(block), blocktag)),
                USERMACRO =,
                "\\newcommand" =,
-               "\\renewcommand" =,
-               COMMENT = {},
+               "\\renewcommand" = {},
+               COMMENT = if (startByte(block) == 1L ||
+                             (!inCodeBlock && last_char == "")) # indented comment line
+                             skipNewline <<- TRUE,
                LIST = writeContent(block, tag),
                ## Avoid Rd.sty's \describe, \Enumerate and \Itemize:
                ## They don't support verbatim arguments, which we might need.
@@ -404,6 +407,7 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                    inCode <<- FALSE
                },
                ## simple wrappers
+               "\\abbr" =,
                "\\acronym" =,
                "\\bold"=,
                "\\dfn"=,
@@ -442,7 +446,7 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                "\\dots" =,
                "\\ldots" = of1(if(inCode || inCodeBlock) "..."  else tag),
                "\\R" = of0(tag, "{}"),
-               "\\donttest" = writeContent(block, tag),
+               "\\donttest" =, "\\dontdiff" = writeContent(block, tag),
                "\\dontrun"= writeDR(block, tag),
                "\\enc" = {
                    ## some people put more things in \enc than a word,
@@ -567,11 +571,11 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
                               "\\arguments"={
                               	  if (concordance)
                               	      conc$saveSrcref(block[[1L]])
-                                  of1('\\item[\\code{')
+                                  of1('\\item[')
                                   inCode <<- TRUE
-                                  writeContent(block[[1L]], tag)
+                                  writeItemAsCode(tag, block[[1L]])
                                   inCode <<- FALSE
-                                  of1('}] ')
+                                  of1('] ')
                                   if (concordance)
                                       conc$saveSrcref(block[[2L]])
                                   writeContent(block[[2L]], tag)
@@ -657,6 +661,23 @@ Rd2latex <- function(Rd, out = "", defines = .Platform$OS.type,
         sectionLevel <<- save
     }
 
+    writeItemAsCode <- function(blocktag, block) {
+        ## Keep this in rsync with writeItemAsCode() in Rd2HTML.R!
+
+        ## Argh.  Quite a few packages put the items in their value
+        ## section inside \code.
+        for(i in which(RdTags(block) == "\\code"))
+            attr(block[[i]], "Rd_tag") <- "Rd"
+
+        s <- as.character.Rd(block)
+        s[s %in% c("\\dots", "\\ldots")] <- "..."
+        s <- trimws(strsplit(paste(s, collapse = ""), ",", fixed = TRUE)[[1]])
+        s <- s[nzchar(s)]
+        s <- sprintf("\\code{%s}", texify(s))
+        s <- paste0(s, collapse = ", ")
+        of1(s)
+    }
+
     Rd <- prepare_Rd(Rd, defines=defines, stages=stages, fragment=fragment, ...)
     Rdfile <- attr(Rd, "Rdfile")
     sections <- RdTags(Rd)
