@@ -1,7 +1,7 @@
 #  File src/library/tools/R/build.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2022 The R Core Team
+#  Copyright (C) 1995-2024 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ function(filename, desc = file.path(dirname(filename), "DESCRIPTION"))
                  "# Remove the previous line if you edit this file",
     		 "",
     		 "# Export all names",
-		 "exportPattern(\".\")",
+                 "exportPattern(\"^[^.]\")",
 		 if (length(pkgs))
 		     c("",
 		       "# Import all packages listed as Imports or Depends",
@@ -157,39 +157,28 @@ inRbuildignore <- function(files, pkgdir) {
             "  --md5                 add MD5 sums",
             "  --log                 log to file 'pkg-00build.log' when processing ",
             "                        the pkgdir with basename 'pkg'",
+            "  --user=               explicitly set the tarball creator name (for 'Packaged:')",
+            "                        instead of 'Sys.info()[\"user\"]' or the \"LOGNAME\" env variable",
             "",
             "Report bugs at <https://bugs.R-project.org>.", sep = "\n")
     }
 
-    add_build_stamp_to_description_file <- function(ldpath, pkgdir)
+    add_build_stamp_to_description_file <- function(ldpath, pkgdir, user)
     {
         db <- .read_description(ldpath)
         if(dir.exists(file.path(pkgdir, "src")))
             db["NeedsCompilation"] <- "yes"
         else if(is.na(db["NeedsCompilation"]))
             db["NeedsCompilation"] <- "no"
-        ## this is an optional function, so could fail
-        user <- Sys.info()["user"]
-        if(user == "unknown") user <- Sys.getenv("LOGNAME")
         db["Packaged"] <-
             sprintf("%s; %s",
                     format(Sys.time(), "%Y-%m-%d %H:%M:%S",
                            tz = 'UTC', usetz = TRUE),
                     user)
-        .write_description(db, ldpath)
-    }
-
-    ## <FIXME>
-    ## This should really be combined with
-    ## add_build_stamp_to_description_file().
-    ## Also, the build code reads DESCRIPTION files too often ...
-    add_expanded_R_fields_to_description_file <- function(ldpath) {
-        db <- .read_description(ldpath)
+        ## also add_expanded_R_fields -- when not empty:
         fields <- .expand_package_description_db_R_fields(db)
-        if(length(fields))
-            .write_description(c(db, fields), ldpath)
+        .write_description(if(length(fields)) c(db, fields) else db, ldpath)
     }
-    ## </FIXME>
 
     temp_install_pkg <- function(pkgdir, libdir) {
 	dir.create(libdir, mode = "0755", showWarnings = FALSE)
@@ -602,7 +591,7 @@ inRbuildignore <- function(files, pkgdir) {
             }
             ## </FIXME>
         }
-                    
+
 	messageLog(Log, "installing the package to process help pages")
 
         dir.create(libdir, mode = "0755", showWarnings = FALSE)
@@ -642,9 +631,9 @@ inRbuildignore <- function(files, pkgdir) {
                 unlink(build_stage23_Rd_db_path)
             saveRDS(stage23, build_stage23_Rd_db_path, version = 2L)
         }
-        
+
 	needRefman <- manual &&
-            parse_description_field(desc, "BuildManual", TRUE) &&
+            parse_description_field(desc, "BuildManual", FALSE) &&
             any(btinfo[, "later"])
 	if (needRefman) {
 	    messageLog(Log, "building the PDF package manual")
@@ -900,6 +889,7 @@ inRbuildignore <- function(files, pkgdir) {
         args <- strsplit(args,'nextArg', fixed = TRUE)[[1L]][-1L]
     }
 
+    user <- NULL
     compression <- "gzip"
     while(length(args)) {
         a <- args[1L]
@@ -949,6 +939,8 @@ inRbuildignore <- function(files, pkgdir) {
         } else if (substr(a, 1, 14) == "--compression=") {
             compression <- match.arg(substr(a, 15, 1000),
                                      c("none", "gzip", "bzip2", "xz"))
+        } else if (substr(a, 1, 7) == "--user=") {
+            user <- substr(a, 8, 64)
         } else if (startsWith(a, "-")) {
             message("Warning: unknown option ", sQuote(a))
         } else pkgs <- c(pkgs, a)
@@ -961,6 +953,10 @@ inRbuildignore <- function(files, pkgdir) {
                 domain = NA)
         compact_vignettes <-"qpdf"
     }
+    if(is.null(user)) { # not set by --user=*
+        user <- Sys.info()[["user"]]
+        if(user == "unknown") user <- Sys.getenv("LOGNAME")
+    }
 
     Sys.unsetenv("R_DEFAULT_PACKAGES")
 
@@ -1109,12 +1105,9 @@ inRbuildignore <- function(files, pkgdir) {
         ## Fix permissions for all files to be at least 644, and dirs 755
         ## Not restricted by umask.
 	if (!WINDOWS) .Call(C_dirchmod, pkgname, group.writable=FALSE)
-        ## Add build stamp to the DESCRIPTION file.
+        ## Add build stamp *and* expaned R fields to the DESCRIPTION file:
         add_build_stamp_to_description_file(file.path(pkgname, "DESCRIPTION"),
-                                            pkgdir)
-        ## Add expanded R fields to the DESCRIPTION file.
-        add_expanded_R_fields_to_description_file(file.path(pkgname,
-                                                            "DESCRIPTION"))
+                                            pkgdir, user)
         messageLog(Log,
                    "checking for LF line-endings in source and make files and shell scripts")
         fix_nonLF_in_source_files(pkgname, Log)
