@@ -85,23 +85,15 @@ read_symbols_from_object_file <- function(f)
     tab
 }
 
-get_system_ABI <- if(.Platform$OS.type == "windows") {
-    function() c(system = "windows", CC = "gcc", CXX = "g++",
-                 F77 = "gfortran", FC = "gfortran")
+## env variable formerly in etc/Renviron, now in ../Makefile
+system_ABI <- Sys.getenv("R_SYSTEM_ABI")
+if((system_ABI == "") || (substr(system_ABI, 1L, 1L) %in% c("@", "?"))) {
+    system_ABI <- character()
 } else {
-    function()
-    {
-        s <- Sys.getenv("R_SYSTEM_ABI")
-        if((s == "") || (substr(s, 1L, 1L) %in% c("@", "?")))
-            return(character())
-        s <- unlist(strsplit(s, ",", fixed = TRUE))
-        names(s) <- c("system", "CC", "CXX", "F77", "FC")
-        s
-    }
+    system_ABI <- unlist(strsplit(system_ABI, ",", fixed = TRUE))
+    names(system_ABI) <- c("system", "CC", "CXX", "F77", "FC")
 }
 
-system_ABI <- get_system_ABI()
-
 ## entry points for std::terminate are commented out as almost all
 ## come from system headers.
 so_symbol_names_table <-
@@ -127,9 +119,6 @@ so_symbol_names_table <-
       "linux, C, gcc, vsprintf, vsprintf",
 #      "linux, C, gcc, vprintf, vfprintf", # but also from REvprintf
       "linux, C, gcc, vsprintf, __vsprintf_chk",
-      "linux, C++, gxx, std::cout, _ZSt4cout",
-      "linux, C++, gxx, std::cerr, _ZSt4cerr",
-      #"linux, C++, gxx, std::terminate, _ZSt9terminatev",
       "linux, C, gcc, rand, rand",
       "linux, C, gcc, random, random",
       "linux, C, gcc, rand_r, rand_r",
@@ -137,6 +126,9 @@ so_symbol_names_table <-
       "linux, C, gcc, srandom, srandom",
       "linux, C, gcc, srandom_r, srandom_r",
       "linux, C, gcc, srand48, srand48",
+      "linux, C++, gxx, std::cout, _ZSt4cout",
+      "linux, C++, gxx, std::cerr, _ZSt4cerr",
+      #"linux, C++, gxx, std::terminate, _ZSt9terminatev",
       ## libc++ variants
       "linux, C++, gxx, std::cout, _ZNSt3__14coutE", # std::__1::cout
       "linux, C++, gxx, std::cerr, _ZNSt3__14cerrE",
@@ -159,19 +151,21 @@ so_symbol_names_table <-
       "linux, Fortran, gfortran, random_number, _gfortran_rand",
       "linux, Fortran, gfortran, random_seed, _gfortran_random_seed_i4",
       "linux, Fortran, gfortran, random_seed, _gfortran_random_seed_i8",
+      "linux, Fortran, gfortran, exit, _gfortran_exit_i4",
+      "linux, Fortran, gfortran, exit, _gfortran_exit_i8",
 
       ## Classic flang from Dec 2017 (and untested since)
-      "linux, Fortran, flang, open, f90io_open03",
-      "linux, Fortran, flang, open, f90io_open2003",
-      "linux, Fortran, flang, close, f90io_close",
-      "linux, Fortran, flang, rewind, f90io_rewind",
-      "linux, Fortran, flang, write, f90io_print_init",
-      "linux, Fortran, flang, print, f90io_print_init",
-      "linux, Fortran, flang, read, f90io_fmt_read",
-      "linux, Fortran, flang, write, f90io_fmt_write",
-      "linux, Fortran, flang, stop, f90_stop",
-      "linux, Fortran, flang, stop, f90_stop08",
-      "linux, Fortran, flang, rand, rand",
+      "linux, Fortran, ClassicFlang, open, f90io_open03",
+      "linux, Fortran, ClassicFlang, open, f90io_open2003",
+      "linux, Fortran, ClassicFlang, close, f90io_close",
+      "linux, Fortran, ClassicFlang, rewind, f90io_rewind",
+      "linux, Fortran, ClassicFlang, write, f90io_print_init",
+      "linux, Fortran, ClassicFlang, print, f90io_print_init",
+      "linux, Fortran, ClassicFlang, read, f90io_fmt_read",
+      "linux, Fortran, ClassicFlang, write, f90io_fmt_write",
+      "linux, Fortran, ClassicFlang, stop, f90_stop",
+      "linux, Fortran, ClassicFlang, stop, f90_stop08",
+      "linux, Fortran, ClassicFlang, rand, rand",
 
       ## and for for the 'flang' in the LLVM tree, currently
       ## with executable 'flang-new'.
@@ -305,6 +299,8 @@ so_symbol_names_table <-
       "macos, Fortran, gfortran, random_number, __gfortran_rand",
       "macos, Fortran, gfortran, random_seed, __gfortran_random_seed_i4",
       "macos, Fortran, gfortran, random_seed, __gfortran_random_seed_i8",
+      "macos, Fortran, gfortran, exit, __gfortran_exit_i4",
+      "macos, Fortran, gfortran, exit, __gfortran_exit_i8",
 
       ## This is old: freebsd defaults to clang these days, and
       ## gfortran and (classic) flang are available (and 'f18' will be)
@@ -443,7 +439,87 @@ so_symbol_names_table <-
       "windows, C, gcc, srand48, srand48",
       "windows, Fortran, gfortran, stop, exit",
       ## next will not show up with static libgfortran
-      "windows, Fortran, gfortran, rand, _gfortran_rand"
+      "windows, Fortran, gfortran, rand, _gfortran_rand",
+      "windows, Fortran, gfortran, random_init, _gfortran_random_init",
+      "windows, Fortran, gfortran, random_number, _gfortran_arandom_r4",
+      "windows, Fortran, gfortran, random_number, _gfortran_arandom_r8",
+      "windows, Fortran, gfortran, random_number, _gfortran_arandom_r16",
+      "windows, Fortran, gfortran, random_number, _gfortran_random_r4",
+      "windows, Fortran, gfortran, random_number, _gfortran_random_r8",
+      "windows, Fortran, gfortran, random_number, _gfortran_random_r16",
+      "windows, Fortran, gfortran, random_seed, _gfortran_random_seed_i4",
+      "windows, Fortran, gfortran, random_seed, _gfortran_random_seed_i8",
+      "windows, Fortran, gfortran, exit, _gfortran_exit_i4",
+      "windows, Fortran, gfortran, exit, _gfortran_exit_i8",
+
+      ## currently copy from Linux
+      ## flang-new but executable already named 'flang'
+      "windows, Fortran, flang, stop, _FortranAStopStatement",
+      "windows, Fortran, flang, stop, _FortranAStopStatementText",
+      "windows, Fortran, flang, open, _FortranAioBeginOpenUnit",
+      "windows, Fortran, flang, close, _FortranAioBeginClose",
+      "windows, Fortran, flang, rewind, _FortranAioBeginRewind",
+      "windows, Fortran, flang, read, _FortranAioInputAscii",
+      "windows, Fortran, flang, read, _FortranAioInputCharacter",
+      "windows, Fortran, flang, read, _FortranAioInputComplex32",
+      "windows, Fortran, flang, read, _FortranAioInputComplex64",
+      "windows, Fortran, flang, read, _FortranAioOutputExternalListInput",
+      "windows, Fortran, flang, read, _FortranAioInputInteger",
+      "windows, Fortran, flang, read, _FortranAioInputLogical",
+      "windows, Fortran, flang, read, _FortranAioInputNamelist",
+      "windows, Fortran, flang, read, _FortranAioInputReal32",
+      "windows, Fortran, flang, read, _FortranAioInputReal64",
+      "windows, Fortran, flang, read, _FortranAioInputUnformattedBlock",
+      "windows, Fortran, flang, print, _FortranAioOutputAscii",
+      "windows, Fortran, flang, print, _FortranAioOutputCharacter",
+      "windows, Fortran, flang, print, _FortranAioOutputComplex32",
+      "windows, Fortran, flang, print, _FortranAioOutputComplex64",
+      "windows, Fortran, flang, print, _FortranAioOutputExternalListOutput",
+      "windows, Fortran, flang, print, _FortranAioOutputInteger128",
+      "windows, Fortran, flang, print, _FortranAioOutputInteger16",
+      "windows, Fortran, flang, print, _FortranAioOutputInteger32",
+      "windows, Fortran, flang, print, _FortranAioOutputInteger64",
+      "windows, Fortran, flang, print, _FortranAioOutputInteger8",
+      "windows, Fortran, flang, print, _FortranAioOutputLogical",
+      "windows, Fortran, flang, print, _FortranAioOutputNamelist",
+      "windows, Fortran, flang, print, _FortranAioOutputReal32",
+      "windows, Fortran, flang, print, _FortranAioOutputReal64",
+      "windows, Fortran, flang, write, _FortranAioOutputAscii",
+      "windows, Fortran, flang, write, _FortranAioOutputCharacter",
+      "windows, Fortran, flang, write, _FortranAioOutputComplex32",
+      "windows, Fortran, flang, write, _FortranAioOutputComplex64",
+      "windows, Fortran, flang, write, _FortranAioOutputExternalListOutput",
+      "windows, Fortran, flang, write, _FortranAioOutputInteger128",
+      "windows, Fortran, flang, write, _FortranAioOutputInteger16",
+      "windows, Fortran, flang, write, _FortranAioOutputInteger32",
+      "windows, Fortran, flang, write, _FortranAioOutputInteger64",
+      "windows, Fortran, flang, write, _FortranAioOutputInteger8",
+      "windows, Fortran, flang, write, _FortranAioOutputLogical",
+      "windows, Fortran, flang, write, _FortranAioOutputNamelist",
+      "windows, Fortran, flang, write, _FortranAioOutputReal32",
+      "windows, Fortran, flang, write, _FortranAioOutputReal64",
+      "windows, Fortran, flang, write, _FortranAioOutputUnformatedBlock",
+      ## Next is a guess.
+      "windows, Fortran, flang, rand, rand_",
+      "windows, Fortran, flang, random_init, _FortranARandomInit",
+      "windows, Fortran, flang, random_number, _FortranARandomNumber",
+      "windows, Fortran, flang, random_seed, _FortranARandomSeed",
+      "windows, Fortran, flang, random_seed, _FortranARandomSeedGet",
+      "windows, Fortran, flang, random_seed, _FortranARandomSeedSize",
+
+      "windows, C++, clang++, std::cout, _ZNSt3__14coutE",
+      "windows, C++, clang++, std::cerr, _ZNSt3__14cerrE",
+      "windows, C++, clang++, std::terminate, _ZSt9terminatev",
+      "windows, C, clang, exit, exit",
+      "windows, C, clang, printf, printf",
+      "windows, C, clang, printf, puts",
+      "windows, C, clang, puts, puts",
+      "windows, C, clang, putchar, putchar",
+      "windows, C, clang, sprintf, sprintf",
+      "windows, C, clang, vprintf, vprintf",
+      "windows, C, clang, vsprintf, vsprintf",
+      "windows, C, clang, rand, rand",
+      "windows, C, clang, srand, srand"
       )
 so_symbol_names_table <-
     do.call(rbind,
@@ -489,7 +565,7 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
             "fft_factor", "fft_work", "Brent_fmin", "optif0",
 
 ## then entry points which are not attribute-hidden
-## and in a non-API header or no header at all.
+## and in a non-API header or no header at all or marked as non-API in a header
 
             "OutDec", "PRIMOFFSET", "RC_fopen", "R_CollectFromIndex",
             "R_CompiledFileName", "R_FileExists",
@@ -587,6 +663,10 @@ nonAPI <- c("chol_", "chol2inv_", "cg_", "ch_", "rg_",
 
 ## non-API and long deprecated in R_ext/RS.h (and as call_S in S.h)
             "call_R",
+## non-API, declared in Defn.h
+            "Rf_setSVector",
+## non-API, declared in Utils.h
+            "Rf_StringFalse", "Rf_StringTrue", "Rf_isBlankString",
 
 ## in the non-API header R_ext/Connections.h
             "R_new_custom_connection", "R_ReadConnection",
@@ -964,7 +1044,9 @@ function(calls, dir = NULL, character_only = TRUE)
 {
     if(!length(calls)) return(NULL)
 
-    ff_call_names <- c(".C", ".Call", ".Fortran", ".External")
+    ff_call_names <-
+        c(".C", ".Call", ".Fortran", ".External",
+          ".Call.graphics", ".External.graphics", ".External2")
     ff_call_args <- lapply(ff_call_names,
                            function(e) args(get(e, baseenv())))
     names(ff_call_args) <- ff_call_names
@@ -977,7 +1059,7 @@ function(calls, dir = NULL, character_only = TRUE)
         dir <- attr(calls, "dir")
 
     package <- # drop name
-        as.vector(.read_description(file.path(dir, "DESCRIPTION"))["Package"])
+        as.vector(.get_package_metadata(dir)["Package"])
 
     symbols <- character()
     nrdb <-
@@ -1008,6 +1090,7 @@ function(calls, dir = NULL, character_only = TRUE)
                    ## Only keep ff calls where .NAME is character
                    ## or (optionally) a name.
                    s <- e[[".NAME"]]
+                   t <- typeof(s)
                    if(is.name(s)) {
                        s <- deparse(s)[1L]
                        if(character_only) {
@@ -1035,12 +1118,12 @@ function(calls, dir = NULL, character_only = TRUE)
                         }
                    ## Could perhaps also record whether 's' was a symbol
                    ## or a character string ...
-                   cbind(cname, s, n)
+                   cbind(cname, s, n, t)
                })
     nrdb <- do.call(rbind, nrdb)
     nrdb <- as.data.frame(unique(nrdb), stringsAsFactors = FALSE)
 
-    if(NROW(nrdb) == 0L || length(nrdb) != 3L) {
+    if(NROW(nrdb) == 0L || length(nrdb) != 4L) {
         message("no native symbols were extracted")
         return(NULL)
     }
@@ -1141,8 +1224,8 @@ function(nrdb, align = TRUE, include_declarations = FALSE)
         "*/")
 
     if(include_declarations) {
-        prepare <- function(nargs)
-            if(nargs > 0) paste(rep.int("void *", nargs), collapse=", ")
+        prepare <- function(nargs, type = "void *")
+            if(nargs > 0) paste(rep.int(type, nargs), collapse=", ")
             else "void"
         decls <- c(
             "/* FIXME: ",
@@ -1155,7 +1238,7 @@ function(nrdb, align = TRUE, include_declarations = FALSE)
                   paste0("extern void ", y$s, "(", args, ");"))
            },
             if(NROW(y <- nrdb$.Call)) {
-                args <- sapply(y$n, function(n) if(n >= 0) prepare(n)
+                args <- sapply(y$n, function(n) if(n >= 0) prepare(n, "SEXP")
                                else "/* FIXME */")
                c("", "/* .Call calls */",
                   paste0("extern SEXP ", y$s, "(", args, ");"))
