@@ -1,7 +1,7 @@
 #  File src/library/utils/R/objects.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2023 The R Core Team
+#  Copyright (C) 1995-2024 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ function()
     c(names(.knownS3Generics), tools:::.get_internal_S3_generics())
 
 .S3methods <-
-function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
+function(generic.function, class, envir=parent.frame(), all.names = FALSE, dropPath = FALSE)
 {
     rbindSome <- function(df, nms, msg) {
         ## rbind.data.frame() -- dropping rows with duplicated names
@@ -91,7 +91,7 @@ function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
     sp <- search()
     if(dropPath) sp <- sp[c(1L, length(sp))]
     methods.called <- identical(sys.call(-1)[[1]], as.symbol("methods"))
-    an <- lapply(sp, ls)
+    an <- lapply(sp, ls, all.names = all.names)
     lens <- lengths(an)
     an <- unlist(an, use.names=FALSE)
     names(an) <- rep.int(sp, lens)
@@ -141,7 +141,7 @@ function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
                 genfun <- methods::finalDefaultMethod(genfun@default)
             .defenv_for_S3_registry(genfun)
         }
-	S3reg <- names(get(".__S3MethodsTable__.", envir = defenv))
+	S3reg <- names(get(".__S3MethodsTable__.", envir = defenv)) # may climb up search()
 	S3reg <- S3reg[startsWith(S3reg, paste0(generic.function,"."))]
         if(length(S3reg))
             info <- rbindSome(info, S3reg, msg =
@@ -171,8 +171,7 @@ function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
                 where <- find(nm, mode = "function")
 		if(length(where))
 		    any(vapply(where, function(w)
-			nzchar(findGeneric(nm, envir=as.environment(w))),
-			       logical(1)))
+			nzchar(findGeneric(nm, envir=as.environment(w))), NA))
 		else FALSE
 	    }, logical(1))
             info <- info[keep, ]
@@ -184,7 +183,8 @@ function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
         ## Note that the generic will not necessarily be visible,
         ## as the package may not be loaded.
         S3reg <- unlist(lapply(loadedNamespaces(), function(i)
-	    ls(get(".__S3MethodsTable__.", envir = asNamespace(i)), pattern = name)))
+	    ls(getNamespace(i)[[".__S3MethodsTable__."]],
+               pattern = name, all.names = all.names)))
         ## now methods like print.summary.aov will be picked up,
         ## so we do look for such mismatches.
         if(length(S3reg))
@@ -208,7 +208,7 @@ function(generic.function, class, envir=parent.frame(), dropPath = FALSE)
 }
 
 methods <-
-function(generic.function, class, dropPath = FALSE)
+function(generic.function, class, all.names = FALSE, dropPath = FALSE)
 {
     envir <- parent.frame()
     if(!missing(generic.function) && !is.character(generic.function)) {
@@ -227,7 +227,7 @@ function(generic.function, class, dropPath = FALSE)
     if (!missing(class) && !is.character(class))
         class <- deparse1(substitute(class))
 
-    s3 <- .S3methods(generic.function, class, envir, dropPath=dropPath)
+    s3 <- .S3methods(generic.function, class, envir, all.names=all.names, dropPath=dropPath)
     s4 <- if(.isMethodsDispatchOn()) methods::.S4methods(generic.function, class)
 
     .MethodsFunction(s3, s4, missing(generic.function))
@@ -299,7 +299,7 @@ getS3method <- function(f, class, optional = FALSE, envir = parent.frame())
 		genfun <- methods::selectMethod(genfun, "ANY")
             .defenv_for_S3_registry(genfun)
 	}
-    S3Table <- get(".__S3MethodsTable__.", envir = defenv)
+    S3Table <- get(".__S3MethodsTable__.", envir = defenv)# climb search()
     if(!is.null(m <- get0(method, envir = S3Table, inherits = FALSE)))
 	m
     else if(optional)
@@ -352,7 +352,7 @@ isS3method <- function(method, f, class, envir = parent.frame())
 		genfun <- methods::selectMethod(genfun, "ANY")
             .defenv_for_S3_registry(genfun)
 	}
-    S3Table <- get(".__S3MethodsTable__.", envir = defenv)
+    S3Table <- defenv[[".__S3MethodsTable__."]]
     ## return
     exists(method, envir = S3Table, inherits = FALSE)
 }
@@ -362,13 +362,13 @@ isS3stdGeneric <- function(f) {
     ## protect against technically valid but bizarre
     ## function(x) { { { UseMethod("gen")}}} by
     ## repeatedly consuming the { until we get to the first non { expr
-    while(is.call(bdexpr) && bdexpr[[1L]] == "{")
+    while(is.call(bdexpr) && bdexpr[[1L]] == quote(`{`))
         bdexpr <- bdexpr[[2L]]
 
     ## We only check if it is a "standard" s3 generic. i.e. the first non-{
     ## expression is a call to UseMethod. This will return FALSE if any
     ## work occurs before the UseMethod call ("non-standard" S3 generic)
-    ret <- is.call(bdexpr) && bdexpr[[1L]] == "UseMethod"
+    ret <- is.call(bdexpr) && bdexpr[[1L]] == quote(UseMethod)
     if(ret)
         names(ret) <- bdexpr[[2L]] ## arg passed to UseMethod naming generic
     ret
@@ -415,7 +415,7 @@ function(x, value)
             if(.isMethodsDispatchOn() && methods::is(genfun, "genericFunction"))
                 genfun <- methods::slot(genfun, "default")@methods$ANY
             defenv <- .defenv_for_S3_registry(genfun)
-            S3Table <- get(".__S3MethodsTable__.", envir = defenv)
+            S3Table <- defenv[[".__S3MethodsTable__."]]
             remappedName <- paste(S3[i, 1L], S3[i, 2L], sep = ".")
             if(exists(remappedName, envir = S3Table, inherits = FALSE))
                 assign(remappedName, value, S3Table)
@@ -475,7 +475,7 @@ function(x, value, ns, pos = -1, envir = as.environment(pos))
             if(.isMethodsDispatchOn() && methods::is(genfun, "genericFunction"))
                 genfun <- methods::slot(genfun, "default")@methods$ANY
             defenv <- .defenv_for_S3_registry(genfun)
-            S3Table <- get(".__S3MethodsTable__.", envir = defenv)
+            S3Table <- defenv[[".__S3MethodsTable__."]]
             remappedName <- paste(S3[i, 1L], S3[i, 2L], sep = ".")
             if(exists(remappedName, envir = S3Table, inherits = FALSE))
                 assign(remappedName, value, S3Table)
