@@ -1,7 +1,7 @@
 #  File src/library/utils/R/packages2.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2019 The R Core Team
+#  Copyright (C) 1995-2023 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ if (.Platform$OS.type == "windows")
     .install.macbinary <- function(...) NULL	# globalVariables isn't available, so use this to suppress the warning
 
 isBasePkg <- function(pkg) {
-  priority <- tryCatch(packageDescription(pkg, fields = "Priority"),
+  priority <- tryCatch(packageDescription(pkg, fields = "Priority", encoding = NA),
                        error = function(e) e, warning = function(e) e)
   identical(priority, "base")
 }
@@ -82,7 +82,12 @@ getDependencies <-
         msg3 <- c(paste0(ngettext(sum(m0),
                                   "A version of this package for your version of R might be available elsewhere,\nsee the ideas at\n",
                                   "Versions of these packages for your version of R might be available elsewhere,\nsee the ideas at\n"),
-                         "https://cran.r-project.org/doc/manuals/r-patched/R-admin.html#Installing-packages")
+                         ## refer to r-patched for released/patched versions
+                         if (grepl("Under development", R.version.string)) {
+                             "https://cran.r-project.org/doc/manuals/r-devel/R-admin.html#Installing-packages"
+                         } else {
+                             "https://cran.r-project.org/doc/manuals/r-patched/R-admin.html#Installing-packages"
+                         })
                  )
 	warning(sprintf(ngettext(sum(m0),
 				 "package %s is not available %s",
@@ -163,6 +168,8 @@ install.packages <-
              keep_outputs = FALSE,
              ...)
 {
+    if(!(is.character(type) && length(type) == 1L))
+        stop(gettextf("'%s' must be a character string", "type"), domain = NA)
     type2 <- .Platform$pkgType
     if (type == "binary") {
         if (type2 == "source")
@@ -460,6 +467,14 @@ install.packages <-
         action <- getOption("install.packages.compile.from.source",
                             "interactive")
         if(!nzchar(Sys.which(Sys.getenv("MAKE", "make")))) action <- "never"
+
+        ## Combining sources and binaries is currently broken (#18396), so
+        ## at least on macOS we want to avoid it as much as we can. If
+        ## binaries exist for all desired packages (regardless of version),
+        ## sources will be ignored.
+        if (grepl("darwin", R.version$platform) && !length(srcOnly))
+            later[later] <- FALSE
+
         if(any(later)) {
             msg <- ngettext(sum(later),
                             "There is a binary version available but the source version is later",
@@ -490,6 +505,8 @@ install.packages <-
         }
         bins <- bins[!later]
 
+        ## This is unsafe (see above), but if there is no binary, there is really no choice.
+        ## If this fails, the user can still use type='source' to recover.
         if(length(srcOnly)) {
             s2 <- srcOnly[!( available[srcOnly, "NeedsCompilation"] %in% "no" )]
             if(length(s2)) {
@@ -641,7 +658,7 @@ install.packages <-
     env <- character()
 
     tlim <- Sys.getenv("_R_INSTALL_PACKAGES_ELAPSED_TIMEOUT_")
-    tlim <- if(is.na(tlim)) 0 else tools:::get_timeout(tlim)
+    tlim <- if(!nzchar(tlim)) 0 else tools:::get_timeout(tlim)
 
     outdir <- getwd()
     if(is.logical(keep_outputs)) {
@@ -697,7 +714,7 @@ install.packages <-
                          paste(c(cmd0, args0), collapse = " ")),
                 domain = NA)
 
-    if(is.null(repos) & missing(contriburl)) {
+    if(is.null(repos) && missing(contriburl)) {
         ## install from local source tarball(s)
         update <- cbind(path.expand(pkgs), lib) # for side-effect of recycling to same length
 
@@ -736,12 +753,12 @@ install.packages <-
     }
 
     ## from here on we deal with source packages in repos
+    av2 <- NULL
     if(is.null(available)) {
         filters <- getOption("available_packages_filters")
         if(!is.null(filters)) {
             available <- available.packages(contriburl = contriburl,
                                             method = method, ...)
-            av2 <- NULL
         } else {
             f <- setdiff(available_packages_filters_default,
                          c("R_version", "duplicates"))
@@ -785,9 +802,16 @@ install.packages <-
         if (Ncpus > 1L && nrow(update) > 1L) {
             tlim_cmd <- character()
             if(tlim > 0) {
-                if(nzchar(timeout <- Sys.which("timeout"))) {
+                if(.Platform$OS.type == "windows" &&
+                   !nzchar(Sys.getenv("R_TIMEOUT")) &&
+                   grepl("\\windows\\system32\\", tolower(Sys.which("timeout")),
+                         fixed=TRUE)) {
+
+                    warning("Windows default 'timeout' command is not usable for parallel installs")
+
+                } else if(nzchar(timeout <- Sys.which(Sys.getenv("R_TIMEOUT", "timeout")))) {
                     ## SIGINT works better and is used for system.
-                    tlim_cmd <- c(shQuote(timeout), "--signal=INT", tlim)
+                    tlim_cmd <- c(shQuote(timeout), "-s INT", tlim)
                 } else
                     warning("timeouts for parallel installs require the 'timeout' command")
             }
@@ -856,17 +880,46 @@ install.packages <-
                 pkgs <- update[, 1L]
                 tss <- sub("[.]ts$", "", dir(".", pattern = "[.]ts$"))
                 failed <- pkgs[!pkgs %in% tss]
-		for (pkg in failed) system(paste0("cat ", pkg, ".out"))
-                warning(gettextf("installation of one or more packages failed,\n  probably %s",
-                                 paste(sQuote(failed), collapse = ", ")),
-                        domain = NA)
+                for (pkg in failed) {
+                    ## targets with failed dependencies are not made (even with -k)
+                    if (file.exists(outfile <- paste0(pkg, ".out")))
+                        system2("cat", outfile)
+                    ##else cat("skipped installing package ", pkg, "\n", sep = "")
+                }
+                n <- length(failed)
+                if (n == 1L)
+                    warning(gettextf("installation of package %s failed",
+                                     sQuote(failed)), domain = NA)
+                else if (n > 1L) {
+                    msg <- paste(sQuote(failed), collapse = ", ")
+                    if(nchar(msg) < 40)
+                        warning(gettextf( "installation of %d packages failed:  %s",
+                                         n, msg),
+                                domain = NA)
+                    else
+                        warning(gettextf( "installation of %d packages failed:\n  %s",
+                                         n, msg),
+                                domain = NA)
+                     }
             }
-            if(keep_outputs)
-                file.copy(paste0(update[, 1L], ".out"), outdir)
+            if(keep_outputs) {
+                outfiles <- paste0(update[, 1L], ".out") # some could be missing
+                file.copy(outfiles[file.exists(outfiles)],
+                          outdir, overwrite = TRUE)
+            }
+            ## Keep binary packages possibly created via --build
+            file.copy(Sys.glob(paste0(update[, 1L], "*.zip")), cwd)
+            file.copy(Sys.glob(paste0(update[, 1L], "*.tgz")), cwd)
+            file.copy(Sys.glob(paste0(update[, 1L], "*.tar.gz")), cwd)
             setwd(cwd); on.exit()
             unlink(tmpd2, recursive = TRUE)
         } else {
-            outfiles <- paste0(update[, 1L], ".out")
+            tmpd2 <- tempfile()
+            if(!dir.create(tmpd2))
+                stop(gettextf("unable to create temporary directory %s",
+                              sQuote(tmpd2)),
+                     domain = NA)
+            outfiles <- file.path(tmpd2, paste0(update[, 1L], ".out"))
             for(i in seq_len(nrow(update))) {
                 outfile <- if(keep_outputs) outfiles[i] else output
                 fil <- update[i, 3L]
@@ -892,10 +945,9 @@ install.packages <-
                             domain = NA)
                 }
             }
-            if(keep_outputs && (outdir != getwd())) {
-                file.copy(outfiles, outdir)
-                file.remove(outfiles)
-            }
+            if(keep_outputs)
+                file.copy(outfiles, outdir, overwrite = TRUE)
+            unlink(tmpd2, recursive = TRUE)
         }
         ## Using stderr is the wish of PR#16420
         if(!quiet && nonlocalrepos && !is.null(tmpd) && is.null(destdir))
@@ -923,10 +975,8 @@ suppressForeignCheck <- function(names, package, add = TRUE)
 
 registerNames <- function(names, package, .listFile, add = TRUE) {
     .simplePackageName <- function(env) {
-        if(exists(".packageName", envir = env, inherits = FALSE))
-           get(".packageName", envir = env)
-        else
-            "(unknown package)"
+        get0(".packageName", envir = env, inherits = FALSE,
+             ifnotfound = "(unknown package)")
     }
     if(missing(package)) {
         env <- topenv(parent.frame(2L)) # We cannot be called directly!
@@ -938,18 +988,13 @@ registerNames <- function(names, package, .listFile, add = TRUE) {
     }
     else
         env <- asNamespace(package)
-    if(exists(.listFile, envir = env, inherits = FALSE))
-        current <- get(.listFile, envir = env)
-    else
-        current <- character()
+
+    current <- get0(.listFile, envir = env, inherits = FALSE, ifnotfound = character())
     if(! missing(names)) {
         if(environmentIsLocked(env))
             stop(gettextf("The namespace for package \"%s\" is locked; no changes in the global variables list may be made.",
                           package))
-        if(add)
-            current <- unique(c(current, names))
-        else
-            current <- names
+        current <- if(add) unique(c(current, names)) else names
         assign(.listFile, current, envir = env)
     }
     current
@@ -958,11 +1003,8 @@ registerNames <- function(names, package, .listFile, add = TRUE) {
 packageName <- function(env = parent.frame()) {
     if (!is.environment(env)) stop("'env' must be an environment")
     env <- topenv(env)
-    if (!is.null(pn <- get0(".packageName", envir = env, inherits = FALSE)))
-	pn
-    else if (identical(env, .BaseNamespaceEnv))
-	"base"
-    ## else NULL
+    get0(".packageName", envir = env, inherits = FALSE) %||%
+        if(identical(env, .BaseNamespaceEnv)) "base" ## else NULL
 }
 
 ##' R's .libPaths() to be used in 'R CMD ...' or similar,
