@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 1997--2023  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2019  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -354,7 +354,7 @@ static Rboolean GetRNGkind(SEXP seeds)
 	if (seeds == R_MissingArg) /* How can this happen? */
 	    error(_("'.Random.seed' is a missing argument with no default"));
 	warning(_("'.Random.seed' is not an integer vector but of type '%s', so ignored"),
-		type2char(TYPEOF(seeds)));
+		R_typeToChar(seeds));
 	goto invalid;
     }
     is = INTEGER(seeds);
@@ -400,17 +400,22 @@ invalid:
     return TRUE;
 }
 
+static void copy_seeds_in(Int32 *i_seed, SEXP seeds, int len_seed)
+{
+    int *p = INTEGER(seeds); // will warn if R INTEGER type changes
+    memcpy(i_seed, p + 1, sizeof(Int32) * len_seed);
+}
 
-void GetRNGstate()
+void GetRNGstate(void)
 {
     /* Get  .Random.seed  into proper variables */
     int len_seed;
     SEXP seeds;
 
     seeds = GetSeedsFromVar();
-    if (seeds == R_UnboundValue) {
+    if (seeds == R_UnboundValue)
 	Randomize(RNG_kind);
-    } else {
+    else {
 	/* this might re-set the generator */
 	if(GetRNGkind(seeds)) return;
 	len_seed = RNG_Table[RNG_kind].n_seed;
@@ -420,36 +425,48 @@ void GetRNGstate()
 	if(LENGTH(seeds) == 1 && RNG_kind != USER_UNIF)
 	    Randomize(RNG_kind);
 	else {
-	    int j, *is = INTEGER(seeds);
-	    for(j = 1; j <= len_seed; j++)
-		RNG_Table[RNG_kind].i_seed[j - 1] = is[j];
+	    copy_seeds_in(RNG_Table[RNG_kind].i_seed, seeds, len_seed);
 	    FixupSeeds(RNG_kind, 0);
 	}
     }
 }
 
-void PutRNGstate()
+static R_INLINE void copy_seeds_out(SEXP seeds, Int32 *i_seed, int len_seed)
 {
-    /* Copy out seeds to  .Random.seed  */
-    int len_seed, j;
-    SEXP seeds;
+    int *p = INTEGER(seeds) + 1; // will warn if R INTEGER type changes
+    memcpy(p, i_seed, sizeof(Int32) * len_seed);
+}
 
-    if (RNG_kind > LECUYER_CMRG || N01_kind > KINDERMAN_RAMAGE || Sample_kind > REJECTION) {
+void PutRNGstate(void)
+{
+    if (RNG_kind > LECUYER_CMRG || N01_kind > KINDERMAN_RAMAGE ||
+	Sample_kind > REJECTION) {
 	warning("Internal .Random.seed is corrupt: not saving");
 	return;
     }
 
-    len_seed = RNG_Table[RNG_kind].n_seed;
+    /* Copy out seeds to  .Random.seed  */
+    int len_seed = RNG_Table[RNG_kind].n_seed;
+    int kinds = RNG_kind + 100 * N01_kind + 10000 * Sample_kind;
 
-    PROTECT(seeds = allocVector(INTSXP, len_seed + 1));
+    SEXP seeds = findVarInFrame(R_GlobalEnv, R_SeedsSymbol);
+    if (NOT_SHARED(seeds) && ATTRIB(seeds) == R_NilValue &&
+	TYPEOF(seeds) == INTSXP && XLENGTH(seeds) == len_seed + 1) {
+	/* it is safe to resuse the existing .Random.seed vector */
+	INTEGER(seeds)[0] = kinds;
+	copy_seeds_out(seeds, RNG_Table[RNG_kind].i_seed, len_seed);
+    }
+    else {
+	/* need to allocate a fresh .Random.seed vector */
+	seeds = PROTECT(allocVector(INTSXP, len_seed + 1));
+	INTEGER(seeds)[0] = kinds;
+	copy_seeds_out(seeds, RNG_Table[RNG_kind].i_seed, len_seed);
 
-    INTEGER(seeds)[0] = RNG_kind + 100 * N01_kind + 10000 * Sample_kind;
-    for(j = 0; j < len_seed; j++)
-	INTEGER(seeds)[j+1] = RNG_Table[RNG_kind].i_seed[j];
-
-    /* assign only in the workspace */
-    defineVar(R_SeedsSymbol, seeds, R_GlobalEnv);
-    UNPROTECT(1);
+	/* assign only in the workspace */
+	defineVar(R_SeedsSymbol, seeds, R_GlobalEnv);
+	INCREMENT_NAMED(seeds);
+	UNPROTECT(1);
+    }
 }
 
 static void RNGkind(RNGtype newkind)
@@ -459,8 +476,9 @@ static void RNGkind(RNGtype newkind)
  */
     if (newkind == (RNGtype)-1) newkind = RNG_DEFAULT;
     switch(newkind) {
-    case WICHMANN_HILL:
     case MARSAGLIA_MULTICARRY:
+	warning(_("RNGkind: Marsaglia-Multicarry has poor statistical properties"));
+    case WICHMANN_HILL:
     case SUPER_DUPER:
     case MERSENNE_TWISTER:
     case KNUTH_TAOCP:
@@ -487,6 +505,12 @@ static void Norm_kind(N01type kind)
 {
     /* N01type is an enumeration type, so this will probably get
        mapped to an unsigned integer type. */
+    if (kind == KINDERMAN_RAMAGE && RNG_kind == MARSAGLIA_MULTICARRY) {
+	warning(_("RNGkind: severe deviations from normality for Kinderman-Ramage + Marsaglia-Multicarry"));
+    }
+    if (kind == AHRENS_DIETER && RNG_kind == MARSAGLIA_MULTICARRY) {
+	warning(_("RNGkind: deviations from normality for Ahrens-Dieter + Marsaglia-Multicarry"));
+    }
     if (kind == (N01type)-1) kind = N01_DEFAULT;
     if (kind > KINDERMAN_RAMAGE)
 	error(_("invalid Normal type in 'RNGkind'"));
@@ -516,7 +540,7 @@ static void Samp_kind(Sampletype kind)
 
 /*------ .Internal interface ------------------------*/
 
-SEXP attribute_hidden do_RNGkind (SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_RNGkind (SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, rng, norm, sample;
 
@@ -544,7 +568,7 @@ SEXP attribute_hidden do_RNGkind (SEXP call, SEXP op, SEXP args, SEXP env)
 }
 
 
-SEXP attribute_hidden do_setseed (SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_setseed (SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP skind, nkind, sampkind;
     int seed;
@@ -652,7 +676,7 @@ MT_sgenrand(Int32 seed)
 }
 
 /* Initialization by "sgenrand()" is an example. Theoretically,
-   there are 2^19937-1 possible states as an intial state.
+   there are 2^19937-1 possible states as an initial state.
    Essential bits in "seed_array[]" is following 19937 bits:
     (seed_array[0]&UPPER_MASK), seed_array[1], ..., seed_array[N-1].
    (seed_array[0]&LOWER_MASK) is discarded.
@@ -821,9 +845,9 @@ static void RNG_Init_R_KT(Int32 seed)
 
 /* Our PRNGs have at most 32 bit of precision. All generators except
    Knuth-TAOCP, Knuth-TAOCP-2002, and possibly the user-supplied ones
-   have 31 or 32 bits bits of precision; the others are assumed to
-   have at least at least 25. */
-static R_INLINE double ru()
+   have 31 or 32 bits of precision; the others are assumed to
+   have at least 25. */
+static R_INLINE double ru(void)
 {
     double U = 33554432.0;
     return (floor(U*unif_rand()) + unif_rand())/U;
@@ -874,5 +898,5 @@ double R_unif_index(double dn)
     return dv;
 }
 
-Sampletype R_sample_kind() { return Sample_kind; }
+Sampletype R_sample_kind(void) { return Sample_kind; }
 
