@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2002--2020  The R Core Team
+ *  Copyright (C) 2002--2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Pulic License as published by
@@ -33,9 +33,10 @@
 #undef pmatch
 
 /* interval at which to check interrupts */
-#define NINTERRUPT 1000000
+/*   if re-enabling, consider a power of two */
+/* #define NINTERRUPT 1000000 */
 
-#include <R_ext/RS.h>		/* for Calloc/Free */
+#include <R_ext/RS.h>		/* for R_Calloc/R_Free */
 
 #include <wchar.h>
 #include <tre/tre.h>
@@ -94,7 +95,7 @@ amatch_regaparams(regaparams_t *params, int patlen,
     }
 }
 
-SEXP attribute_hidden do_agrep(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_agrep(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP pat, vec, ind, ans;
     SEXP opt_costs, opt_bounds;
@@ -236,7 +237,8 @@ SEXP attribute_hidden do_agrep(SEXP call, SEXP op, SEXP args, SEXP env)
 	} else {
 	    const char *s = translateChar(STRING_ELT(vec, i));
 	    if(mbcslocale && !mbcsValid(s))
-		error(_("input string %d is invalid in this locale"), i+1);
+		error(_("input string %lld is invalid in this locale"),
+		      (long long)i+1);
 	    rc = tre_regaexec(&reg, s, &match, params, 0);
 	    vmaxset(vmax);
 	}
@@ -316,7 +318,7 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
     if(opt_counts) {
 	PROTECT(counts = alloc3DArray(INTSXP, nx, ny, 3));
 	PROTECT(trafos = allocMatrix(STRSXP, nx, ny));
-	buf = Calloc(buflen, char);
+	buf = R_Calloc(buflen, char);
     }
 
     for(i = 0; i < nx; i++) {
@@ -349,14 +351,14 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
 		     */
 		    nr = nxi + 1;
 		    nc = nyj + 1;
-		    dists = Calloc(nr * nc, double);
+		    dists = R_Calloc(nr * nc, double);
 		    MAT(dists, 0, 0) = 0;
 		    for(k = 1; k < nr; k++)
 			MAT(dists, k, 0) = k * cost_del;
 		    for(l = 1; l < nc; l++)
 			MAT(dists, 0, l) = l * cost_ins;
 		    if(opt_counts) {
-			paths = Calloc(nr * nc, char);
+			paths = R_Calloc(nr * nc, char);
 			for(k = 1; k < nr; k++)
 			    MAT(paths, k, 0) = 'D';
 			for(l = 1; l < nc; l++)
@@ -405,7 +407,7 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
 			    k = nxi; l = nyj; m = k + l; nz = m;
 			    need = 2 * m + 1;
 			    if(buflen < need) {
-				buf = Realloc(buf, need , char);
+				buf = R_Realloc(buf, need , char);
 				buflen = need;
 			    }
 			    /* Need to read backwards and fill forwards. */
@@ -435,9 +437,9 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
 			    COUNTS(i, j, 2) = nsub;
 			    SET_STRING_ELT(trafos, i + nx * j, mkChar(buf));
 			}
-			Free(paths);
+			R_Free(paths);
 		    }
-		    Free(dists);
+		    R_Free(dists);
 		}
 	    }
 	}
@@ -454,7 +456,7 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
     }
 
     if(opt_counts) {
-	Free(buf);
+	R_Free(buf);
 	PROTECT(dimnames = allocVector(VECSXP, 3));
 	PROTECT(names = allocVector(STRSXP, 3));
 	SET_STRING_ELT(names, 0, mkChar("ins"));
@@ -483,7 +485,7 @@ adist_full(SEXP x, SEXP y, double *costs, Rboolean opt_counts)
 
 #define OFFSETS(I, J, K)	INTEGER(offsets)[I + J * nx + K * nxy]
 
-SEXP attribute_hidden do_adist(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_adist(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, y;
     SEXP ans, counts, offsets, dimnames, names, elt;
@@ -731,7 +733,7 @@ SEXP attribute_hidden do_adist(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
-SEXP attribute_hidden do_aregexec(SEXP call, SEXP op, SEXP args, SEXP env)
+attribute_hidden SEXP do_aregexec(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP pat, vec, ans, matchpos, matchlen;
     SEXP opt_bounds, opt_costs;
@@ -872,8 +874,8 @@ SEXP attribute_hidden do_aregexec(SEXP call, SEXP op, SEXP args, SEXP env)
 	    else {
 		t = translateChar(STRING_ELT(vec, i));
 		if(mbcslocale && !mbcsValid(t))
-		    error(_("input string %d is invalid in this locale"),
-			  i + 1);
+		    error(_("input string %lld is invalid in this locale"),
+			  (long long)i + 1);
 		rc = tre_regaexec(&reg, t,
 				  &match, params, 0);
 		vmaxset(vmax);
