@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2020  The R Core Team
+ *  Copyright (C) 2000-2023  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,45 +24,50 @@
 #include <Defn.h>
 #include <Internal.h>
 
+static SEXP checkArgIsSymbol(SEXP x) {
+    if (TYPEOF(x) != SYMSXP)
+	error("argument must be a symbol");
+    return x;
+}
+
 /* .Internal(lapply(X, FUN)) */
 
 /* This is a special .Internal, so has unevaluated arguments.  It is
-   called from a closure wrapper, so X and FUN are promises.
+   called from a closure wrapper, so X and FUN will be symbols that
+   are bound to promises in rho.
 
    FUN must be unevaluated for use in e.g. bquote .
 */
-SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    PROTECT_INDEX px;
+    PROTECT_INDEX pidx, cidx;
 
     checkArity(op, args);
     SEXP X, XX, FUN;
-    PROTECT_WITH_INDEX(X =CAR(args), &px);
+    X = checkArgIsSymbol(CAR(args));
     XX = PROTECT(eval(CAR(args), rho));
     R_xlen_t n = xlength(XX);  // a vector, so will be valid.
-    FUN = CADR(args);
+    FUN = checkArgIsSymbol(CADR(args));
     Rboolean realIndx = n > INT_MAX;
 
     SEXP ans = PROTECT(allocVector(VECSXP, n));
     SEXP names = getAttrib(XX, R_NamesSymbol);
     if(!isNull(names)) setAttrib(ans, R_NamesSymbol, names);
 
-    /* Build call: FUN(XX[[<ind>]], ...) */
-
-    SEXP ind = PROTECT(allocVector(realIndx ? REALSXP : INTSXP, 1));
+    /* Build call: FUN(X[[<ind>]], ...) */
     SEXP isym = install("i");
+    SEXP tmp = PROTECT(lang3(R_Bracket2Symbol, X, isym));
+    SEXP R_fcall = PROTECT(lang3(FUN, tmp, R_DotsSymbol));
+    MARK_NOT_MUTABLE(R_fcall);
+
+    /* Create the loop index variable and value */
+    SEXP ind = allocVector(realIndx ? REALSXP : INTSXP, 1);
+    PROTECT_WITH_INDEX(ind, &pidx);
     defineVar(isym, ind, rho);
     INCREMENT_NAMED(ind);
+    R_varloc_t loc = R_findVarLocInFrame(rho, isym);
+    PROTECT_WITH_INDEX(loc.cell, &cidx);
 
-    /* Notice that it is OK to have one arg to LCONS do memory
-       allocation and not PROTECT the result (LCONS does memory
-       protection of its args internally), but not both of them,
-       since the computation of one may destroy the other */
-
-    SEXP tmp = PROTECT(LCONS(R_Bracket2Symbol,
-			LCONS(X, LCONS(isym, R_NilValue))));
-    SEXP R_fcall = PROTECT(LCONS(FUN,
-				 LCONS(tmp, LCONS(R_DotsSymbol, R_NilValue))));
 
     for(R_xlen_t i = 0; i < n; i++) {
 	if (realIndx) REAL(ind)[0] = (double)(i + 1);
@@ -70,6 +75,14 @@ SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 	tmp = R_forceAndCall(R_fcall, 1, rho);
 	if (MAYBE_REFERENCED(tmp)) tmp = lazy_duplicate(tmp);
 	SET_VECTOR_ELT(ans, i, tmp);
+	if (ind != R_GetVarLocValue(loc) || MAYBE_SHARED(ind)) {
+	    /* ind has been captured or removed by FUN so fix it up */
+	    REPROTECT(ind = duplicate(ind), pidx);
+	    defineVar(isym, ind, rho);
+	    INCREMENT_NAMED(ind);
+	    loc = R_findVarLocInFrame(rho, isym);
+	    REPROTECT(loc.cell, cidx);
+	}
     }
 
     UNPROTECT(6);
@@ -79,7 +92,7 @@ SEXP attribute_hidden do_lapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 /* .Internal(vapply(X, FUN, FUN.VALUE, USE.NAMES)) */
 
 /* This is a special .Internal */
-SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP R_fcall, ans, names = R_NilValue, rowNames = R_NilValue,
 	X, XX, FUN, value, dim_v;
@@ -113,7 +126,7 @@ SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 	commonType != INTSXP  && commonType != LGLSXP &&
 	commonType != RAWSXP  && commonType != STRSXP &&
 	commonType != VECSXP)
-	error(_("type '%s' is not supported"), type2char(commonType));
+	error(_("type '%s' is not supported"), R_typeToChar(value));
     dim_v = getAttrib(value, R_DimSymbol);
     array_value = (TYPEOF(dim_v) == INTSXP && LENGTH(dim_v) >= 1);
     PROTECT(ans = allocVector(commonType, n*commonLen));
@@ -161,8 +174,8 @@ SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 		val = lazy_duplicate(val); // Need to duplicate? Copying again anyway
 	    PROTECT_WITH_INDEX(val, &indx);
 	    if (length(val) != commonLen)
-		error(_("values must be length %d,\n but FUN(X[[%d]]) result is length %d"),
-		       commonLen, i+1, length(val));
+		error(_("values must be length %d,\n but FUN(X[[%lld]]) result is length %d"),
+		       commonLen, (long long)i+1, length(val));
 	    valType = TYPEOF(val);
 	    if (valType != commonType) {
 		Rboolean okay = FALSE;
@@ -173,8 +186,8 @@ SEXP attribute_hidden do_vapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 		case INTSXP:  okay = (valType == LGLSXP); break;
 		}
 		if (!okay)
-		    error(_("values must be type '%s',\n but FUN(X[[%d]]) result is type '%s'"),
-			  type2char(commonType), i+1, type2char(valType));
+		    error(_("values must be type '%s',\n but FUN(X[[%lld]]) result is type '%s'"),
+			  R_typeToChar(value), (long long)i+1, R_typeToChar(val));
 		REPROTECT(val = coerceVector(val, commonType), indx);
 	    }
 	    /* Take row names from the first result only */
@@ -320,7 +333,7 @@ static SEXP do_one(SEXP X, SEXP FUN, SEXP classes, SEXP deflt,
     else return lazy_duplicate(deflt);
 }
 
-SEXP attribute_hidden do_rapply(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_rapply(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP X, FUN, classes, deflt, how, ans;
 
@@ -381,7 +394,7 @@ static int islistfactor(SEXP X)
 
 /* is this a tree with only factor leaves? */
 // currently only called from unlist()
-SEXP attribute_hidden do_islistfactor(SEXP call, SEXP op, SEXP args, SEXP rho)
+attribute_hidden SEXP do_islistfactor(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
     SEXP X = CAR(args);
