@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997--2019  The R Core Team
+ *  Copyright (C) 1997--2023  The R Core Team
  *  Copyright (C) 2003--2016  The R Foundation
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *
@@ -183,15 +183,16 @@ void formatIntegerS(SEXP x, R_xlen_t n, int *fieldwidth)
        true for non-ALTREPs or "exploded" ALTREPs
     */
     sorted = INTEGER_IS_SORTED(x);
-    /* if we're not formatting/printing the whole thing 
+    /* if we're not formatting/printing the whole thing
        ALTINTEGER_MIN/MAX will give us the wrong thing
        anyway */
     if(n == XLENGTH(x) && KNOWN_SORTED(sorted)) {
-	tmpmin = ALTINTEGER_MIN(x, TRUE);
-	tmpmax = ALTINTEGER_MAX(x, TRUE);
+	tmpmin = PROTECT(ALTINTEGER_MIN(x, TRUE));
+	tmpmax = PROTECT(ALTINTEGER_MAX(x, TRUE));
 	naflag = KNOWN_NA_1ST(sorted) ?
 	    INTEGER_ELT(x, 0) == NA_INTEGER :
 	    INTEGER_ELT(x, XLENGTH(x) - 1) == NA_INTEGER;
+	UNPROTECT(2); /* tmpmin, tmpmax */
     }
 
     /*
@@ -208,8 +209,11 @@ void formatIntegerS(SEXP x, R_xlen_t n, int *fieldwidth)
     if(tmpmin != NULL && tmpmax != NULL &&
        TYPEOF(tmpmin) == INTSXP && TYPEOF(tmpmax) == INTSXP) {
 	int l; /* only needed here so defined locally */
+	PROTECT(tmpmin);
+	PROTECT(tmpmax);
 	xmin = INTEGER_ELT(tmpmin, 0);
 	xmax = INTEGER_ELT(tmpmax, 0);
+	UNPROTECT(2); /* tmpmin, tmpmax */
 	/* naflag set above */
 
 	/* this is identical logic to what formatInteger
@@ -285,8 +289,7 @@ static void format_via_sprintf(double r, int d, int *kpower, int *nsig)
 #if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
 static const long double tbl[] =
 {
-    /* Powers exactly representable with 64 bit mantissa (except the first, which is only used with digits=0) */
-    1e-1,
+    /* Powers exactly representable with 64 bit mantissa */
     1e00, 1e01, 1e02, 1e03, 1e04, 1e05, 1e06, 1e07, 1e08, 1e09,
     1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
     1e20, 1e21, 1e22, 1e23, 1e24, 1e25, 1e26, 1e27
@@ -295,7 +298,6 @@ static const long double tbl[] =
 #else
 static const double tbl[] =
 {
-    1e-1,
     1e00, 1e01, 1e02, 1e03, 1e04, 1e05, 1e06, 1e07, 1e08, 1e09,
     1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
     1e20, 1e21, 1e22
@@ -340,7 +342,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
         long double r_prec = r;
         /* use exact scaling factor in long double precision, if possible */
         if (abs(kp) <= KP_MAX) {
-            if (kp > 0) r_prec /= tbl[kp+1]; else if (kp < 0) r_prec *= tbl[ -kp+1];
+            if (kp > 0) r_prec /= tbl[kp]; else if (kp < 0) r_prec *= tbl[ -kp];
         }
 #ifdef HAVE_POWL
 	// powl is C99 but only added to FreeBSD in 2017.
@@ -352,7 +354,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
         else
             r_prec /= Rexp10((double) kp);
 #endif
-        if (r_prec < tbl[R_print.digits]) {
+        if (r_prec < tbl[R_print.digits - 1]) {
             r_prec *= 10.0;
             kp--;
         }
@@ -364,7 +366,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
 	double r_prec = r;
         /* use exact scaling factor in double precision, if possible */
         if (abs(kp) <= KP_MAX) {
-            if (kp >= 0) r_prec /= tbl[kp+1]; else r_prec *= tbl[ -kp+1];
+            if (kp >= 0) r_prec /= tbl[kp]; else r_prec *= tbl[ -kp];
         }
         /* For IEC60559 1e-308 is not representable except by gradual underflow.
            Shifting by 303 allows for any potential denormalized numbers x,
@@ -375,7 +377,7 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
             r_prec = (r_prec * 1e+303)/Rexp10((double)(kp+303));
         else
             r_prec /= Rexp10((double)kp);
-        if (r_prec < tbl[R_print.digits]) {
+        if (r_prec < tbl[R_print.digits - 1]) {
             r_prec *= 10.0;
             kp--;
         }
@@ -408,9 +410,9 @@ scientific(const double *x, int *neg, int *kpower, int *nsig, Rboolean *rounding
 	int rgt = R_print.digits - *kpower;
 	/* bound rgt by 0 and KP_MAX */
 	rgt = rgt < 0 ? 0 : rgt > KP_MAX ? KP_MAX : rgt;
-	double fuzz = 0.5/(double)tbl[1 + rgt];
+	double fuzz = 0.5/(double)tbl[rgt];
 	// kpower can be bigger than the table.
-	*roundingwidens = *kpower > 0 && *kpower <= KP_MAX && r < tbl[*kpower + 1] - fuzz;
+	*roundingwidens = *kpower > 0 && *kpower <= KP_MAX && r < tbl[*kpower] - fuzz;
     }
 }
 
@@ -526,19 +528,22 @@ void formatRealS(SEXP x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 		      });
 }
 
+#ifdef formatComplex_tricky
 #ifdef formatComplex_USING_signif
 /*   From complex.c. */
 void z_prec_r(Rcomplex *r, const Rcomplex *x, double digits);
 #endif
+#endif
 
-/* As from 2.2.0 the number of digits applies to real and imaginary parts
-   together, not separately */
+/* From R 2.2.0 to 4.3.z, the number of digits applied to real and imaginary parts
+   together, not separately.  Since R 4.4.0, Re(.) and Im(.) are treated seperately. */
 void formatComplex(const Rcomplex *x, R_xlen_t n,
 		   int *wr, int *dr, int *er, // (w,d,e) for Re(.)
 		   int *wi, int *di, int *ei, // (w,d,e) for Im(.)
 		   int nsmall)
 {
 /* format.info() for  x[1..n] for both Re & Im */
+#ifdef formatComplex_tricky // R 4.3.z and earlier
     Rboolean all_re_zero = TRUE, all_im_zero = TRUE,
 	naflag = FALSE,
 	rnan = FALSE, rposinf = FALSE, rneginf = FALSE,
@@ -556,8 +561,8 @@ void formatComplex(const Rcomplex *x, R_xlen_t n,
 	/* Now round */
 	z_prec_r(&tmp, &(x[i]), R_print.digits);
 #else
-	tmp.r = x[i].r;
-	tmp.i = x[i].i;
+ 	tmp.r = x[i].r;
+ 	tmp.i = x[i].i;
 #endif
 	if(ISNA(tmp.r) || ISNA(tmp.i)) {
 	    naflag = TRUE;
@@ -697,12 +702,43 @@ void formatComplex(const Rcomplex *x, R_xlen_t n,
     if (inan    && *wi < 3) *wi = 3;
     if (iposinf && *wi < 3) *wi = 3;
 
-    /* finally, ensure that there is space for NA */
+#else // R >= 4.4.0 : no longer "tricky"
+    double
+	*Re = (double *) R_alloc(n, sizeof(double)),
+	*Im = (double *) R_alloc(n, sizeof(double));
 
+# ifdef formatComplex_NA_give_NA // as previously in all S and R versions:
+    Rboolean naflag = FALSE;
+    R_xlen_t i1 = 0;
+    for (R_xlen_t i = 0; i < n; i++) {
+	if(ISNA(x[i].r) || ISNA(x[i].i)) {
+	    naflag |= TRUE;
+	} else {
+	    Re[i1] =      x[i].r;
+	    Im[i1] = fabs(x[i].i); // in "Re +/- Im", the '-' does not take more space
+	    i1++;
+	} // 0 <= i1 <= i < n; i1 == length(Re) == length(Im)
+    }
+    formatReal(Re, i1, wr, dr, er, nsmall);
+    formatReal(Im, i1, wi, di, ei, nsmall);
+
+    /* finally, ensure that there is space for NA */
     if (naflag && *wr+*wi+2 < R_print.na_width)
 	*wr += (R_print.na_width -(*wr + *wi + 2));
+
+# else // even simpler: can lead to extra " " e.g. for c(NA, 1+2i)
+    for (R_xlen_t i = 0; i < n; i++) {
+	Re[i] =      x[i].r;
+	Im[i] = fabs(x[i].i); // in "Re +/- Im", the '-' does not take more space
+    }
+    formatReal(Re, n, wr, dr, er, nsmall);
+    formatReal(Im, n, wi, di, ei, nsmall);
+# endif
+#endif
 }
 
+
+
 void formatComplexS(SEXP x, R_xlen_t n, int *wr, int *dr, int *er,
 		   int *wi, int *di, int *ei, int nsmall)
 {
