@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2002-2014	The R Core Team.
+ *  Copyright (C) 2002-2024	The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@
 #include <R_ext/Linpack.h>
 
 
-/*  These get the declarations of some routines refernced here but
+/*  These get the declarations of some routines referenced here but
     not explicitly declared.    This is necessary when we link with
     a C++ compiler because the linkage changes as the declarations
     are (currently) within extern "C" blocks.
@@ -73,7 +73,7 @@ static R_CallMethodDef callMethods [] = {
     {NULL, NULL, 0}
 };
 
-
+/* F77_SYMBOL is defined in Defn.h */
 #define FDEF(name, n)  {#name, (DL_FUNC) &F77_SYMBOL(name), n, NULL}
 static R_FortranMethodDef fortranMethods[] = {
     /* LINPACK */
@@ -89,9 +89,10 @@ static R_FortranMethodDef fortranMethods[] = {
 };
 
 
-void attribute_hidden
+attribute_hidden void
 R_init_base(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, callMethods, fortranMethods, NULL);
     R_useDynamicSymbols(dll, FALSE);
+    R_forceSymbols(dll, TRUE);
 }
