@@ -1,7 +1,7 @@
 /*
   R : A Computer Language for Statistical Data Analysis
   Copyright (C) 1995-1996   Robert Gentleman and Ross Ihaka
-  Copyright (C) 1997-2015   The R Core Team
+  Copyright (C) 1997-2023   The R Core Team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,10 +26,14 @@
 #include <config.h>
 #endif
 
-#include "Defn.h"
+#include <Defn.h>
 #include "Fileio.h" /* for R_fopen */
 #include "Startup.h"
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 /* These are used in ../gnuwin32/system.c, ../unix/sys-std.c */
 SA_TYPE SaveAction = SA_SAVEASK;
 SA_TYPE	RestoreAction = SA_RESTORE;
@@ -41,7 +45,7 @@ static Rboolean DebugInitFile = FALSE;
  *  INITIALIZATION AND TERMINATION ACTIONS
  */
 
-void attribute_hidden R_InitialData(void)
+attribute_hidden void R_InitialData(void)
 {
     R_RestoreGlobalEnv();
 }
@@ -50,11 +54,14 @@ void attribute_hidden R_InitialData(void)
 attribute_hidden
 FILE *R_OpenLibraryFile(const char *file)
 {
-    char buf[PATH_MAX];
-    FILE *fp;
+    char *buf = NULL;
+    FILE *fp = NULL;
 
-    snprintf(buf, PATH_MAX, "%s/library/base/R/%s", R_Home, file);
-    fp = R_fopen(buf, "r");
+    Rasprintf_malloc(&buf, "%s/library/base/R/%s", R_Home, file);
+    if (buf) {
+	fp = R_fopen(buf, "r");
+	free(buf);
+    }
     return fp;
 }
 
@@ -69,19 +76,22 @@ char *R_LibraryFileName(const char *file, char *buf, size_t bsize)
 attribute_hidden
 FILE *R_OpenSysInitFile(void)
 {
-    char buf[PATH_MAX];
-    FILE *fp;
+    char *buf = NULL;
+    FILE *fp = NULL;
 
-    snprintf(buf, PATH_MAX, "%s/library/base/R/Rprofile", R_Home);
-    fp = R_fopen(buf, "r");
+    Rasprintf_malloc(&buf, "%s/library/base/R/Rprofile", R_Home);
+    if (buf) {
+	fp = R_fopen(buf, "r");
+	free(buf);
+    }
     return fp;
 }
 
 attribute_hidden
 FILE *R_OpenSiteFile(void)
 {
-    char buf[PATH_MAX];
-    FILE *fp;
+    char *buf = NULL;
+    FILE *fp = NULL;
 
     fp = NULL;
     if (LoadSiteFile) {
@@ -91,11 +101,19 @@ FILE *R_OpenSiteFile(void)
 	    else return NULL;
 	}
 #ifdef R_ARCH
-	snprintf(buf, PATH_MAX, "%s/etc/%s/Rprofile.site", R_Home, R_ARCH);
-	if ((fp = R_fopen(buf, "r"))) return fp;
+	Rasprintf_malloc(&buf, "%s/etc/%s/Rprofile.site", R_Home, R_ARCH);
+	if (buf) {
+	    fp = R_fopen(buf, "r");
+	    free(buf);
+	    if (fp) return fp;
+	    buf = NULL;
+	}
 #endif
-	snprintf(buf, PATH_MAX, "%s/etc/Rprofile.site", R_Home);
-	if ((fp = R_fopen(buf, "r"))) return fp;
+	Rasprintf_malloc(&buf, "%s/etc/Rprofile.site", R_Home);
+	if (buf) {
+	    fp = R_fopen(buf, "r");
+	    free(buf);
+	}
     }
     return fp;
 }
@@ -110,18 +128,28 @@ static char workspace_name[1000] = ".RData";
   drag-and-drop on Windows.
  */
 #else
-static char workspace_name[PATH_MAX] = ".RData";
+static char *workspace_name = ".RData";
 
 attribute_hidden
-void set_workspace_name(const char *fn)
+Rboolean set_workspace_name(const char *fn)
 {
-    strncpy(workspace_name, fn, PATH_MAX);
-    workspace_name[PATH_MAX - 1] = '\0';
+    static Rboolean previously_allocated = FALSE;
+    size_t needed = strlen(fn) + 1;
+    char *new_wsn = (char *)malloc(needed);
+
+    if (!new_wsn)
+	return FALSE;
+    if (previously_allocated)
+	free(workspace_name);
+    previously_allocated = TRUE;
+    strncpy(new_wsn, fn, needed);
+    workspace_name = new_wsn;
+    return TRUE;
 }
 #endif
 
 attribute_hidden
-const char* get_workspace_name()
+const char* get_workspace_name(void)
 {
     return workspace_name;
 }
@@ -143,10 +171,17 @@ void R_SaveGlobalEnv(void)
  *  INITIALIZATION HELPER CODE
  */
 
-void R_DefParams(Rstart Rp)
+/* RstartVersion is the version of the passed Rstart structure to fill in.
+   Returns 0 on success, a negative number if the version is too old,
+   a positive number if it is too new. */
+int R_DefParamsEx(Rstart Rp, int RstartVersion)
 {
+    Rp->RstartVersion = RstartVersion;
+    if (RstartVersion < 0) return -1;
+    if (RstartVersion > 1) return 1;
+    
     Rp->R_Quiet = FALSE;
-    Rp->R_Slave = FALSE;
+    Rp->R_NoEcho = FALSE;
     Rp->R_Interactive = TRUE;
     Rp->R_Verbose = FALSE;
     Rp->RestoreAction = SA_RESTORE;
@@ -159,8 +194,22 @@ void R_DefParams(Rstart Rp)
     Rp->max_vsize = R_SIZE_T_MAX;
     Rp->max_nsize = R_SIZE_T_MAX;
     Rp->ppsize = R_PPSSIZE;
+    Rp->nconnections = 128;
     Rp->NoRenviron = FALSE;
     R_SizeFromEnv(Rp);
+
+    if (RstartVersion > 0) {
+#ifdef Win32
+	Rp->EmitEmbeddedUTF8 = FALSE;
+	R_DefCallbacks(Rp, RstartVersion);
+#endif
+    }
+    return 0;
+}
+
+void R_DefParams(Rstart Rp)
+{
+    R_DefParamsEx(Rp, 0); /* version 0 for now supported */
 }
 
 #define Max_Nsize 50000000	/* about 1.4Gb 32-bit, 2.8Gb 64-bit */
@@ -176,6 +225,32 @@ void R_SizeFromEnv(Rstart Rp)
     R_size_t value;
     char *p, msg[256];
 
+    if ((p = getenv("R_MAX_VSIZE"))) {
+	value = R_Decode2Long(p, &ierr);
+	if(ierr != 0 || value > Max_Vsize)
+	    R_ShowMessage("WARNING: invalid R_MAX_VSIZE ignored\n");
+	else if(value < Min_Vsize) {
+	    snprintf(msg, 256,
+		     "WARNING: R_MAX_VSIZE smaller than Min_Vsize = %lu is ignored\n",
+		     (unsigned long) Min_Vsize);
+	    R_ShowMessage(msg);
+	}
+	else
+	    Rp->max_vsize = value;
+    }
+#if defined(__APPLE__) && defined(_SC_PHYS_PAGES) && defined(_SC_PAGE_SIZE)
+    /* For now only on macOS place a default limit on the vector heap
+       size to avoid having R killed due to memory overcommit.
+       Setting the limit at the maximum of 16Gb and available physical
+       memory seems reasonable, but there may be better options. LT */
+    else {
+	R_size_t pages = sysconf(_SC_PHYS_PAGES);
+	R_size_t page_size = sysconf(_SC_PAGE_SIZE);
+	R_size_t sysmem = pages * page_size;
+	R_size_t MinMaxVSize = 17179869184; /* 16 Gb */
+	Rp->max_vsize = sysmem > MinMaxVSize ? sysmem : MinMaxVSize;
+    }
+#endif
     if((p = getenv("R_VSIZE"))) {
 	value = R_Decode2Long(p, &ierr);
 	if(ierr != 0 || value > Max_Vsize)
@@ -234,11 +309,32 @@ static void SetSize(R_size_t vsize, R_size_t nsize)
 	R_NSize = nsize;
 }
 
+static void SetMaxSize(R_size_t vsize, R_size_t nsize)
+{
+    char msg[1024];
+
+    if (!R_SetMaxVSize(vsize)) {
+	/* vsfac is still 1 */
+	snprintf(msg, 1024,
+		 "WARNING: too small maximum for v(ector heap)size '%lu' ignored,"
+		 " the current usage %gM is already larger\n",
+		 (unsigned long) vsize, R_VSize / Mega);
+	R_ShowMessage(msg);
+    }
+
+    if (!R_SetMaxNSize(nsize)) {
+	snprintf(msg, 1024,
+		 "WARNING: too small maximum for language heap (n)size '%lu' ignored,"
+		 " the current usage '%lu' is already larger\n",
+		 (unsigned long) nsize, (unsigned long) R_NSize);
+	R_ShowMessage(msg);
+    }
+}
 
 void R_SetParams(Rstart Rp)
 {
     R_Quiet = Rp->R_Quiet;
-    R_Slave = Rp->R_Slave;
+    R_NoEcho = Rp->R_NoEcho;
     R_Interactive = Rp->R_Interactive;
     R_Verbose = Rp->R_Verbose;
     RestoreAction = Rp->RestoreAction;
@@ -247,9 +343,9 @@ void R_SetParams(Rstart Rp)
     LoadInitFile = Rp->LoadInitFile;
     DebugInitFile = Rp->DebugInitFile;
     SetSize(Rp->vsize, Rp->nsize);
-    R_SetMaxNSize(Rp->max_nsize);
-    R_SetMaxVSize(Rp->max_vsize);
+    SetMaxSize(Rp->max_vsize, Rp->max_nsize);
     R_SetPPSize(Rp->ppsize);
+    R_SetNconn(Rp->nconnections);
 #ifdef Win32
     R_SetWin32(Rp);
 #endif
