@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1997-2023   The R Core Team
+ *  Copyright (C) 1997-2024   The R Core Team
  *  Copyright (C) 1995-1996   Robert Gentleman and Ross Ihaka
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -362,6 +362,7 @@ int R_system(const char *command)
 extern char ** environ;
 #endif
 
+// .Internal(Sys.getenv(x, unset))
 attribute_hidden SEXP do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     int i, j;
@@ -376,7 +377,7 @@ attribute_hidden SEXP do_getenv(SEXP call, SEXP op, SEXP args, SEXP env)
 	error(_("wrong type for argument"));
 
     i = LENGTH(CAR(args));
-    if (i == 0) {
+    if (i == 0) { // full list of environment variables
 #ifdef Win32
 	int n = 0, N;
 	wchar_t **w;
@@ -456,36 +457,37 @@ static int Rputenv(const char *nm, const char *val)
 #endif
 
 
+// .Internal(Sys.setenv(nm, val)) : (nm_1=val_1, nm_2=val_2, ..., nm_<n>=val_<n>)
 attribute_hidden SEXP do_setenv(SEXP call, SEXP op, SEXP args, SEXP env)
 {
 #if defined(HAVE_PUTENV) || defined(HAVE_SETENV)
-    int i, n;
-    SEXP ans, nm, vars;
 
     checkArity(op, args);
 
-    if (!isString(nm = CAR(args)))
+    SEXP nm = CAR(args);
+    if (!isString(nm))
 	error(_("wrong type for argument"));
-    if (!isString(vars = CADR(args)))
+    SEXP val = CADR(args);
+    if (!isString(val))
 	error(_("wrong type for argument"));
-    if(LENGTH(nm) != LENGTH(vars))
-	error(_("wrong length for argument"));
+    if(LENGTH(nm) != LENGTH(val))
+	error(_("'%s' and '%s' are of different lengths"), "names", "val");
 
-    n = LENGTH(vars);
-    PROTECT(ans = allocVector(LGLSXP, n));
+    int i, n = LENGTH(val);
+    SEXP ans = PROTECT(allocVector(LGLSXP, n));
 #ifdef HAVE_SETENV
     for (i = 0; i < n; i++)
 	LOGICAL(ans)[i] = setenv(translateChar(STRING_ELT(nm, i)),
-				 translateChar(STRING_ELT(vars, i)),
+				 translateChar(STRING_ELT(val, i)),
 				 1) == 0;
 #elif defined(Win32)
     for (i = 0; i < n; i++)
 	LOGICAL(ans)[i] = Rwputenv(wtransChar(STRING_ELT(nm, i)),
-				   wtransChar(STRING_ELT(vars, i))) == 0;
+				   wtransChar(STRING_ELT(val, i))) == 0;
 #else
     for (i = 0; i < n; i++)
 	LOGICAL(ans)[i] = Rputenv(translateChar(STRING_ELT(nm, i)),
-				  translateChar(STRING_ELT(vars, i))) == 0;
+				  translateChar(STRING_ELT(val, i))) == 0;
 #endif
     UNPROTECT(1);
     return ans;
@@ -495,30 +497,29 @@ attribute_hidden SEXP do_setenv(SEXP call, SEXP op, SEXP args, SEXP env)
 #endif
 }
 
+// .Internal(Sys.unsetenv(nm))
 attribute_hidden SEXP do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    int i, n;
-    SEXP ans, vars;
-
     checkArity(op, args);
 
-    if (!isString(vars = CAR(args)))
+    SEXP nm = CAR(args);
+    if (!isString(nm))
 	error(_("wrong type for argument"));
-    n = LENGTH(vars);
+    int i, n = LENGTH(nm);
 
 #if defined(HAVE_UNSETENV) || defined(HAVE_PUTENV_UNSET) || defined(HAVE_PUTENV_UNSET2)
 #ifdef HAVE_UNSETENV
-    for (i = 0; i < n; i++) unsetenv(translateChar(STRING_ELT(vars, i)));
+    for (i = 0; i < n; i++) unsetenv(translateChar(STRING_ELT(nm, i)));
 #elif defined(HAVE_PUTENV_UNSET)
     for (i = 0; i < n; i++) {
 	char buf[1000];
-	snprintf(buf, 1000, "%s",  translateChar(STRING_ELT(vars, i)));
+	snprintf(buf, 1000, "%s",  translateChar(STRING_ELT(nm, i)));
 	putenv(buf);
     }
 #elif defined(HAVE_PUTENV_UNSET2)
 # ifdef Win32
     for (i = 0; i < n; i++) {
-	const wchar_t *w = wtransChar(STRING_ELT(vars, i));
+	const wchar_t *w = wtransChar(STRING_ELT(nm, i));
 	wchar_t buf[2*wcslen(w)];
 	wcscpy(buf, w);
 	wcscat(buf, L"=");
@@ -527,7 +528,7 @@ attribute_hidden SEXP do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
 # else
     for (i = 0; i < n; i++) {
 	char buf[1000];
-	snprintf(buf, 1000, "%s=", translateChar(STRING_ELT(vars, i)));
+	snprintf(buf, 1000, "%s=", translateChar(STRING_ELT(nm, i)));
 	putenv(buf);
     }
 # endif
@@ -535,12 +536,12 @@ attribute_hidden SEXP do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
 
 #elif defined(HAVE_PUTENV) || defined(HAVE_SETENV)
     warning(_("this system cannot unset environment variables: setting to \"\""));
-    n = LENGTH(vars);
+    n = LENGTH(nm);
     for (i = 0; i < n; i++) {
 #ifdef HAVE_SETENV
-	setenv(translateChar(STRING_ELT(vars, i)), "", 1);
+	setenv(translateChar(STRING_ELT(nm, i)), "", 1);
 #else
-	Rputenv(translateChar(STRING_ELT(vars, i)), "");
+	Rputenv(translateChar(STRING_ELT(nm, i)), "");
 #endif
     }
 
@@ -548,9 +549,9 @@ attribute_hidden SEXP do_unsetenv(SEXP call, SEXP op, SEXP args, SEXP env)
     warning(_("'Sys.unsetenv' is not available on this system"));
 #endif
 
-    PROTECT(ans = allocVector(LGLSXP, n));
+    SEXP ans = PROTECT(allocVector(LGLSXP, n));
     for (i = 0; i < n; i++)
-	LOGICAL(ans)[i] = !getenv(translateChar(STRING_ELT(vars, i)));
+	LOGICAL(ans)[i] = !getenv(translateChar(STRING_ELT(nm, i)));
     UNPROTECT(1);
     return ans;
 }
@@ -579,6 +580,15 @@ write_one (unsigned int namescount, const char * const *names, void *data)
 }
 #endif
 
+// Copied from platform.c (only used condiitonally there)
+// case-insensitive string comparison
+int static R_strieql(const char *a, const char *b)
+{
+    while (*a && *b && toupper(*a) == toupper(*b)) { a++; b++; }
+    return (*a == 0 && *b == 0);
+}
+
+
 #include "RBufferUtils.h"
 
 /* iconv(x, from, to, sub, mark) */
@@ -632,14 +642,24 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	toRaw = asLogical(CAR(args));
 	if(toRaw == NA_LOGICAL)
 	    error(_("invalid '%s' argument"), "toRaw");
-	/* some iconv's allow "UTF8", but libiconv does not */
-	if(streql(from, "UTF8") || streql(from, "utf8") ) from = "UTF-8";
-	if(streql(to, "UTF8") || streql(to, "utf8") ) to = "UTF-8";
+	/* some iconv's allow "UTF8", but GNU libiconv does not */
+	if(R_strieql(from, "UTF8")) from = "UTF-8";
+	if(R_strieql(to, "UTF8")) to = "UTF-8";
 	if(streql(to, "UTF-8")) isUTF8 = TRUE;
-	if(streql(to, "latin1") || streql(to, "ISO_8859-1")
-	    || streql(to, "CP1252")) isLatin1 = TRUE;
+	if(R_strieql(to, "latin1") || R_strieql(to, "ISO_8859-1")
+	    || R_strieql(to, "CP1252")) isLatin1 = TRUE;
 	if(streql(to, "") && known_to_be_latin1) isLatin1 = TRUE;
 	if(streql(to, "") && known_to_be_utf8) isUTF8 = TRUE;
+#ifdef OS_MUSL
+	if(R_strieql(from, "latin-2") || R_strieql(from, "latin2") )
+	    from = "iso88592";
+	if(R_strieql(to, "latin-2") || R_strieql(to, "latin2") )
+	    to = "iso88592";
+	if(R_strieql(from, "latin-9") || R_strieql(from, "latin9") )
+	    from = "iso885915";
+	if(R_strieql(to, "latin-9") || R_strieql(to, "latin9") )
+	    to = "iso885915";
+#endif
 	isRawlist = (TYPEOF(x) == VECSXP);
 	if(isRawlist) {
 	    if(toRaw)
@@ -771,6 +791,16 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 	    } else if(res == -1 && sub &&
 		      (errno == EILSEQ || errno == EINVAL)) {
 		/* it seems this gets thrown for non-convertible input too */
+		/* 
+		   Should re-set with a stateful encoding, but some iconv
+                   implementations forget byte-order learned from BOM.
+
+		res = Riconv(obj, NULL, NULL, &outbuf, &outb);	
+		if (res == -1 && errno == E2BIG) {
+		    R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
+		    goto top_of_loop;
+		}
+		*/
 		if(fromUTF8 && streql(sub, "Unicode")) {
 		    if(outb < 13) {
 			R_AllocStringBuffer(2*cbuff.bufsize, &cbuff);
@@ -786,16 +816,18 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			    ucs = (R_wchar_t) wc;
 			inbuf += clen; inb -= clen;
 			if(ucs < 65536) {
-			    // gcc 7 objects to this with unsigned int
+			    // gcc 7 objected to this with unsigned int
 			    snprintf(outbuf, 9, "<U+%04X>", (unsigned short) ucs);
 			    outbuf += 8; outb -= 8;
 			} else {
 			    /* R_wchar_t is unsigned int on Windows, 
 			       otherwise wchar_t (usually int).
 			       In any case Unicode points <= 0x10FFFF
+			       so one could argue against zero-padding here.
 			    */
-			    snprintf(outbuf, 13, "<U+%08X>", (unsigned int) ucs);
-			    outbuf += 12; outb -= 12;
+			    snprintf(outbuf, 13, "<U+%04X>", (unsigned int) ucs);
+			    size_t l = strlen(outbuf);
+			    outbuf += l; outb -= l;
 			}
 		    }
 		    goto next_char;
@@ -814,7 +846,7 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
 			    ucs = (R_wchar_t) wc;
 			inbuf += clen; inb -= clen;
 			if(ucs < 65536) {
-			    // gcc 7 objects to this with unsigned int
+			    // gcc 7 objected to this with unsigned int
 			    snprintf(outbuf, 7, "\\u%04x", (unsigned short) ucs);
 			    outbuf += 6; outb -= 6;
 			} else {
@@ -882,7 +914,7 @@ attribute_hidden SEXP do_iconv(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP __x__ = (x);            \
     if(TYPEOF(__x__) != CHARSXP) \
 	error(_("'%s' must be called on a CHARSXP, but got '%s'"), \
-	      __func__, type2char(TYPEOF(__x__)));                 \
+	      __func__, R_typeToChar(__x__));			   \
 } while(0);
 
 cetype_t getCharCE(SEXP x)
@@ -932,7 +964,7 @@ size_t Riconv (void *cd, const char **inbuf, size_t *inbytesleft,
 {
     /* here libiconv has const char **, glibc has char ** for inbuf */
     return iconv((iconv_t) cd, (ICONV_CONST char **) inbuf, inbytesleft,
-		 outbuf, outbytesleft);
+                outbuf, outbytesleft);
 }
 
 int Riconv_close (void *cd)
@@ -1032,7 +1064,8 @@ next_char:
 	R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	goto top_of_loop;
     } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
-	if(outb < 13) {
+	res = Riconv(obj, NULL, NULL, &outbuf, &outb);
+	if((res == -1 && errno == E2BIG) || outb < 13) {
 	    R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	    goto top_of_loop;
 	}
@@ -1240,7 +1273,8 @@ next_char:
 	R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	goto top_of_loop;
     } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
-	if(outb < 5) {
+	res = Riconv(obj, NULL, NULL, &outbuf, &outb);
+	if((res == -1 && errno == E2BIG) || outb < 5) {
 	    R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	    goto top_of_loop;
 	}
@@ -1432,7 +1466,8 @@ next_char:
 	R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	goto top_of_loop;
     } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
-	if(outb < 5 * sizeof(wchar_t)) {
+	res = Riconv(obj, NULL, NULL, &outbuf, &outb);
+	if((res == -1 && errno == E2BIG) || outb < 5 * sizeof(wchar_t)) {
 	    R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	    goto top_of_loop;
 	}
@@ -1531,6 +1566,11 @@ next_char:
 	R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
 	goto top_of_loop;
     } else if(res == -1 && (errno == EILSEQ || errno == EINVAL)) {
+	res = Riconv(obj, NULL, NULL, &outbuf, &outb);
+	if(res == -1 && errno == E2BIG) {
+	    R_AllocStringBuffer(2*cbuff->bufsize, cbuff);
+	    goto top_of_loop;
+	}
 	size_t inb_per_char = fromWchar ? sizeof(wchar_t) : 1;
 
 	/* ensure space in cbuff for substitution */	
@@ -2280,7 +2320,7 @@ attribute_hidden SEXP do_glob(SEXP call, SEXP op, SEXP args, SEXP env)
     {
 	wchar_t *w = globbuf.gl_pathv[i];
 	char *buf;
-	int nb = wcstoutf8(NULL, w, INT_MAX);
+	size_t nb = wcstoutf8(NULL, w, (size_t)INT_MAX + 2);
 	buf = R_AllocStringBuffer(nb, &cbuff);
 	wcstoutf8(buf, w, nb);
 	SET_STRING_ELT(ans, i, mkCharCE(buf, CE_UTF8));
