@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C)  2001-2020   The R Core Team.
+ *  Copyright (C)  2001-2023   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,19 +29,9 @@
 #define R_USE_SIGNALS 1
 #include <Defn.h>
 #include <Rconnections.h>
-#include <R_ext/R-ftp-http.h>
 #include "sock.h"
 #include <errno.h>
 
-#ifdef Win32
-# ifndef EINTR
-#  define EINTR                   WSAEINTR
-# endif
-# ifndef EWOULDBLOCK
-#  define EWOULDBLOCK             WSAEWOULDBLOCK
-# endif
-#endif
-
 static void listencleanup(void *data)
 {
     int *psock = data;
@@ -65,6 +55,14 @@ static Rboolean sock_open(Rconnection con)
 		warning("port %d cannot be opened", this->port);
 		return FALSE;
 	    }
+#ifdef Unix
+	    if (sock1 >= FD_SETSIZE) {
+		/* R_SockListen below would fail */
+		R_SockClose(sock1);
+		warning(_("file descriptor is too large for select()"));
+		return FALSE;
+	    }
+#endif
 	    {
 		RCNTXT cntxt;
 
@@ -91,20 +89,32 @@ static Rboolean sock_open(Rconnection con)
 		return FALSE;
 	    }
 	}
+#ifdef Unix
+	if (sock >= FD_SETSIZE && (con->canwrite || con->blocking)) {
+	    /* Reading/writing via such socket would fail */
+	    R_SockClose(sock);
+	    warning(_("file descriptor is too large for select()"));
+	    return FALSE;
+	}
+#endif
 	free(con->description);
-	con->description = (char *) malloc(strlen(buf) + 10);
-	sprintf(con->description, "<-%s:%d", buf, this->port);
+	size_t sz = strlen(buf) + 10;
+	con->description = (char *) malloc(sz); // FIXME check allocation 
+	snprintf(con->description, sz, "<-%s:%d", buf, this->port);
     } else {
 	sock = R_SockConnect(this->port, con->description, timeout);
 	if(sock < 0) {
 	    warning("%s:%d cannot be opened", con->description, this->port);
 	    return FALSE;
 	}
-	sprintf(buf, "->%s:%d", con->description, this->port);
+	snprintf(buf, 256, "->%s:%d", con->description, this->port);
 	strcpy(con->description, buf);
     }
     this->fd = sock;
 
+    if (this->options & RSC_SET_TCP_NODELAY)
+	R_set_nodelay(sock);
+
     mlen = (int) strlen(con->mode);
     con->isopen = TRUE;
     if(mlen >= 2 && con->mode[mlen - 1] == 'b') con->text = FALSE;
@@ -142,10 +152,11 @@ static ssize_t sock_read_helper(Rconnection con, void *ptr, size_t size)
 	    do
 		res = R_SockRead(this->fd, this->inbuf, 4096, 
 				 con->blocking, this->timeout);
-	    while (-res == EINTR);
 #ifdef Win32
-	    if (! con->blocking && -res == EAGAIN) {
+	    while (-res == WSAEINTR);
+	    if (! con->blocking && -res == WSAEWOULDBLOCK) {
 #else
+	    while (-res == EINTR);
 	    if (! con->blocking && (-res == EAGAIN || -res == EWOULDBLOCK)) {
 #endif
 		con->incomplete = TRUE;
@@ -193,13 +204,13 @@ static size_t sock_write(const void *ptr, size_t size, size_t nitems,
 			 Rconnection con)
 {
     Rsockconn this = (Rsockconn)con->private;
-    ssize_t n = R_SockWrite(this->fd, ptr, (int)(size * nitems),
+    ssize_t n = R_SockWrite(this->fd, ptr, (size_t)(size * nitems),
 			    this->timeout)/((ssize_t)size);
     return n > 0 ? n : 0;
 }
 
 Rconnection in_R_newsock(const char *host, int port, int server, int serverfd,
-			 const char * const mode, int timeout)
+			 const char * const mode, int timeout, int options)
 {
     Rconnection new;
 
@@ -236,6 +247,7 @@ Rconnection in_R_newsock(const char *host, int port, int server, int serverfd,
     ((Rsockconn)new->private)-> server = server;
     ((Rsockconn)new->private)-> timeout = timeout;
     ((Rsockconn)new->private)-> serverfd = serverfd;
+    ((Rsockconn)new->private)-> options = options;
     return new;
 }
 
@@ -276,6 +288,15 @@ Rconnection in_R_newservsock(int port)
 	      port);
 	/* for Solaris 12.5 */ new = NULL;
     }
+#ifdef Unix
+    if (sock >= FD_SETSIZE) {
+	/* R_SockListen (accept) called from sock_open would fail */
+	free(new->private); free(new->description); free(new->class); free(new);
+	R_SockClose(sock);
+	error(_("file descriptor is too large for select()"));
+	/* for Solaris 12.5 */ new = NULL;
+    }
+#endif
     ((Rservsockconn)new->private)-> fd = sock;
     new->isopen = TRUE;
 
