@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001--2019  The R Core Team.
+ *  Copyright (C) 2001--2023  The R Core Team.
  *  Copyright (C) 2003--2010  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <ctype.h>  /* for toupper */
 #include <limits.h> /* for PATH_MAX */
 #include <stdlib.h> /* for realpath */
+#include <string.h> /* for strcpy */
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h> /* for realpath on some systems */
@@ -62,8 +63,7 @@ static char La_norm_type(const char *typstr)
     char typup;
 
     if (strlen(typstr) != 1)
-	error(
-	    _("argument type[1]='%s' must be a character string of string length 1"),
+	error(_("argument type[1]='%s' must be a character string of string length 1"),
 	    typstr);
     typup = (char) toupper(*typstr);
     if (typup == '1')
@@ -93,11 +93,23 @@ static char La_rcond_type(const char *typstr)
     return typup; /* 'O' or 'I' */
 }
 
+/* Lapack valid 'uplo' (upper/lower) for triangular/symmetric matrices: */
+static char La_valid_uplo(const char *uplostr)
+{
+    if (strlen(uplostr) != 1)
+	error(_("argument type[1]='%s' must be a character string of string length 1"),
+	      uplostr);
+    char uplo = (char) toupper(*uplostr);
+    if (uplo != 'U' && uplo != 'L')
+	error(_("argument type[1]='%s' must be 'U' or 'L'"), uplostr);
+    return uplo; /* 'U' or 'L' */
+}
+
+
 /* La.svd, called from svd */
 static SEXP La_svd(SEXP jobu, SEXP x, SEXP s, SEXP u, SEXP vt)
 {
-    if (!isString(jobu))
-	error("'jobu' must be a character string");
+    if (!isString(jobu)) error(_("'%s' must be a character string"), "jobu");
     int *xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP)),
 	n = xdims[0], p = xdims[1], nprot = 2;
 
@@ -336,9 +348,8 @@ static SEXP La_dlange(SEXP A, SEXP type)
     double *work = NULL;
     char typNorm[] = {'\0', '\0'};
 
-    if (!isMatrix(A)) error(_("'A' must be a numeric matrix"));
-    if (!isString(type))
-	error(_("'type' must be a character string"));
+    if (!isMatrix(A)) error(_("'%s' must be a numeric matrix"), "A");
+    if (!isString(type)) error(_("'%s' must be a character string"), "type");
     if (!isReal(A)) {
 	A = PROTECT(coerceVector(A, REALSXP)); nprot++;
     }
@@ -366,9 +377,8 @@ static SEXP La_dgecon(SEXP A, SEXP norm)
     double anorm, *work;
     char typNorm[] = {'\0', '\0'};
 
-    if (!isMatrix(A)) error(_("'A' must be a numeric matrix"));
-    if (!isString(norm))
-	error(_("'norm' must be a character string"));
+    if (!isMatrix(A)) error(_("'%s' must be a numeric matrix"), "A");
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
     A = PROTECT(isReal(A) ? duplicate(A) : coerceVector(A, REALSXP));
 
     xdims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
@@ -388,7 +398,7 @@ static SEXP La_dgecon(SEXP A, SEXP norm)
     if (info) {
 	if (info < 0) {
 	    UNPROTECT(2);
-	    error(_("error [%d] from Lapack 'dgetrf()'"), info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "dgetrf()");
 	}
 	else { /* i := info > 0:  LU decomp. is completed, but  U[i,i] = 0
 		* <==> singularity */
@@ -405,18 +415,58 @@ static SEXP La_dgecon(SEXP A, SEXP norm)
 		     /* rcond = */ REAL(val),
 		     work, iwork, &info FCONE);
     UNPROTECT(2);
-    if (info) error(_("error [%d] from Lapack 'dgecon()'"), info);
+    if (info) error(_("error code %d from Lapack routine '%s'"), info, "dgecon()");
+
+    return val;
+}
+
+/* Real case of kappa.tri (and also rcond for a triangular matrix), both
+ * uplo = "U" or uplo = "L" */
+static SEXP La_dtrcon3(SEXP A, SEXP norm, SEXP uplo)
+{
+    int *xdims, n, nprot = 0, info;
+    char typNorm[] = {'\0', '\0'};
+    char uploC[]   = {'\0', '\0'};
+
+    if (!isMatrix(A)) error(_("'%s' must be a numeric matrix"), "A");
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
+    if (!isString(uplo)) error(_("'%s' must be a character string"), "uplo");
+    if (!isReal(A)) {
+	nprot++;
+	A = PROTECT(coerceVector(A, REALSXP));
+    }
+    xdims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
+    n = xdims[0];
+    if(n != xdims[1]) {
+	UNPROTECT(nprot);
+	error(_("'A' must be a *square* matrix"));
+    }
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+    uploC  [0] = La_valid_uplo(CHAR(asChar(uplo)));
+
+    nprot++;
+    SEXP val = PROTECT(allocVector(REALSXP, 1));
+
+    F77_CALL(dtrcon)(typNorm, uploC, "N", &n, REAL(A), &n,
+		     REAL(val),
+		     /* work : */ (double *) R_alloc(3*(size_t)n, sizeof(double)),
+		     /* iwork: */ (int *)    R_alloc(n, sizeof(int)),
+		     &info FCONE FCONE FCONE);
+    UNPROTECT(nprot);
+    if (info) error(_("error [%d] from Lapack 'dtrcon()'"), info);
     return val;
 }
 
 /* Real case of kappa.tri (and also rcond for a triangular matrix) */
+/* hardcoded  uplo = "U" -- using only upper triangular entries */
 static SEXP La_dtrcon(SEXP A, SEXP norm)
 {
     int *xdims, n, nprot = 0, info;
     char typNorm[] = {'\0', '\0'};
 
-    if (!isMatrix(A)) error(_("'A' must be a numeric matrix"));
-    if (!isString(norm)) error(_("'norm' must be a character string"));
+    if (!isMatrix(A)) error(_("'%s' must be a numeric matrix"), "A");
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
     if (!isReal(A)) {
 	nprot++;
 	A = PROTECT(coerceVector(A, REALSXP));
@@ -439,10 +489,43 @@ static SEXP La_dtrcon(SEXP A, SEXP norm)
 		     /* iwork: */ (int *)    R_alloc(n, sizeof(int)),
 		     &info FCONE FCONE FCONE);
     UNPROTECT(nprot);
-    if (info) error(_("error [%d] from Lapack 'dtrcon()'"), info);
+    if (info) error(_("error code %d from Lapack routine '%s'"), info, "dtrcon()");
+
     return val;
 }
 
+/* norm(<Complex>) except for 2-norm */
+static SEXP La_zlange(SEXP A, SEXP type)
+{
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
+    int *xdims, m, n;
+    double *work = NULL;
+    char typNorm[] = {'\0', '\0'};
+
+    if (!(isMatrix(A) && isComplex(A)))
+	error(_("'%s' must be a complex matrix"), "A");
+    if (!isString(type)) error(_("'%s' must be a character string"), "type");
+
+    xdims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
+    m = xdims[0];
+    n = xdims[1]; /* m x n  matrix {using Lapack naming convention} */
+
+    typNorm[0] = La_norm_type(CHAR(asChar(type)));
+
+    SEXP val = PROTECT(allocVector(REALSXP, 1));
+    if(*typNorm == 'I') work = (double *) R_alloc((size_t)m, sizeof(Rcomplex));
+    REAL(val)[0] = F77_CALL(zlange)(typNorm, &m, &n, COMPLEX(A), &m, work FCONE);
+
+    UNPROTECT(1);
+    return val;
+
+#else
+    error(_("Fortran complex functions are not available on this platform"));
+    return R_NilValue; /* -Wall */
+#endif
+}
+
+
 /* Complex case of rcond, for general matrices */
 static SEXP La_zgecon(SEXP A, SEXP norm)
 {
@@ -452,9 +535,9 @@ static SEXP La_zgecon(SEXP A, SEXP norm)
     double anorm, *rwork;
     char typNorm[] = {'\0', '\0'};
 
-    if (!isString(norm)) error(_("'norm' must be a character string"));
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
     if (!(isMatrix(A) && isComplex(A)))
-	error(_("'A' must be a complex matrix"));
+	error(_("'%s' must be a complex matrix"), "A");
     dims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     n = dims[0];
     if(n != dims[1]) error(_("'A' must be a *square* matrix"));
@@ -476,14 +559,12 @@ static SEXP La_zgecon(SEXP A, SEXP norm)
     if (info) {
 	if (info < 0) {
 	    UNPROTECT(1);
-	    error(_("error [%d] from Lapack 'zgetrf()'"), info);
+	    error(_("error code %d from Lapack routine '%s'"), info, "zgetrf()");
 	} else {
 	    REAL(val)[0] = 0.; /* rcond = 0 <==> singularity */
 	    UNPROTECT(1);
 	    return val;
 	}
-	UNPROTECT(1);
-	error(_("error [%d] from Lapack 'zgetrf()'"), info);
     }
 
     F77_CALL(zgecon)(typNorm, &n, avals, &n, &anorm,
@@ -491,7 +572,7 @@ static SEXP La_zgecon(SEXP A, SEXP norm)
 		     /* work : */ (Rcomplex *) R_alloc(4*(size_t)n, sizeof(Rcomplex)),
 		     rwork, &info FCONE);
     UNPROTECT(1);
-    if (info) error(_("error [%d] from Lapack 'zgecon()'"), info);
+    if (info) error(_("error code %d from Lapack routine '%s'"), info, "zgecon()");
     return val;
 
 #else
@@ -501,6 +582,7 @@ static SEXP La_zgecon(SEXP A, SEXP norm)
 }
 
 /* Complex case of kappa.tri (and also rcond for a triangular matrix) */
+/* hardcoded  uplo = "U" -- using only upper triangular entries */
 static SEXP La_ztrcon(SEXP A, SEXP norm)
 {
 #ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
@@ -508,10 +590,9 @@ static SEXP La_ztrcon(SEXP A, SEXP norm)
     int *dims, n, info;
     char typNorm[] = {'\0', '\0'};
 
-    if (!isString(norm))
-	error(_("'norm' must be a character string"));
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
     if (!(isMatrix(A) && isComplex(A)))
-	error(_("'A' must be a complex matrix"));
+	error(_("'%s' must be a complex matrix"), "A");
     dims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     n = dims[0];
     if(n != dims[1])
@@ -527,6 +608,43 @@ static SEXP La_ztrcon(SEXP A, SEXP norm)
 		     /* rwork: */ (double *)   R_alloc(n, sizeof(double)),
 		     &info FCONE FCONE FCONE);
     UNPROTECT(1);
+    if (info) error(_("error code %d from Lapack routine '%s'"), info, "ztrcon()");
+    return val;
+#else
+    error(_("Fortran complex functions are not available on this platform"));
+    return R_NilValue; /* -Wall */
+#endif
+} // La_ztrcon()
+
+/* Complex case of kappa.tri (and also rcond for a triangular matrix) */
+static SEXP La_ztrcon3(SEXP A, SEXP norm, SEXP uplo)
+{
+#ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
+    SEXP val;
+    int *dims, n, info;
+    char typNorm[] = {'\0', '\0'};
+    char uploC[]   = {'\0', '\0'};
+
+    if (!(isMatrix(A) && isComplex(A)))
+	error(_("'%s' must be a complex matrix"), "A");
+    if (!isString(norm)) error(_("'%s' must be a character string"), "norm");
+    if (!isString(uplo)) error(_("'%s' must be a character string"), "uplo");
+    dims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
+    n = dims[0];
+    if(n != dims[1])
+	error(_("'A' must be a *square* matrix"));
+
+    typNorm[0] = La_rcond_type(CHAR(asChar(norm)));
+    uploC  [0] = La_valid_uplo(CHAR(asChar(uplo)));
+
+    val = PROTECT(allocVector(REALSXP, 1));
+
+    F77_CALL(ztrcon)(typNorm, uploC, "N", &n, COMPLEX(A), &n,
+		     REAL(val),
+		     /* work : */ (Rcomplex *) R_alloc(2*(size_t)n, sizeof(Rcomplex)),
+		     /* rwork: */ (double *)   R_alloc(n, sizeof(double)),
+		     &info FCONE FCONE FCONE);
+    UNPROTECT(1);
     if (info) error(_("error [%d] from Lapack 'ztrcon()'"), info);
     return val;
 #else
@@ -535,18 +653,20 @@ static SEXP La_ztrcon(SEXP A, SEXP norm)
 #endif
 }
 
+
 /* Complex case of solve.default: see the comments in La_solve */
-static SEXP La_solve_cmplx(SEXP A, SEXP Bin)
+static SEXP La_solve_cmplx(SEXP A, SEXP Bin, SEXP tolin)
 {
 #ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
     int n, p, info, *ipiv, *Adims, *Bdims;
     Rcomplex *avals;
     SEXP B, Adn, Bdn;
 
-    if (!isMatrix(A)) error(_("'a' must be a complex matrix"));
+    if (!isMatrix(A)) error(_("'%s' must be a complex matrix"), "a");
     Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     n = Adims[0];
     if(n == 0) error(_("'a' is 0-diml"));
+    size_t nl = n;
     int n2 = Adims[1];
     if(n2 != n) error(_("'a' (%d x %d) must be square"), n, n2);
     Adn = getAttrib(A, R_DimNamesSymbol);
@@ -577,7 +697,7 @@ static SEXP La_solve_cmplx(SEXP A, SEXP Bin)
 	if (!isNull(Adn)) setAttrib(B, R_NamesSymbol, VECTOR_ELT(Adn, 1));
     }
     Bin = PROTECT(coerceVector(Bin, CPLXSXP));
-    Memcpy(COMPLEX(B), COMPLEX(Bin), (size_t)n * p);
+    Memcpy(COMPLEX(B), COMPLEX(Bin), nl * p);
 
     ipiv = (int *) R_alloc(n, sizeof(int));
 
@@ -586,8 +706,8 @@ static SEXP La_solve_cmplx(SEXP A, SEXP Bin)
 	A = coerceVector(A, CPLXSXP);
 	avals = COMPLEX(A);
     } else {
-	avals = (Rcomplex *) R_alloc((size_t)n * n, sizeof(Rcomplex));
-	Memcpy(avals, COMPLEX(A), (size_t) n * n);
+	avals = (Rcomplex *) R_alloc(nl * nl, sizeof(Rcomplex));
+	Memcpy(avals, COMPLEX(A), nl * nl);
     }
     PROTECT(A);
     F77_CALL(zgesv)(&n, &p, avals, &n, ipiv, COMPLEX(B), &n, &info);
@@ -595,7 +715,25 @@ static SEXP La_solve_cmplx(SEXP A, SEXP Bin)
 	error(_("argument %d of Lapack routine %s had invalid value"),
 	      -info, "zgesv");
     if (info > 0)
-	error(_("Lapack routine zgesv: system is exactly singular"));
+	error(_("Lapack routine %s: system is exactly singular: U[%d,%d] = 0"),
+	      "zgesv", info, info);
+    int OK = 1;
+    for (size_t i = 0; i < nl*nl; i++)
+	if (!isfinite(avals[i].r) || !isfinite(avals[i].i)) {OK = 0; break;}
+    double tol = asReal(tolin);
+    if(OK == 1 && tol > 0) {
+	char one[2] = "1";
+	double anorm = F77_CALL(zlange)(one, &n, &n, COMPLEX(A), &n,
+					(double*) NULL FCONE);
+	Rcomplex *work = (Rcomplex *) R_alloc(2*nl, sizeof(Rcomplex));
+	double *rwork = (double *) R_alloc(2*nl, sizeof(double));
+	double rcond;
+	F77_CALL(zgecon)(one, &n, avals, &n, &anorm, &rcond, work, rwork,
+			 &info FCONE);
+	if (rcond < tol)
+	    error(_("system is computationally singular: reciprocal condition number = %g"),
+		  rcond);
+    }
     UNPROTECT(3);  /* B, Bin, A */
     return B;
 #else
@@ -613,7 +751,7 @@ static SEXP La_qr_cmplx(SEXP Ain)
     double *rwork;
 
     if (!(isMatrix(Ain) && isComplex(Ain)))
-	error(_("'a' must be a complex matrix"));
+	error(_("'%s' must be a complex matrix"), "a");
     SEXP Adn = getAttrib(Ain, R_DimNamesSymbol);
     Adims = INTEGER(coerceVector(getAttrib(Ain, R_DimSymbol), INTSXP));
     m = Adims[0]; n = Adims[1];
@@ -673,7 +811,7 @@ static SEXP qr_coef_cmplx(SEXP Q, SEXP Bin)
     Rcomplex *work, tmp;
 
     k = LENGTH(tau);
-    if (!isMatrix(Bin)) error(_("'b' must be a complex matrix"));
+    if (!isMatrix(Bin)) error(_("'%s' must be a complex matrix"), "b");
 
     if (!isComplex(Bin)) B = PROTECT(coerceVector(Bin, CPLXSXP));
     else B = PROTECT(duplicate(Bin));
@@ -719,7 +857,7 @@ static SEXP qr_qy_cmplx(SEXP Q, SEXP Bin, SEXP trans)
 
     k = LENGTH(tau);
     if (!(isMatrix(Bin) && isComplex(Bin)))
-	error(_("'b' must be a complex matrix"));
+	error(_("'%s' must be a complex matrix"), "b");
     tr = asLogical(trans);
     if(tr == NA_LOGICAL) error(_("invalid '%s' argument"), "trans");
 
@@ -754,8 +892,7 @@ static SEXP qr_qy_cmplx(SEXP Q, SEXP Bin, SEXP trans)
 static SEXP La_svd_cmplx(SEXP jobu, SEXP x, SEXP s, SEXP u, SEXP v)
 {
 #ifdef HAVE_FORTRAN_DOUBLE_COMPLEX
-    if (!(isString(jobu)))
-	error(_("'jobu' must be a character string"));
+    if (!isString(jobu)) error(_("'%s' must be a character string"), "jobu");
     int *xdims = INTEGER(coerceVector(getAttrib(x, R_DimSymbol), INTSXP));
     int n = xdims[0], p = xdims[1];
     const char *jz = CHAR(STRING_ELT(jobu, 0));
@@ -940,7 +1077,7 @@ static SEXP La_rg_cmplx(SEXP x, SEXP only_values)
 
 static SEXP La_chol(SEXP A, SEXP pivot, SEXP stol)
 {
-    if (!isMatrix(A)) error(_("'a' must be a numeric matrix"));
+    if (!isMatrix(A)) error(_("'%s' must be a numeric matrix"), "a");
 
     SEXP ans = PROTECT(isReal(A) ? duplicate(A): coerceVector(A, REALSXP));
     SEXP adims = getAttrib(A, R_DimSymbol);
@@ -960,7 +1097,7 @@ static SEXP La_chol(SEXP A, SEXP pivot, SEXP stol)
 	F77_CALL(dpotrf)("U", &m, REAL(ans), &m, &info FCONE);
 	if (info != 0) {
 	    if (info > 0)
-		error(_("the leading minor of order %d is not positive definite"),
+		error(_("the leading minor of order %d is not positive"),
 		      info);
 	    error(_("argument %d of Lapack routine %s had invalid value"),
 		  -info, "dpotrf");
@@ -975,7 +1112,7 @@ static SEXP La_chol(SEXP A, SEXP pivot, SEXP stol)
 			 FCONE);
 	if (info != 0) {
 	    if (info > 0)
-		warning(_("the matrix is either rank-deficient or indefinite"));
+		warning(_("the matrix is either rank-deficient or not positive definite"));
 	    else
 		error(_("argument %d of Lapack routine %s had invalid value"),
 		      -info, "dpstrf");
@@ -1016,7 +1153,7 @@ static SEXP La_chol2inv(SEXP A, SEXP size)
 	    if (TYPEOF(adims) != INTSXP) error("non-integer dims");
 	    Amat = PROTECT(coerceVector(A, REALSXP)); nprot++;
 	    m = INTEGER(adims)[0]; n = INTEGER(adims)[1];
-	} else error(_("'a' must be a numeric matrix"));
+	} else error(_("'%s' must be a numeric matrix"), "a");
 
 	if (sz > n) { UNPROTECT(nprot); error(_("'size' cannot exceed ncol(x) = %d"), n); }
 	if (sz > m) { UNPROTECT(nprot); error(_("'size' cannot exceed nrow(x) = %d"), m); }
@@ -1050,15 +1187,16 @@ static SEXP La_chol2inv(SEXP A, SEXP size)
 static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
 {
     int n, p;
-    double *avals, anorm, rcond, tol = asReal(tolin), *work;
+    double *avals,  tol = asReal(tolin);
     SEXP B, Adn, Bdn;
 
     if (!(isMatrix(A) &&
 	  (TYPEOF(A) == REALSXP || TYPEOF(A) == INTSXP || TYPEOF(A) == LGLSXP)))
-	error(_("'a' must be a numeric matrix"));
+	error(_("'%s' must be a numeric matrix"), "a");
     int *Adims = INTEGER(coerceVector(getAttrib(A, R_DimSymbol), INTSXP));
     n = Adims[0];
     if(n == 0) error(_("'a' is 0-diml"));
+    size_t nl = n;
     int n2 = Adims[1];
     if(n2 != n) error(_("'a' (%d x %d) must be square"), n, n2);
     Adn = getAttrib(A, R_DimNamesSymbol);
@@ -1091,7 +1229,7 @@ static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
 	if (!isNull(Adn)) setAttrib(B, R_NamesSymbol, VECTOR_ELT(Adn, 1));
     }
     PROTECT(Bin = coerceVector(Bin, REALSXP));
-    Memcpy(REAL(B), REAL(Bin), (size_t)n * p);
+    Memcpy(REAL(B), REAL(Bin), nl * p);
 
     int *ipiv = (int *) R_alloc(n, sizeof(int));
 
@@ -1100,8 +1238,8 @@ static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
 	A = coerceVector(A, REALSXP);
 	avals = REAL(A);
     } else {
-	avals = (double *) R_alloc((size_t)n * n, sizeof(double));
-	Memcpy(avals, REAL(A), (size_t)n * n);
+	avals = (double *) R_alloc(nl * nl, sizeof(double));
+	Memcpy(avals, REAL(A), nl * nl);
     }
     PROTECT(A);
     int info;
@@ -1112,11 +1250,16 @@ static SEXP La_solve(SEXP A, SEXP Bin, SEXP tolin)
     if (info > 0)
 	error(_("Lapack routine %s: system is exactly singular: U[%d,%d] = 0"),
 	      "dgesv", info, info);
-    if(tol > 0) {
+    // LAPACK 3.11.0 fails here if A contains NaNs
+    int OK = 1;
+    for (size_t i = 0; i < nl*nl; i++)
+	if (!isfinite(avals[i])) {OK = 0; break;}
+    if(OK == 1 && tol > 0) {
 	char one[2] = "1";
-	anorm = F77_CALL(dlange)(one, &n, &n, REAL(A), &n,
-				 (double*) NULL FCONE);
-	work = (double *) R_alloc(4*(size_t)n, sizeof(double));
+	double rcond;
+	double anorm = F77_CALL(dlange)(one, &n, &n, REAL(A), &n,
+					(double*) NULL FCONE);
+	double *work = (double *) R_alloc(4*nl, sizeof(double));
 	F77_CALL(dgecon)(one, &n, avals, &n, &anorm, &rcond, work, ipiv,
 			 &info FCONE);
 	if (rcond < tol)
@@ -1132,7 +1275,7 @@ static SEXP La_qr(SEXP Ain)
 {
     int m, n;
 
-    if (!isMatrix(Ain)) error(_("'a' must be a numeric matrix"));
+    if (!isMatrix(Ain)) error(_("'%s' must be a numeric matrix"), "a");
     SEXP Adn = getAttrib(Ain, R_DimNamesSymbol);
     int *Adims = INTEGER(coerceVector(getAttrib(Ain, R_DimSymbol), INTSXP));
     m = Adims[0]; n = Adims[1];
@@ -1187,7 +1330,7 @@ static SEXP La_qr(SEXP Ain)
 /* Real case of qr.coef */
 static SEXP qr_coef_real(SEXP Q, SEXP Bin)
 {
-    if (!isMatrix(Bin)) error(_("'b' must be a numeric matrix"));
+    if (!isMatrix(Bin)) error(_("'%s' must be a numeric matrix"), "b");
 
     SEXP B = PROTECT(isReal(Bin) ? duplicate(Bin) : coerceVector(Bin, REALSXP)),
 	qr  = VECTOR_ELT(Q, 0), // qr$qr
@@ -1229,7 +1372,7 @@ static SEXP qr_qy_real(SEXP Q, SEXP Bin, SEXP trans)
     double *work, tmp;
 
     k = LENGTH(tau);
-    if (!isMatrix(Bin)) error(_("'b' must be a numeric matrix"));
+    if (!isMatrix(Bin)) error(_("'%s' must be a numeric matrix"), "b");
     tr = asLogical(trans);
     if(tr == NA_LOGICAL) error(_("invalid '%s' argument"), "trans");
 
@@ -1262,7 +1405,7 @@ static SEXP det_ge_real(SEXP Ain, SEXP logarithm)
     int info, sign = 1, useLog = asLogical(logarithm);
     double modulus = 0.0; /* -Wall */
 
-    if (!isMatrix(Ain)) error(_("'a' must be a numeric matrix"));
+    if (!isMatrix(Ain)) error(_("'%s' must be a numeric matrix"), "a");
     if (useLog == NA_LOGICAL) error(_("argument 'logarithm' must be logical"));
     SEXP A = PROTECT(isReal(Ain) ? duplicate(Ain): coerceVector(Ain, REALSXP));
     int *Adims = INTEGER(coerceVector(getAttrib(Ain, R_DimSymbol), INTSXP));
@@ -1328,11 +1471,14 @@ static SEXP mod_do_lapack(SEXP call, SEXP op, SEXP args, SEXP env)
     case 5: ans = La_rs(CAR(args), CADR(args)); break;
     case 51: ans = La_rs_cmplx(CAR(args), CADR(args)); break;
     case 6: ans = La_dlange(CAR(args), CADR(args)); break;
+    case 61: ans = La_zlange(CAR(args), CADR(args)); break;
     case 7: ans = La_dgecon(CAR(args), CADR(args)); break;
     case 8: ans = La_dtrcon(CAR(args), CADR(args)); break;
+    case 81: ans = La_dtrcon3(CAR(args), CADR(args), CADDR(args)); break;
     case 9: ans = La_zgecon(CAR(args), CADR(args)); break;
     case 10: ans = La_ztrcon(CAR(args), CADR(args)); break;
-    case 11: ans = La_solve_cmplx(CAR(args), CADR(args)); break;
+    case 13: ans = La_ztrcon3(CAR(args), CADR(args), CADDR(args)); break;
+    case 11: ans = La_solve_cmplx(CAR(args), CADR(args), CADDR(args)); break;
 
     case 100: ans = La_solve(CAR(args), CADR(args), CADDR(args)); break;
     case 101: ans = La_qr(CAR(args)); break;
@@ -1366,7 +1512,7 @@ static SEXP mod_do_lapack(SEXP call, SEXP op, SEXP args, SEXP env)
 	ans = La_svd_cmplx(a1, a2, a3, a4, CAR(args));
 	break;
     }
-    case 1000:
+    case 1000: // La_version()
     {
 	int major, minor, patch;
 	char str[20];
@@ -1375,7 +1521,7 @@ static SEXP mod_do_lapack(SEXP call, SEXP op, SEXP args, SEXP env)
 	ans = mkString(str);
 	break;
     }
-    case 1001:
+    case 1001: // La_library()
     {
 #if defined(HAVE_DLADDR) && defined(HAVE_REALPATH)
 	Dl_info dl_info;
@@ -1383,14 +1529,31 @@ static SEXP mod_do_lapack(SEXP call, SEXP op, SEXP args, SEXP env)
 	   pointer, which is not allowed by ISO C, but there is no compliant
 	   alternative to using dladdr() */
 	// dladdr has first arg void * on Solaris.  This is not POSIX.
+#ifdef __clang__
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wpedantic"
+#elif defined __GNUC__
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wpedantic"	
+#endif
 	if (dladdr((void *) F77_NAME(ilaver), &dl_info)) {
 	    char buf[PATH_MAX+1];
 	    char *res = realpath(dl_info.dli_fname, buf);
 	    if (res) {
-		ans = mkString(res);
+		SEXP nfo = R_NilValue;
+		if (strstr(res, "flexiblas"))
+		    nfo = R_flexiblas_info();
+		if (isNull(nfo))
+		    nfo = mkChar(res);
+		ans = ScalarString(nfo);
 		break;
 	    }
 	}
+#ifdef __clang__
+# pragma clang diagnostic pop
+#elif defined __GNUC__
+# pragma GCC diagnostic pop
+#endif
 #endif
 	ans = mkString(""); /* LAPACK library not known */
 	break;
@@ -1411,7 +1574,7 @@ void
 R_init_lapack(DllInfo *info)
 {
     R_LapackRoutines *tmp;
-    tmp = Calloc(1, R_LapackRoutines);
+    tmp = R_Calloc(1, R_LapackRoutines);
 
     tmp->do_lapack = mod_do_lapack;
     R_setLapackRoutines(tmp);
