@@ -29,7 +29,7 @@ double dlogis(double x, double location, double scale, int give_log)
 	return x + location + scale;
 #endif
     if (scale <= 0.0)
-	ML_ERR_return_NAN;
+	ML_WARN_return_NAN;
 
     x = fabs((x - location) / scale);
     e = exp(-x);
