@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998--2018  The R Core Team
+ *  Copyright (C) 1998--2022  The R Core Team
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
  *  based on code (C) 1979 and later Royal Statistical Society
  *
@@ -33,8 +33,8 @@
 /**----------- DEBUGGING -------------
  *
  *	make CFLAGS='-DDEBUG_qbeta  ...'
- *MM (w/ Debug, w/o Optimization):
- (cd `R-devel RHOME`/src/nmath ; gcc -I. -I../../src/include -I../../../R/src/include  -DHAVE_CONFIG_H -fopenmp -g -pedantic -Wall --std=gnu99 -DDEBUG_q -DDEBUG_qbeta -Wcast-align -Wclobbered  -c ../../../R/src/nmath/qbeta.c -o qbeta.o; cd ../..; make R)
+ *MM (w/ Debug, w/o Optimization): if not manually adding   -dDEBUG_qbeta  , use
+ (cd `R-devel-qbeta-dbg RHOME`/src/nmath ; make qbeta.o; cd ../..; make R)
 */
 #ifdef DEBUG_qbeta
 # define R_ifDEBUG_printf(...) REprintf(__VA_ARGS__)
@@ -69,6 +69,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 
     double qbet[2];// = { qbeta(), 1 - qbeta() }
     qbeta_raw(alpha, p, q, lower_tail, log_p,
+	      // swap_01 ,  log_q_cut ,      n_N
 	      MLOGICAL_NA, USE_LOG_X_CUTOFF, n_NEWTON_FREE, qbet);
     return qbet[0];
 }
@@ -81,7 +82,7 @@ static const double
 // Too extreme: inaccuracy in pbeta(); e.g for  qbeta(0.95, 1e-9, 20):
 // -> in pbeta() --> bgrat(..... b*z == 0 underflow, hence inaccurate pbeta()
     /* DBL_very_MIN  = 0x0.0000001p-1022, // = 2^-1050 = 2^(-1022 - 28) */
-    /* DBL_log_v_MIN = -1050. * M_LN2, // = log(DBL_very_MIN) */
+    /* DBL_log_v_MIN = -1050. * M_LN2, // = log(DBL_very_MIN) ~= -727.8045 */
 // the most extreme -- not ok, as pbeta() then behaves strangely,
 // e.g., for  qbeta(0.95, 1e-8, 20):
     /* DBL_very_MIN  = 0x0.0000000000001p-1022, // = 2^-1074 = 2^(-1022 -52) */
@@ -189,9 +190,19 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
     logbeta = lbeta(p, q);
 
     swap_tail = (swap_choose) ? (p_ > 0.5) : swap_01;
+
+    R_ifDEBUG_printf(
+	"qbeta(%g, %g, %g, lower_t=%d, log_p=%d, swap_01=%d, log_q_cut=%g, n_N=%d):%s\n"
+	"  swap_tail=%s :",
+	alpha, p,q, lower_tail, log_p, swap_01, log_q_cut, n_N,
+	(log_p && (p_ == 0. || p_ == 1.)) ? (p_==0.?" p_=0":" p_=1") : "",
+	(swap_tail ? "TRUE": "F"));
+
+    int n_maybe_swaps = 0;
+maybe_swap:
     // change tail; default (swap_01 = NA): afterwards 0 < a <= 1/2
-    if(swap_tail) { /* change tail, swap  p <-> q :*/
-	a = R_DT_CIv(alpha); // = 1 - p_ < 1/2
+    if(swap_tail) { /* change tail, swap copies of {p,q}:  p <-> q :*/
+	a = R_DT_CIv(alpha); // = 1 - p_ , is < 1/2 if(swap_choose)
 	/* la := log(a), but without numerical cancellation: */
 	la = R_DT_Clog(alpha);
 	pp = q; qq = p;
@@ -201,6 +212,7 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 	la = R_DT_log(alpha);
 	pp = p; qq = q;
     }
+    n_maybe_swaps++;
 
     /* calculate the initial approximation */
 
@@ -215,37 +227,37 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
      */
     double acu = fmax2(acu_min, pow(10., -13. - 2.5/(pp * pp) - 0.5/(a * a)));
     // try to catch  "extreme left tail" early
-    double tx, u0 = (la + log(pp) + logbeta) / pp; // = log(x_0)
+    double tx,
+	u0 = (la + log(pp) + logbeta) / pp, // = log(x_0)
+	rp = pp*(1.-qq)/(pp+1.);
     static const double
 	log_eps_c = M_LN2 * (1. - DBL_MANT_DIG);// = log(DBL_EPSILON) = -36.04..
-    r = pp*(1.-qq)/(pp+1.);
 
     t = 0.2;
     // FIXME: Factor 0.2 is a bit arbitrary;  '1' is clearly much too much.
-
+    Rboolean u0_maybe = (M_LN2 * DBL_MIN_EXP < u0 && u0 < -0.01);
+    /* 1. cannot allow exp(u0) = 0 ==> exp(u1) = exp(u0) = 0
+     * 2. must: u0 < 0, but too close to 0 <==> x = exp(u0) = 0.99.. */
     R_ifDEBUG_printf(
-	"qbeta(%g, %g, %g, lower_t=%d, log_p=%d):%s\n"
-	"  swap_tail=%d, la=%#8g, u0=%#8g (bnd: %g (%g)) ",
-	alpha, p,q, lower_tail, log_p,
-	(log_p && (p_ == 0. || p_ == 1.)) ? (p_==0.?" p_=0":" p_=1") : "",
-	swap_tail, la, u0,
-	(t*log_eps_c - log(fabs(pp*(1.-qq)*(2.-qq)/(2.*(pp+2.)))))/2.,
-	 t*log_eps_c - log(fabs(r))
+	"  n_maybe_swaps=%d, la=%#8g, u0=%#8g -> u0_maybe=%s (bnd: %g (%g)); ",
+	n_maybe_swaps, la, u0, (u0_maybe ? "TRUE" : "F"),
+	u0_maybe ? (t*log_eps_c - log(fabs(pp*(1.-qq)*(2.-qq)/(2.*(pp+2.)))))/2. : -0.,
+	u0_maybe ?  t*log_eps_c - log(fabs(rp)) : -0.
 	);
 
-    if(M_LN2 * DBL_MIN_EXP < u0 && // cannot allow exp(u0) = 0 ==> exp(u1) = exp(u0) = 0
-       u0 < -0.01 && // (must: u0 < 0, but too close to 0 <==> x = exp(u0) = 0.99..)
+    double u_n = 1.; // to be  log(xinbta) <==> xinbta = exp(u_n).  1 is impossible
+    if(u0_maybe &&
        // qq <= 2 && // <--- "arbitrary"
-       // u0 <  t*log_eps_c - log(fabs(r)) &&
+       // u0 <  t*log_eps_c - log(fabs(rp)) &&
        u0 < (t*log_eps_c - log(fabs(pp*(1.-qq)*(2.-qq)/(2.*(pp+2.)))))/2.)
     {
 // TODO: maybe jump here from below, when initial u "fails" ?
 // L_tail_u:
 	// MM's one-step correction (cheaper than 1 Newton!)
-	r = r*exp(u0);// = r*x0
-	if(r > -1.) {
-	    u = u0 - log1p(r)/pp;
-	    R_ifDEBUG_printf("u1-u0=%9.3g --> choosing u = u1\n", u-u0);
+	rp = rp*exp(u0);// = rp*x0
+	if(rp > -1.) {
+	    u = u0 - log1p(rp)/pp;
+	    R_ifDEBUG_printf("u1-u0=%9.3g --> choosing u = u1 = %g\n", u-u0, u);
 	} else {
 	    u = u0;
 	    R_ifDEBUG_printf("cannot cheaply improve u0\n");
@@ -258,14 +270,14 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
     // y := y_\alpha in AS 64 := Hastings(1955) approximation of qnorm(1 - a) :
     r = sqrt(-2 * la);
     y = r - (const1 + const2 * r) / (1. + (const3 + const4 * r) * r);
-
+    R_ifDEBUG_printf("y_a(AS64)=%g\n  ", y);
     if (pp > 1 && qq > 1) { // use  Carter(1947), see AS 109, remark '5.'
 	r = (y * y - 3.) / 6.;
 	s = 1. / (pp + pp - 1.);
 	t = 1. / (qq + qq - 1.);
 	h = 2. / (s + t);
 	w = y * sqrt(h + r) / h - (t - s) * (r + 5. / 6. - 2. / (3. * h));
-	R_ifDEBUG_printf("p,q > 1 => w=%g", w);
+	R_ifDEBUG_printf("p,q > 1 => w=%g <= 300 ? ", w);
 	if(w > 300) { // exp(w+w) is huge or overflows
 	    t = w+w + log(qq) - log(pp); // = argument of log1pexp(.)
 	    u = // log(xinbta) = - log1p(qq/pp * exp(w+w)) = -log(1 + exp(t))
@@ -280,33 +292,44 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 	r = qq + qq;
 	/* A slightly more stable version of  t := \chi^2_{alpha} of AS 64
 	 * t = 1. / (9. * qq); t = r * R_pow_di(1. - t + y * sqrt(t), 3);  */
-	t = 1. / (3. * sqrt(qq));
+	t = 1. / (3. * sqrt(qq)); // = sqrt(t) of formula above
 	t = r * R_pow_di(1. + t*(-t + y), 3);// = \chi^2_{alpha} of AS 64
-	s = 4. * pp + r - 2.;// 4p + 2q - 2 = numerator of new t = (...) / chi^2
-	R_ifDEBUG_printf("min(p,q) <= 1: t=%g", t);
+	s = 4. * pp + r - 2.;// 4p + 2q - 2 = numerator of new t' = s / t = s / chi^2
+	R_ifDEBUG_printf("min(p,q) <= 1: t=%g, s=%g", t, s);
 	if (t == 0 || (t < 0. && s >= t)) { // cannot use chisq approx
 	    // x0 = 1 - { (1-a)*q*B(p,q) } ^{1/q}    {AS 65}
 	    // xinbta = 1. - exp((log(1-a)+ log(qq) + logbeta) / qq);
-	    double l1ma;/* := log(1-a), directly from alpha (as 'la' above):
-			 * FIXME: not worth it? log1p(-a) always the same ?? */
-	    if(swap_tail)
-		l1ma = R_DT_log(alpha);
-	    else
-		l1ma = R_DT_Clog(alpha);
-	    R_ifDEBUG_printf(" t <= 0 : log1p(-a)=%.15g, better l1ma=%.15g\n", log1p(-a), l1ma);
+	    double l1ma = /* := log(1-a), directly from alpha (as 'la' above);
+			   though only seen very small improvements */
+		swap_tail ? R_DT_log(alpha) : R_DT_Clog(alpha);
+
+	    R_ifDEBUG_printf(
+		" t <= 0  => AS65: log1p(-a)=%7g, better l1ma=%.15g, relD=%g\n",
+		log1p(-a), l1ma, log1p(-a)/l1ma - 1);
+
 	    double xx = (l1ma + log(qq) + logbeta) / qq;
+	    R_ifDEBUG_printf("  xx = (l1ma + log(qq) + logbeta) / qq = %.10g; ", xx);
 	    if(xx <= 0.) {
 		xinbta = -expm1(xx);
 		u = R_Log1_Exp (xx);// =  log(xinbta) = log(1 - exp(...A...))
+		R_ifDEBUG_printf(" xx <= 0 useful ==> u, xinbta\n");
+
 	    } else { // xx > 0 ==> 1 - e^xx < 0 .. is nonsense
-		R_ifDEBUG_printf(" xx=%g > 0: xinbta:= 1-e^xx < 0\n", xx);
-		xinbta = 0; u = ML_NEGINF; /// FIXME can do better?
+		R_ifDEBUG_printf(" xx > 0: xinbta:= 1-e^xx < 0 'nonsense'; ");
+		// Try MM's one-step correction (or else u0)
+		double r_ = rp*exp(u0);
+		if(r_ > -1.) {
+		    u = u0 - log1p(r_)/pp; R_ifDEBUG_printf("u:= u0 - log1p(r_)/pp = %g\n");
+		} else {
+		    u = u0;                R_ifDEBUG_printf("u:= u0 = %g\n");
+		}
+		xinbta = exp(u);
 	    }
 	} else {
 	    t = s / t;
-	    R_ifDEBUG_printf(" t > 0 or s < t < 0:  new t = %g ( > 1 ?)\n", t);
+	    R_ifDEBUG_printf(" t > 0 or s < t < 0:  new t := s/t = %g ( > 1 ?)\n", t);
 	    if (t <= 1.) { // cannot use chisq, either
-		u = (la + log(pp) + logbeta) / pp;
+		u = u0;
 		xinbta = exp(u);
 	    } else { // (1+x0)/(1-x0) = t,  solved for x0 :
 		xinbta = 1. - 2. / (t + 1.);
@@ -316,25 +339,29 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
     }
 
     // Problem: If initial u is completely wrong, we make a wrong decision here
-    if(swap_choose &&
+    if(swap_choose &&  //   vvvv/ why -exp(*)? u  on log-x scale! Swapping can be very good, but needs smart if(..)
        (( swap_tail && u >= -exp(  log_q_cut)) || // ==> "swap back"
 	(!swap_tail && u >= -exp(4*log_q_cut) && pp / qq < 1000.) // ==> "swap now"
 	   )) {
-	// "revert swap" -- and use_log_x
+	if(swap_tail)
+	    R_ifDEBUG_printf("\"swap back\" as u = %g >= -exp(log_q_cut);", u);
+	else
+	    R_ifDEBUG_printf("\"swap now\" as u = %g >= -exp(4*log_q_cut) && pp/qq = %g < 1000;",
+			     u, pp/qq);
+	// reverse swap (and typically use_log_x)
 	swap_tail = !swap_tail;
-	R_ifDEBUG_printf(" u = %g (e^u = xinbta = %.16g) ==> ", u, xinbta);
+
 	if(swap_tail) { // "swap now" (much less easily)
-	    a = R_DT_CIv(alpha); // needed ?
+	    a  = R_DT_CIv(alpha); // needed ?
 	    la = R_DT_Clog(alpha);
 	    pp = q; qq = p;
 	}
-	else { // swap back :
+	else { // "swap back" :
 	    a = p_;
 	    la = R_DT_log(alpha);
 	    pp = p; qq = q;
 	}
-	R_ifDEBUG_printf("\"%s\"; la = %g\n",
-			 (swap_tail ? "swap now" : "swap back"), la);
+	R_ifDEBUG_printf(" ==> la = %g\n", la);
 	// we could redo computations above, but this should be stable
 	u = R_Log1_Exp(u);
 	xinbta = exp(u);
@@ -344,7 +371,7 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
    2) The correction step can go outside (u_n > 0 ==>  e^u > 1 is illegal)
    e.g., for  qbeta(0.2066, 0.143891, 0.05)
 */
-    } else R_ifDEBUG_printf("\n");
+    }
 
     if(!use_log_x)
 	use_log_x = (u < log_q_cut);// <==> xinbta = e^u < exp(log_q_cut)
@@ -357,7 +384,6 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 		     ((bad_init && !bad_u) ? ", ** bad_init **" : ""),
 		     (use_log_x ? ", on u = LOG(x) SCALE" : ""));
 
-    double u_n = 1.; // -Wall
     tx = xinbta; // keeping "original initial x" (for now)
 
     if(bad_u || u < log_q_cut) {
@@ -368,7 +394,7 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 	w = pbeta_raw(DBL_very_MIN, pp, qq, TRUE, log_p);
 	if(w > (log_p ? la : a)) {
 	    R_ifDEBUG_printf(
-		" quantile is left of %g; \"convergence\"\n", DBL_very_MIN);
+		" quantile is left of %g: boundary \"convergence\"\n", DBL_very_MIN);
 	    if(log_p || fabs(w - a) < fabs(0 - a)) { // DBL_very_MIN is better than 0
 		tx   = DBL_very_MIN;
 		u_n  = DBL_log_v_MIN;// = log(DBL_very_MIN)
@@ -379,8 +405,9 @@ qbeta_raw(double alpha, double p, double q, int lower_tail, int log_p,
 	    use_log_x = log_p; add_N_step = FALSE; goto L_return;
 	}
 	else {
-	    R_ifDEBUG_printf(" pbeta(%g, *) = %g <= %g (= %s) --> continuing\n",
-			     DBL_log_v_MIN, w, (log_p ? la : a), (log_p ? "la" : "a"));
+	    R_ifDEBUG_printf(" pbeta(%g, %g, %g, T, log) = %g <= %g (= %s) --> continuing\n",
+			     DBL_very_MIN, pp,qq, w,
+			     (log_p ? la : a), (log_p ? "la" : "a"));
 	    if(u  < DBL_log_v_MIN) {
 		u = DBL_log_v_MIN;// = log(DBL_very_MIN)
 		xinbta = DBL_very_MIN;
@@ -435,14 +462,23 @@ L_Newton:
 	    */
 	    w = (y == ML_NEGINF) // y = -Inf  well possible: we are on log scale!
 		? 0. : (y - la) * exp(y - u + logbeta + r * u + t * R_Log1_Exp(u));
-	    if(!R_FINITE(w))
-		break;
+	    R_ifDEBUG_printf("N(i=%2d): u=%#20.16g, lnpb(e^u)=%#15.11g, w=%#12.7g",
+			     i_pb, u, y, w);
+	    if(!R_FINITE(w)) {
+		// what we should do is ==> go back, get better starting value
+		R_ifDEBUG_printf(" -- not finite --> %s\n",
+				 (n_maybe_swaps <= 1) ? "goto maybe_swap" : "give up");
+		if(n_maybe_swaps <= 1)
+		    goto maybe_swap;
+		/* else  was 'break;' ...
+		   but rather give up returning NaN directly as in "normal scale" Newton */
+		ML_WARNING(ME_DOMAIN, "");
+		qb[0] = qb[1] = ML_NAN; return;
+	    }
 	    if (i_pb >= n_N && w * wprev <= 0.)
 		prev = fmax2(fabs(adj),fpu);
-	    R_ifDEBUG_printf(
-		"N(i=%2d): u=%#20.16g, pb(e^u)=%#15.9g, w=%#15.9g, %s prev=%g,",
-		i_pb, u, y, w,
-		(i_pb >= n_N && w * wprev <= 0.) ? "new" : "old", prev);
+	    R_ifDEBUG_printf(", %s prev=%g,",
+			     (i_pb >= n_N && w * wprev <= 0.) ? "new" : "old", prev);
 	    g = 1;
 	    for (i_inn=0; i_inn < 1000; i_inn++) {
 		adj = g * w;
@@ -465,7 +501,7 @@ L_Newton:
 	    // (cancellation in (u_n -u) => may differ from adj:
 	    double D = fmin2(fabs(adj), fabs(u_n - u));
 	    /* R_ifDEBUG_printf(" delta(u)=%g\n", u_n - u); */
-	    R_ifDEBUG_printf(" it{in}=%d, delta(u)=%9.3g, D/|.|=%.3g\n",
+	    R_ifDEBUG_printf(" it{in}=%d, d.(u)=%6.3g, D/|.|=%.3g\n",
 			     i_inn, u_n - u, D/fabs(u_n + u));
 	    if (D <= 4e-16 * fabs(u_n + u))
 		goto L_converged;
@@ -479,16 +515,6 @@ L_Newton:
 	for (i_pb=0; i_pb < 1000; i_pb++) {
 	    y = pbeta_raw(xinbta, pp, qq, /*lower_tail = */ TRUE, log_p);
 	    // delta{y} :   d_y = y - (log_p ? la : a);
-#ifdef IEEE_754
-	    if(!R_FINITE(y) && !(log_p && y == ML_NEGINF))// y = -Inf  is ok if(log_p)
-#else
-	    if (errno)
-#endif
-		{ // ML_WARN_return_NAN :
-		    ML_WARNING(ME_DOMAIN, "");
-		    qb[0] = qb[1] = ML_NAN; return;
-		}
-
 
 	    /* w := Newton step size  (F(.) - a) / F'(.)  or,
 	     * --   log: (lF - la) / (F' / F) = exp(lF) * (lF - la) / F'
@@ -496,10 +522,25 @@ L_Newton:
 	    w = log_p
 		? (y - la) * exp(y + logbeta + r * log(xinbta) + t * log1p(-xinbta))
 		: (y - a)  * exp(    logbeta + r * log(xinbta) + t * log1p(-xinbta));
+	    if(!R_FINITE(w)) {
+		// what we should do is ==> go back, get better starting value
+		R_ifDEBUG_printf(
+		    "N(i=%2d): x0=%#19.15g, pb(x0)=%#15.11g, w=%#12.7g -- is not finite --> %s\n",
+		    i_pb, xinbta, y, w, (n_maybe_swaps <= 2) ? "goto maybe_swap" : "give up");
+		if(n_maybe_swaps <= 2) {
+		    if(!log_p && n_maybe_swaps == 2) use_log_x = TRUE; // try now
+		    if(!log_p || n_maybe_swaps <= 1)
+			goto maybe_swap ;
+		}
+		/* else  was 'break;' ...
+		   but rather give up returning NaN directly as in "normal scale" Newton */
+		ML_WARNING(ME_DOMAIN, "");
+		qb[0] = qb[1] = ML_NAN; return;
+	    }
 	    if (i_pb >= n_N && w * wprev <= 0.)
 		prev = fmax2(fabs(adj),fpu);
 	    R_ifDEBUG_printf(
-		"N(i=%2d): x0=%#17.15g, pb(x0)=%#15.9g, w=%#15.9g, %s prev=%g,",
+		"N(i=%2d): x0=%#19.15g, pb(x0)=%#15.11g, w=%#12.7g, %s prev=%g,",
 		i_pb, xinbta, y, w,
 		(i_pb >= n_N && w * wprev <= 0.) ? "new" : "old", prev);
 	    g = 1;
@@ -536,7 +577,7 @@ L_Newton:
     ML_WARNING(ME_PRECISION, "qbeta");
 
 L_converged:
-    log_ = log_p || use_log_x; // only for printing
+    log_ = log_p || use_log_x;
     R_ifDEBUG_printf(" %s: Final delta(y) = %g%s\n",
 		     warned ? "_NO_ convergence" : "converged",
 		     y - (log_ ? la : a), (log_ ? " (log_)" : ""));
@@ -559,7 +600,8 @@ L_converged:
 		(log_ ? ", log_" : ""), fabs(y - (log_ ? la : a)));
     }
 L_return:
-    if(give_log_q) { // ==> use_log_x , too
+    // use  if (use_log_x) u_n else tx   {and u_n is on log scale}
+    if(give_log_q) { // {currently not used from R's qbeta()} ==> use_log_x , too
 	if(!use_log_x) // (see if claim above is true)
 	    MATHLIB_WARNING(
 		"qbeta() L_return, u_n=%g;  give_log_q=TRUE but use_log_x=FALSE -- please report!",
@@ -575,16 +617,22 @@ L_return:
 	    if(add_N_step) {
 		/* add one last Newton step on original x scale, e.g., for
 		   qbeta(2^-98, 0.125, 2^-96) */
-		xinbta = exp(u_n);
+		if(u_n != 1.) // u_n has been computed above
+		    xinbta = exp(u_n);
 		y = pbeta_raw(xinbta, pp, qq, /*lower_tail = */ TRUE, log_p);
 		w = log_p
 		    ? (y - la) * exp(y + logbeta + r * log(xinbta) + t * log1p(-xinbta))
 		    : (y - a)  * exp(    logbeta + r * log(xinbta) + t * log1p(-xinbta));
-		tx = xinbta - w;
-		R_ifDEBUG_printf(" Final Newton correction(non-log scale):\n"
-								   //   \n  xinbta=%.16g
-				 "  xinbta=%.16g, y=%g, w=-Delta(x)=%g. \n=> new x=%.16g\n",
-		    xinbta, y, w, tx);
+		if(R_FINITE(w)) {
+		    tx = xinbta - w;
+		    R_ifDEBUG_printf(" Final Newton correction(non-log scale):\n"
+				     "  xinbta=%.16g, y=%g, w=-Delta(x)=%g. \n=> new x=%.16g\n",
+				     xinbta, y, w, tx);
+		} else { // Newton step w  cannot be used
+		    R_ifDEBUG_printf(" Final Newton correction(non-log scale), canNOT use step 'w':\n"
+				     "  xinbta=%.16g, y=%g, w=-Delta(x)=%g\n=> keep x=xinbta\n", xinbta, y, w);
+		    tx = xinbta;
+		}
 	    } else {
 		if(swap_tail) {
 		    qb[0] = -expm1(u_n); qb[1] =  exp  (u_n);
