@@ -489,7 +489,7 @@ int Rf_initialize_R(int ac, char **av)
 	if(!ifp) R_Suicide(_("creating temporary file for '-e' failed"));
 	unlink(ifile);
 	res = fwrite(cmdlines, strlen(cmdlines)+1, 1, ifp);
-	if(res != 1) error("fwrite error in initialize_R");
+	if(res != 1) R_Suicide("fwrite error in initialize_R");
 	fflush(ifp);
 	rewind(ifp);
     }
